/* * C o p y r i g h t *(also read COPYING)* * * * * * * * * * * * * * * * * * *
 *                                                                             *
 *  Copyright (C) 2002  <Philip Van Hoof>                                      *
 *                                                                             *
 *  This program is free software; you can redistribute it and/or modify       *
 *  it under the terms of the GNU General Public License as published by       *
 *  the Free Software Foundation; either version 2 of the License, or          *
 *  (at your option) any later version.                                        *
 *                                                                             *
 *  This program is distributed in the hope that it will be useful,            *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *
 *  GNU General Public License for more details.                               *
 *                                                                             *
 *  You should have received a copy of the GNU General Public License          *
 *  along with this program; if not, write to the Free Software                *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
 *                                                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
#include <sys/time.h>
#include <time.h>
#include <string.h>
#include "mainwin.h"
#include "aboutwin.h"
#include "newitemwin.h"
#include "textitemwin.h"
#include "dataitemwin.h"
#include "gnomefilelist.h"
#include "prefswin.h"
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xmlIO.h>
#include <applet-widget.h>
#include "config.h"

static void mainwin_class_init(MainWinClass *klass);
static void mainwin_init(MainWin *mwin);
static void mainwin_destroy(GtkObject *object);

static GtkWindowClass *parent_class = NULL;

static gulong XA_COMPOUND_TEXT = 0;
static const struct {
  gulong      *atom;
  const gchar *atom_name;
} atoms[] = {
  { &XA_COMPOUND_TEXT,                  "COMPOUND_TEXT", },
};


//{ Utils
gchar* g_unichar_to_utf8 (guint16 c) {
  guint len = 0;    
  int first;
  int i;
  gchar *outbuf;
  outbuf = (gchar*) g_malloc0(sizeof(gchar)*8);

  if (c < 0x80)
    {
      first = 0;
      len = 1;
    }
  else if (c < 0x800)
    {
      first = 0xc0;
      len = 2;
    }
  else if (c < 0x10000)
    {
      first = 0xe0;
      len = 3;
    }
   else if (c < 0x200000)
    {
      first = 0xf0;
      len = 4;
    }
  else if (c < 0x4000000)
    {
      first = 0xf8;
      len = 5;
    }
  else
    {
      first = 0xfc;
      len = 6;
    }

	for (i = len - 1; i > 0; --i)
	{
		outbuf[i] = (c & 0x3f) | 0x80;
		c >>= 6;
	}
	outbuf[0] = c | first;
	outbuf[len+1] = '\0';

	return outbuf;
}


gchar* ucs2tostr(guchar *data, gint length) {
	guint16 *ucs2 = (guint16*) data;
	gint i;
	GString *str;

	str = g_string_new (NULL);
	i = 0;
	while (i < length / 2)
	{
		g_string_append (str, g_unichar_to_utf8(ucs2[i]));
		++i;
	}
	return str->str;
}

//}
//{ Menus and menu functions
/* If you change a menu, also change it in the popmainwinmenu*/
/* And set the datapointer in the mainwin_new function to mwin !*/
static GnomeUIInfo mainwinmenu_file_menu_uiinfo[] =
{ /* The mainmenus filemenu */
#define MAINWINMENU_FILE_NEWITEM 0
  GNOMEUIINFO_MENU_NEW_ITEM (N_("_New item..."), NULL, on_mainwinmenu_newitem_activate, NULL),
  {
#define MAINWINMENU_FILE_DELETEITEM 1
    GNOME_APP_UI_ITEM, N_("_Delete items"),
    NULL,
    (gpointer) on_mainwinmenu_deleteitem_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE,
    GDK_D, GDK_CONTROL_MASK, NULL
  },
#define MAINWINMENU_FILE_SAVEITEMSTODISK 2
  {
    GNOME_APP_UI_ITEM, N_("_Save selected items to disk..."),
    NULL,
    (gpointer) on_mainwinmenu_file_savetodisk_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE_AS,
    GDK_S, GDK_CONTROL_MASK, NULL
  },
#define MAINWINMENU_FILE_SAVEALLITEMSTODISK 3
  {
    GNOME_APP_UI_ITEM, N_("_Save all items to disk..."),
    NULL,
    (gpointer) on_mainwinmenu_file_savealltodisk_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE_AS,
    GDK_A, GDK_CONTROL_MASK, NULL
  },
 
#define MAINWINMENU_FILE_OPENITEMFROMDISK 4
  {
    GNOME_APP_UI_ITEM, N_("_Load items from disk..."),
    NULL,
    (gpointer) on_mainwinmenu_file_openfromdisk_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_OPEN,
    GDK_O, GDK_CONTROL_MASK, NULL
  },
#define MAINWINMENU_FILE_SEPARATOR 5
  GNOMEUIINFO_SEPARATOR,
#define MAINWINMENU_FILE_EXIT 6
  GNOMEUIINFO_MENU_EXIT_ITEM (on_mainwinmenu_exit_activate, NULL),
#define MAINWINMENU_FILE_END 7
  GNOMEUIINFO_END
};

static GnomeUIInfo mainwinmenu_edit_menu_uiinfo[] =
{ /* The mainmenus edit menu */
#define MAINWINMENU_EDIT_CUT 0
  GNOMEUIINFO_MENU_CUT_ITEM (on_mainwinmenu_cut_activate, NULL),
#define MAINWINMENU_EDIT_COPY 1
  GNOMEUIINFO_MENU_COPY_ITEM (on_mainwinmenu_copy_activate, NULL),
#define MAINWINMENU_EDIT_PASTE 2
  GNOMEUIINFO_MENU_PASTE_ITEM (on_mainwinmenu_paste_activate, NULL),
#define MAINWINMENU_EDIT_CLEAR 3
  GNOMEUIINFO_MENU_CLEAR_ITEM (on_mainwinmenu_clear_activate, NULL),
#define MAINWINMENU_EDIT_SEPARATOR 4
  GNOMEUIINFO_SEPARATOR,
#define MAINWINMENU_EDIT_SELECT_ALL 5
  GNOMEUIINFO_MENU_SELECT_ALL_ITEM (on_mainwinmenu_selectall_activate, NULL),
{
#define MAINWINMENU_EDIT_SELECT_NONE 6
    GNOME_APP_UI_ITEM, N_("_Select none"),
    NULL,
    (gpointer) on_mainwinmenu_selectnone_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_UNDO,
    0, (GdkModifierType) 0, NULL
  },
#define MAINWINMENU_EDIT_SEPARATORS 7
  GNOMEUIINFO_SEPARATOR,
#define MAINWINMENU_EDIT_APPLET 8
  {
    GNOME_APP_UI_ITEM, N_("_Applet"),
    NULL,
    (gpointer) on_mainwinmenu_applet_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOTTOM,
    0, (GdkModifierType) 0, NULL
  },	

#define MAINWINMENU_EDIT_END 9
  GNOMEUIINFO_END
};

static GnomeUIInfo mainwinmenu_settings_menu_uiinfo[] =
{ /*The mainmenus settings menu */
#define MAINWINMENU_SETTINGS_PREFERENCES 0
  GNOMEUIINFO_MENU_PREFERENCES_ITEM (on_mainwinmenu_preferences_activate, NULL),
#define MAINWINMENU_SETTINGS_END 1
  GNOMEUIINFO_END
};

static GnomeUIInfo mainwinmenu_help_menu_uiinfo[] =
{ /* The mainmenus helpmenu */
#define MAINWINMENU_HELP_ABOUT 0
  GNOMEUIINFO_MENU_ABOUT_ITEM (on_mainwinmenu_about_activate, NULL),
#define MAINWINMENU_HELP_END 1
  GNOMEUIINFO_END
};

static GnomeUIInfo mainwinmenu_uiinfo[] =
{ /* The mainmenu */
#define MAINWINMENU_FILE 0
  GNOMEUIINFO_MENU_FILE_TREE (mainwinmenu_file_menu_uiinfo),
#define MAINWINMENU_EDIT 1
  GNOMEUIINFO_MENU_EDIT_TREE (mainwinmenu_edit_menu_uiinfo),
#define MAINWINMENU_SETTINGS 2
  GNOMEUIINFO_MENU_SETTINGS_TREE (mainwinmenu_settings_menu_uiinfo),
#define MAINWINMENU_HELP 3
  GNOMEUIINFO_MENU_HELP_TREE (mainwinmenu_help_menu_uiinfo),
#define MAINWINMENU_END 4
  GNOMEUIINFO_END
};

static GnomeUIInfo popmainwinmenu_file_menu_uiinfo[] =
{ /* The filemenu of the popupmenu */
#define POPMAINWINMENU_FILE_NEWITEM 0
  GNOMEUIINFO_MENU_NEW_ITEM (N_("_New item..."), NULL, on_mainwinmenu_newitem_activate, NULL),
  {
#define POPMAINWINMENU_FILE_DELETEITEM 1
    GNOME_APP_UI_ITEM, N_("_Delete items"),
    NULL,
    (gpointer) on_mainwinmenu_deleteitem_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE,
    0, (GdkModifierType) 0, NULL
  },
#define POPMAINWINMENU_FILE_SAVEITEMSTODISK 2
  {
    GNOME_APP_UI_ITEM, N_("_Save selected items to disk..."),
    NULL,
    (gpointer) on_mainwinmenu_file_savetodisk_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE_AS,
    0, (GdkModifierType) 0, NULL
  },
#define POPMAINWINMENU_FILE_SAVEALLITEMSTODISK 3
  {
    GNOME_APP_UI_ITEM, N_("_Save all items to disk..."),
    NULL,
    (gpointer) on_mainwinmenu_file_savealltodisk_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE_AS,
    GDK_A, GDK_CONTROL_MASK, NULL
  },
 #define POPMAINWINMENU_FILE_OPENITEMFROMDISK 4
  {
    GNOME_APP_UI_ITEM, N_("_Load items from disk..."),
    NULL,
    (gpointer) on_mainwinmenu_file_openfromdisk_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_OPEN,
    0, (GdkModifierType) 0, NULL
  },

#define POPMAINWINMENU_FILE_SEPARATOR 5
  GNOMEUIINFO_SEPARATOR,
#define POPMAINWINMENU_FILE_EXIT 6
  GNOMEUIINFO_MENU_EXIT_ITEM (on_mainwinmenu_exit_activate, NULL),
#define POPMAINWINMENU_FILE_END 7
  GNOMEUIINFO_END
};

static GnomeUIInfo popmainwinmenu_edit_menu_uiinfo[] =
{ /* The edit menu of the popupmenu */
#define POPMAINWINMENU_EDIT_CUT 0
  GNOMEUIINFO_MENU_CUT_ITEM (on_mainwinmenu_cut_activate, NULL),
#define POPMAINWINMENU_EDIT_COPY 1
  GNOMEUIINFO_MENU_COPY_ITEM (on_mainwinmenu_copy_activate, NULL),
#define POPMAINWINMENU_EDIT_PASTE 2
  GNOMEUIINFO_MENU_PASTE_ITEM (on_mainwinmenu_paste_activate, NULL),
#define POPMAINWINMENU_EDIT_CLEAR 3
  GNOMEUIINFO_MENU_CLEAR_ITEM (on_mainwinmenu_clear_activate, NULL),
#define POPMAINWINMENU_EDIT_SEPARATOR 4
  GNOMEUIINFO_SEPARATOR,
#define POPMAINWINMENU_EDIT_SELECT_ALL 5
  GNOMEUIINFO_MENU_SELECT_ALL_ITEM (on_mainwinmenu_selectall_activate, NULL),
{
#define POPMAINWINMENU_EDIT_SELECT_NONE 6
    GNOME_APP_UI_ITEM, N_("_Select none"),
    NULL,
    (gpointer) on_mainwinmenu_selectnone_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_UNDO,
    0, (GdkModifierType) 0, NULL
  },	
  
#define POPMAINWINMENU_EDIT_SEPARATOR2 7
  GNOMEUIINFO_SEPARATOR,
#define POPMAINWINMENU_EDIT_APPLET 8
  {
    GNOME_APP_UI_ITEM, N_("_Applet"),
    NULL,
    (gpointer) on_mainwinmenu_applet_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOTTOM,
    0, (GdkModifierType) 0, NULL
  },	
#define POPMAINWINMENU_EDIT_END 9
  GNOMEUIINFO_END
};

static GnomeUIInfo popmainwinmenu_settings_menu_uiinfo[] =
{ /* The settings menu of the popupmenu */
#define POPMAINWINMENU_SETTINGS_PREFERENCES 0
  GNOMEUIINFO_MENU_PREFERENCES_ITEM (on_mainwinmenu_preferences_activate, NULL),
#define POPMAINWINMENU_SETTINGS_END 1
  GNOMEUIINFO_END
};

static GnomeUIInfo popmainwinmenu_help_menu_uiinfo[] =
{ /* The helpmenu of the popupmenu */
#define POPMAINWINMENU_HELP_ABOUT 0
  GNOMEUIINFO_MENU_ABOUT_ITEM (on_mainwinmenu_about_activate, NULL),
#define POPMAINWINMENU_HELP_END 1
  GNOMEUIINFO_END
};



static GnomeUIInfo popmainwinmenu_uiinfo[] =
{ /* The submenu of the popupmenu (shown if mainmenu is hidden) */
#define POPMAINWINMENU_FILE 0
  GNOMEUIINFO_MENU_FILE_TREE (popmainwinmenu_file_menu_uiinfo),
#define POPMAINWINMENU_EDIT 1
  GNOMEUIINFO_MENU_EDIT_TREE (popmainwinmenu_edit_menu_uiinfo),
#define POPMAINWINMENU_SETTINGS 2
  GNOMEUIINFO_MENU_SETTINGS_TREE (popmainwinmenu_settings_menu_uiinfo),
#define POPMAINWINMENU_HELP 3
  GNOMEUIINFO_MENU_HELP_TREE (popmainwinmenu_help_menu_uiinfo),
#define POPMAINWINMENU_END 4
  GNOMEUIINFO_END
};

static GnomeUIInfo cliplistpopupmenu_uiinfo[] =
{ /* The popupmenu */
#define CLIPLISTPOPUPMENU_NEWITEM 0
  GNOMEUIINFO_MENU_NEW_ITEM (N_("_New"), NULL, on_cliplistpopupmenu_new_activate, NULL),
#define CLIPLISTPOPUPMENU_EDITITEM 1
{
    GNOME_APP_UI_ITEM, N_("_Edit item"),
    NULL,
    (gpointer) on_cliplistpopupmenu_edit_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE_AS,
    0, (GdkModifierType) 0, NULL
  },
#define CLIPLISTPOPUPMENU_F_SEPARATOR 2
  GNOMEUIINFO_SEPARATOR,  
#define CLIPLISTPOPUPMENU_CUTITEM 3
  GNOMEUIINFO_MENU_CUT_ITEM (on_cliplistpopupmenu_cut_activate, NULL),
#define CLIPLISTPOPUPMENU_COPYITEM 4
  GNOMEUIINFO_MENU_COPY_ITEM (on_cliplistpopupmenu_copy_activate, NULL),
#define CLIPLISTPOPUPMENU_PASTEITEM 5
  GNOMEUIINFO_MENU_PASTE_ITEM (on_cliplistpopupmenu_paste_activate, NULL),
#define CLIPLISTPOPUPMENU_FILEMENU 6
  GNOMEUIINFO_SUBTREE (N_("_Menu"), popmainwinmenu_uiinfo),
#define CLIPLISTPOPUPMENU_S_SEPARATOR 7
  GNOMEUIINFO_SEPARATOR,  

#define CLIPLISTPOPUPMENU_MOVE_UP 8
  {
    GNOME_APP_UI_ITEM, N_("Move item _up"),
    NULL,
    (gpointer) on_mainwinmenu_moveup_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_UP,
    0, (GdkModifierType) 0, NULL
  },
#define CLIPLISTPOPUPMENU_MOVE_DOWN 9
  {
    GNOME_APP_UI_ITEM, N_("Move item _down"),
    NULL,
    (gpointer) on_mainwinmenu_movedown_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_DOWN,
    0, (GdkModifierType) 0, NULL
  },
  
#define CLIPLISTPOPUPMENU_EDIT_SEPARATOR 10
  GNOMEUIINFO_SEPARATOR,  
#define CLIPLISTPOPUPMENU_CONVERT 11
  {
    GNOME_APP_UI_ITEM, N_("C_onvert data to items"),
    NULL,
    (gpointer) on_cliplistpopupmenu_convert_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_REDO,
    0, (GdkModifierType) 0, NULL
  },

#define CLIPLISTPOPUPMENU_EDIT_CLEAR 12
  GNOMEUIINFO_MENU_CLEAR_ITEM (on_mainwinmenu_clear_activate, NULL), 
#define CLIPLISTPOPUPMENU_EDIT_SELECT_ALL 13
  GNOMEUIINFO_MENU_SELECT_ALL_ITEM (on_mainwinmenu_selectall_activate, NULL),
{
#define CLIPLISTPOPUPMENU_EDIT_SELECT_NONE 14
    GNOME_APP_UI_ITEM, N_("S_elect none"),
    NULL,
    (gpointer) on_mainwinmenu_selectnone_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_UNDO,
    0, (GdkModifierType) 0, NULL
  },	
  
#define CLIPLISTPOPUPMENU_END 15
  GNOMEUIINFO_END
};





void app_createhintsfor_menus(MainWin *mwin)
{ /* Creates hints and accel_groups for all menus */
	gint i;
	for (i=0; i<MAINWINMENU_FILE_END; i++) {
		if (mainwinmenu_file_menu_uiinfo[i].widget) {
			gtk_widget_ref (mainwinmenu_file_menu_uiinfo[i].widget);
			gtk_accel_group_attach(mwin->accel_group, GTK_OBJECT(mainwinmenu_file_menu_uiinfo[i].widget));
		}
	}
	for (i=0; i<MAINWINMENU_EDIT_END; i++) {
		if (mainwinmenu_edit_menu_uiinfo[i].widget) {
			gtk_widget_ref (mainwinmenu_edit_menu_uiinfo[i].widget);
			gtk_accel_group_attach(mwin->accel_group, GTK_OBJECT(mainwinmenu_edit_menu_uiinfo[i].widget));
		}
	}

	for (i=0; i<MAINWINMENU_SETTINGS_END; i++) {
		if (mainwinmenu_settings_menu_uiinfo[i].widget) {
			gtk_widget_ref (mainwinmenu_settings_menu_uiinfo[i].widget);
			gtk_accel_group_attach(mwin->accel_group, GTK_OBJECT(mainwinmenu_settings_menu_uiinfo[i].widget));
		}
	}

	for (i=0; i<MAINWINMENU_HELP_END; i++) {
		if (mainwinmenu_help_menu_uiinfo[i].widget) {
			gtk_widget_ref (mainwinmenu_help_menu_uiinfo[i].widget);
			gtk_accel_group_attach(mwin->accel_group, GTK_OBJECT(mainwinmenu_help_menu_uiinfo[i].widget));
		}
	}

	for (i=0; i<POPMAINWINMENU_FILE_END; i++) {
		if (popmainwinmenu_file_menu_uiinfo[i].widget) {
			gtk_widget_ref (popmainwinmenu_file_menu_uiinfo[i].widget);
			gtk_accel_group_attach(mwin->accel_group, GTK_OBJECT(popmainwinmenu_file_menu_uiinfo[i].widget));
		}
	}

	for (i=0; i<POPMAINWINMENU_EDIT_END; i++) {
		if (popmainwinmenu_edit_menu_uiinfo[i].widget) {
			gtk_widget_ref (popmainwinmenu_edit_menu_uiinfo[i].widget);
			gtk_accel_group_attach(mwin->accel_group, GTK_OBJECT(popmainwinmenu_edit_menu_uiinfo[i].widget));
		}
	}

	for (i=0; i<POPMAINWINMENU_SETTINGS_END; i++) {
		if (popmainwinmenu_settings_menu_uiinfo[i].widget) {
			gtk_widget_ref (popmainwinmenu_settings_menu_uiinfo[i].widget);
			gtk_accel_group_attach(mwin->accel_group, GTK_OBJECT(popmainwinmenu_settings_menu_uiinfo[i].widget));
		}
	}
	for (i=0; i<POPMAINWINMENU_HELP_END; i++) {
		if (popmainwinmenu_help_menu_uiinfo[i].widget) {
			gtk_widget_ref (popmainwinmenu_help_menu_uiinfo[i].widget);
			gtk_accel_group_attach(mwin->accel_group, GTK_OBJECT(popmainwinmenu_help_menu_uiinfo[i].widget));
		}
	}

	for (i=0; i<CLIPLISTPOPUPMENU_END; i++) {
		if (cliplistpopupmenu_uiinfo[i].widget) {
			gtk_widget_ref (cliplistpopupmenu_uiinfo[i].widget);
			gtk_accel_group_attach(mwin->accel_group, GTK_OBJECT(cliplistpopupmenu_uiinfo[i].widget));
		}
	}
}

void app_prepare_menus(MainWin *mwin)
{ /* Creates all menu's */
	gint i;
	
	for (i=0; i<MAINWINMENU_FILE_END; i++) {
		mainwinmenu_file_menu_uiinfo[i].user_data = mwin;
	}
	for (i=0; i<MAINWINMENU_EDIT_END; i++) {
		mainwinmenu_edit_menu_uiinfo[i].user_data = mwin;
	}

	for (i=0; i<MAINWINMENU_SETTINGS_END; i++) {
		mainwinmenu_settings_menu_uiinfo[i].user_data = mwin;
	}

	for (i=0; i<MAINWINMENU_HELP_END; i++) {
		mainwinmenu_help_menu_uiinfo[i].user_data = mwin;
	}

	for (i=0; i<POPMAINWINMENU_FILE_END; i++) {
		popmainwinmenu_file_menu_uiinfo[i].user_data = mwin;
	}

	for (i=0; i<POPMAINWINMENU_EDIT_END; i++) {
		popmainwinmenu_edit_menu_uiinfo[i].user_data = mwin;
	}

	for (i=0; i<POPMAINWINMENU_SETTINGS_END; i++) {
		popmainwinmenu_settings_menu_uiinfo[i].user_data = mwin;
	}
	for (i=0; i<POPMAINWINMENU_HELP_END; i++) {
		popmainwinmenu_help_menu_uiinfo[i].user_data = mwin;
	}

	for (i=0; i<CLIPLISTPOPUPMENU_END; i++) {
		cliplistpopupmenu_uiinfo[i].user_data = mwin;
	}
}


void app_update_menus(MainWin *mwin)
{/* Function that updates the status of the menuitems */
	GList *selected;

		/* This needs documentation for users */
	if ((mwin->prefs->show_menu == FALSE) || (mwin->prefs->applet == TRUE)) {
		gtk_widget_show (GTK_WIDGET (cliplistpopupmenu_uiinfo[CLIPLISTPOPUPMENU_FILEMENU].widget));
	} else {
		gtk_widget_hide (GTK_WIDGET (cliplistpopupmenu_uiinfo[CLIPLISTPOPUPMENU_FILEMENU].widget));
	}

	
	if (mwin->openfilelist_inuse==TRUE) {
		/* If the openfile dialog is in use */
		gtk_widget_set_sensitive (GTK_WIDGET (mainwinmenu_file_menu_uiinfo[MAINWINMENU_FILE_OPENITEMFROMDISK].widget), FALSE);
		gtk_widget_set_sensitive (GTK_WIDGET (popmainwinmenu_file_menu_uiinfo[POPMAINWINMENU_FILE_OPENITEMFROMDISK].widget), FALSE);
	} else {
		gtk_widget_set_sensitive (GTK_WIDGET (mainwinmenu_file_menu_uiinfo[MAINWINMENU_FILE_OPENITEMFROMDISK].widget), TRUE);
		gtk_widget_set_sensitive (GTK_WIDGET (popmainwinmenu_file_menu_uiinfo[POPMAINWINMENU_FILE_OPENITEMFROMDISK].widget), TRUE);
	}
	
	if (mwin->savefilelist_inuse==TRUE) {
		/* If the savefile dialog is in use */
		gtk_widget_set_sensitive (GTK_WIDGET (mainwinmenu_file_menu_uiinfo[MAINWINMENU_FILE_SAVEITEMSTODISK].widget), FALSE);
		gtk_widget_set_sensitive (GTK_WIDGET (popmainwinmenu_file_menu_uiinfo[POPMAINWINMENU_FILE_SAVEITEMSTODISK].widget), FALSE);	
		
		gtk_widget_set_sensitive (GTK_WIDGET (mainwinmenu_file_menu_uiinfo[MAINWINMENU_FILE_SAVEALLITEMSTODISK].widget), FALSE);
		gtk_widget_set_sensitive (GTK_WIDGET (popmainwinmenu_file_menu_uiinfo[POPMAINWINMENU_FILE_SAVEALLITEMSTODISK].widget), FALSE);
		
	} else {

		gtk_widget_set_sensitive (GTK_WIDGET (mainwinmenu_file_menu_uiinfo[MAINWINMENU_FILE_SAVEITEMSTODISK].widget), TRUE);
		gtk_widget_set_sensitive (GTK_WIDGET (popmainwinmenu_file_menu_uiinfo[POPMAINWINMENU_FILE_SAVEITEMSTODISK].widget), TRUE);			
		/* the saveallitems menuitem is set sensitive in next if statement */
	}
	
	if ((GTK_CLIST(mwin->cliplist)->rows > 0) && (mwin->savefilelist_inuse==FALSE)) {
		gtk_widget_set_sensitive (GTK_WIDGET (mainwinmenu_file_menu_uiinfo[MAINWINMENU_FILE_SAVEALLITEMSTODISK].widget), TRUE);
		gtk_widget_set_sensitive (GTK_WIDGET (popmainwinmenu_file_menu_uiinfo[POPMAINWINMENU_FILE_SAVEALLITEMSTODISK].widget), TRUE);
	} else {
		gtk_widget_set_sensitive (GTK_WIDGET (mainwinmenu_file_menu_uiinfo[MAINWINMENU_FILE_SAVEALLITEMSTODISK].widget), FALSE);
		gtk_widget_set_sensitive (GTK_WIDGET (popmainwinmenu_file_menu_uiinfo[POPMAINWINMENU_FILE_SAVEALLITEMSTODISK].widget), FALSE);
	}

	if (mwin->copypaste) {
		if (g_list_first (mwin->copypaste)) {
			gtk_widget_set_sensitive (GTK_WIDGET (mainwinmenu_edit_menu_uiinfo[MAINWINMENU_EDIT_PASTE].widget), TRUE);
			gtk_widget_set_sensitive (GTK_WIDGET (popmainwinmenu_edit_menu_uiinfo[POPMAINWINMENU_EDIT_PASTE].widget), TRUE);
			gtk_widget_set_sensitive (GTK_WIDGET (cliplistpopupmenu_uiinfo[CLIPLISTPOPUPMENU_PASTEITEM].widget), TRUE);
			goto ddis;
		} else goto dis;
	} else goto dis;

dis:
	gtk_widget_set_sensitive (GTK_WIDGET (mainwinmenu_edit_menu_uiinfo[MAINWINMENU_EDIT_PASTE].widget), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (popmainwinmenu_edit_menu_uiinfo[POPMAINWINMENU_EDIT_PASTE].widget), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (cliplistpopupmenu_uiinfo[CLIPLISTPOPUPMENU_PASTEITEM].widget), FALSE);
ddis:	
	
	selected = GTK_CLIST(mwin->cliplist)->selection;
	if (selected) {
		selected = g_list_first (selected);
		if (selected) {
			gtk_widget_set_sensitive (GTK_WIDGET (popmainwinmenu_file_menu_uiinfo[POPMAINWINMENU_FILE_SAVEITEMSTODISK].widget), TRUE);

			gtk_widget_set_sensitive (GTK_WIDGET (popmainwinmenu_edit_menu_uiinfo[POPMAINWINMENU_EDIT_CUT].widget), TRUE);
			gtk_widget_set_sensitive (GTK_WIDGET (popmainwinmenu_edit_menu_uiinfo[POPMAINWINMENU_EDIT_COPY].widget), TRUE);
			gtk_widget_set_sensitive (GTK_WIDGET (mainwinmenu_edit_menu_uiinfo[MAINWINMENU_EDIT_CUT].widget), TRUE);
			gtk_widget_set_sensitive (GTK_WIDGET (mainwinmenu_edit_menu_uiinfo[MAINWINMENU_EDIT_COPY].widget), TRUE);
	
			gtk_widget_set_sensitive (GTK_WIDGET (cliplistpopupmenu_uiinfo[CLIPLISTPOPUPMENU_CUTITEM].widget), TRUE);
			gtk_widget_set_sensitive (GTK_WIDGET (cliplistpopupmenu_uiinfo[CLIPLISTPOPUPMENU_COPYITEM].widget), TRUE);
			gtk_widget_set_sensitive (GTK_WIDGET (cliplistpopupmenu_uiinfo[CLIPLISTPOPUPMENU_EDITITEM].widget), TRUE);
			gtk_widget_set_sensitive (GTK_WIDGET (cliplistpopupmenu_uiinfo[CLIPLISTPOPUPMENU_MOVE_DOWN].widget), TRUE);
			gtk_widget_set_sensitive (GTK_WIDGET (cliplistpopupmenu_uiinfo[CLIPLISTPOPUPMENU_MOVE_UP].widget), TRUE);
			gtk_widget_set_sensitive (GTK_WIDGET (cliplistpopupmenu_uiinfo[CLIPLISTPOPUPMENU_CONVERT].widget), TRUE);

			gtk_widget_set_sensitive (GTK_WIDGET (mainwinmenu_file_menu_uiinfo[MAINWINMENU_FILE_DELETEITEM].widget), TRUE);
			gtk_widget_set_sensitive (GTK_WIDGET (mwin->toolbar_deleteitem), TRUE);
			gtk_widget_set_sensitive (GTK_WIDGET (mwin->toolbar_edititem), TRUE);
		} else goto deactivate;
	} else goto deactivate;
	return;

deactivate:
	
	gtk_widget_set_sensitive (GTK_WIDGET (popmainwinmenu_edit_menu_uiinfo[POPMAINWINMENU_EDIT_CUT].widget), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (popmainwinmenu_edit_menu_uiinfo[POPMAINWINMENU_EDIT_COPY].widget), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (mainwinmenu_edit_menu_uiinfo[MAINWINMENU_EDIT_CUT].widget), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (mainwinmenu_edit_menu_uiinfo[MAINWINMENU_EDIT_COPY].widget), FALSE);

	gtk_widget_set_sensitive (GTK_WIDGET (cliplistpopupmenu_uiinfo[CLIPLISTPOPUPMENU_CUTITEM].widget), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (cliplistpopupmenu_uiinfo[CLIPLISTPOPUPMENU_COPYITEM].widget), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (cliplistpopupmenu_uiinfo[CLIPLISTPOPUPMENU_EDITITEM].widget), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (cliplistpopupmenu_uiinfo[CLIPLISTPOPUPMENU_MOVE_DOWN].widget), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (cliplistpopupmenu_uiinfo[CLIPLISTPOPUPMENU_MOVE_UP].widget), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (cliplistpopupmenu_uiinfo[CLIPLISTPOPUPMENU_CONVERT].widget), FALSE);

	gtk_widget_set_sensitive (GTK_WIDGET (popmainwinmenu_file_menu_uiinfo[POPMAINWINMENU_FILE_SAVEITEMSTODISK].widget), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (mainwinmenu_file_menu_uiinfo[MAINWINMENU_FILE_SAVEITEMSTODISK].widget), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (mainwinmenu_file_menu_uiinfo[MAINWINMENU_FILE_DELETEITEM].widget), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (mwin->toolbar_deleteitem), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (mwin->toolbar_edititem), FALSE);
	return;
}






//}

//{ Widget stuff
GtkType mainwin_get_type(void)
{ /* mainwin gettype */
	static GtkType mainwin_type = 0;
	if (!mainwin_type)
	{
		static const GtkTypeInfo mainwin_info =
		{	
			"MainWin",
			sizeof (MainWin),
			sizeof (MainWinClass),
			(GtkClassInitFunc) mainwin_class_init,
			(GtkObjectInitFunc) mainwin_init,
			NULL,
			NULL,
			(GtkClassInitFunc) NULL,
		};
		mainwin_type = gtk_type_unique (GTK_TYPE_WINDOW, &mainwin_info);
	}
	return (mainwin_type);
}

static void mainwin_class_init(MainWinClass *klass)
{ /* on mainwin classinit */
	GtkObjectClass *object_class;
	object_class = (GtkObjectClass*)klass;
	object_class->destroy = mainwin_destroy;
	parent_class = gtk_type_class(GTK_TYPE_WINDOW);
}

static void mainwin_init(MainWin *mwin)
{ /* On mainwin init */

}

static void mainwin_destroy(GtkObject *object)
{ /* on mainwin destroy */
	GtkWidget *mwin;

	g_return_if_fail(object != NULL);
	g_return_if_fail(GNOME_IS_MAINWIN(object));
	mwin = GTK_WIDGET(object);
	gtk_widget_destroy(mwin);
	gtk_exit(0);
}





//}


//{ free functions
void saveasdata_free (SaveAsData *sa) 
{ /* free's a SaveAsData */
	g_list_free(sa->items);
	g_free(sa->path);
	g_free(sa);
}

void selection_free (Selection *item) 
{ /* free's a Selection */
  g_free (item->time);
  g_free (item->type);
  g_free (item->from);
  g_free (item->data);
  g_free(item);
}


void prefs_free (Prefs *p) 
{ /* free's a Pref */
	g_free (p->custom_selection_type);
	g_free (p);
}


//}

//{ Preferences functions
gboolean saveprefs_tofile (Prefs *prefs)
{/* Saves the preferences (to GConf in our case) */
	GConfEngine *conf;
	GError *err=NULL;
	if (prefs != NULL) {
		if (gconf_is_initialized ()) {
			conf = gconf_engine_get_default ();
			gconf_engine_ref (conf);
			if (!gconf_engine_set_bool (conf, "/apps/gcm/prefs/notfirst_time", TRUE, &err)) return FALSE;
			if (!gconf_engine_set_int (conf, "/apps/gcm/prefs/selection_type", prefs->selection_type, &err)) return FALSE;
			if (!gconf_engine_set_string (conf, "/apps/gcm/prefs/custom_selection_type", prefs->custom_selection_type, &err)) return FALSE;
			if (!gconf_engine_set_int (conf, "/apps/gcm/prefs/target_type", prefs->target_type, &err)) return FALSE;
			if (!gconf_engine_set_string (conf, "/apps/gcm/prefs/custom_target_type", prefs->custom_target_type, &err)) return FALSE;
			if (!gconf_engine_set_bool (conf, "/apps/gcm/prefs/beep_on_collect", prefs->beep_on_collect, &err)) return FALSE;
			if (!gconf_engine_set_bool (conf, "/apps/gcm/prefs/auto_collect", prefs->auto_collect, &err)) return FALSE;
#ifdef DEVELOPMENT
			if (!gconf_engine_set_bool (conf, "/apps/gcm/prefs/debug", prefs->debug, &err)) return FALSE;
#endif
			if (!gconf_engine_set_bool (conf, "/apps/gcm/prefs/show_cliplist_titles", prefs->show_cliplist_titles, &err)) return FALSE;
			if (!gconf_engine_set_bool (conf, "/apps/gcm/prefs/show_statusbar", prefs->show_statusbar, &err)) return FALSE;
			if (!gconf_engine_set_bool (conf, "/apps/gcm/prefs/show_toolbar", prefs->show_toolbar, &err)) return FALSE;
			if (!gconf_engine_set_bool (conf, "/apps/gcm/prefs/show_menu", prefs->show_menu, &err)) return FALSE;
			if (!gconf_engine_set_bool (conf, "/apps/gcm/prefs/show_from", prefs->show_from, &err)) return FALSE;
			if (!gconf_engine_set_bool (conf, "/apps/gcm/prefs/show_type", prefs->show_type, &err)) return FALSE;
			if (!gconf_engine_set_bool (conf, "/apps/gcm/prefs/show_time", prefs->show_time, &err)) return FALSE;
			if (!gconf_engine_set_bool (conf, "/apps/gcm/prefs/show_datap", prefs->show_datap, &err)) return FALSE;
			if (!gconf_engine_set_bool (conf, "/apps/gcm/prefs/multisel", prefs->multisel, &err)) return FALSE;
			if (!gconf_engine_set_bool (conf, "/apps/gcm/prefs/applet", prefs->applet, &err)) return FALSE;
			if (!gconf_engine_set_bool (conf, "/apps/gcm/prefs/followmode", prefs->followmode, &err)) return FALSE;
			if (!gconf_engine_set_bool (conf, "/apps/gcm/prefs/unselall", prefs->unselall, &err)) return FALSE;
			if (!gconf_engine_set_bool (conf, "/apps/gcm/prefs/selectnew", prefs->selectnew, &err)) return FALSE;
			if (!gconf_engine_set_int (conf, "/apps/gcm/prefs/maxitems", prefs->maxitems, &err)) return FALSE;
			if (!gconf_engine_set_bool (conf, "/apps/gcm/prefs/usemaxitems", prefs->usemaxitems, &err)) return FALSE;
		} else {
			gnome_error_dialog(_("Unable to initialize GConf. Check your GConf package.")); 
			return FALSE;
		}
	} else {
		return FALSE;
	}
	return TRUE;
}


Prefs* getprefs_fromfile (void)
{/* Loads the preferences from file (GConf in our case) */
	Prefs *prefs=NULL;
	GConfEngine *conf;
	GError *err=NULL;

	conf = gconf_engine_get_default ();
	gconf_engine_ref (conf);
	prefs = g_malloc0(sizeof(Prefs));

	/* Fetched from gconf */
	if (gconf_engine_get_bool (conf, "/apps/gcm/prefs/notfirst_time", &err)) {
		prefs->selection_type = gconf_engine_get_int (conf, "/apps/gcm/prefs/selection_type", &err);
		prefs->custom_selection_type = gconf_engine_get_string (conf, "/apps/gcm/prefs/custom_selection_type", &err);
		prefs->target_type = gconf_engine_get_int (conf, "/apps/gcm/prefs/target_type", &err);
		prefs->custom_target_type = gconf_engine_get_string (conf, "/apps/gcm/prefs/custom_target_type", &err);
		prefs->beep_on_collect = gconf_engine_get_bool (conf, "/apps/gcm/prefs/beep_on_collect", &err);
		prefs->auto_collect = gconf_engine_get_bool (conf, "/apps/gcm/prefs/auto_collect", &err);
#ifdef DEVELOPMENT
		prefs->debug = gconf_engine_get_bool (conf, "/apps/gcm/prefs/debug", &err);
#endif
		prefs->show_toolbar = gconf_engine_get_bool (conf, "/apps/gcm/prefs/show_toolbar", &err);
		prefs->show_menu = gconf_engine_get_bool (conf, "/apps/gcm/prefs/show_menu", &err);
		prefs->show_cliplist_titles = gconf_engine_get_bool (conf, "/apps/gcm/prefs/show_cliplist_titles", &err);
		prefs->show_statusbar = gconf_engine_get_bool (conf, "/apps/gcm/prefs/show_statusbar", &err);
		prefs->show_from = gconf_engine_get_bool (conf, "/apps/gcm/prefs/show_from", &err);
		prefs->show_type = gconf_engine_get_bool (conf, "/apps/gcm/prefs/show_type", &err);
		prefs->show_time = gconf_engine_get_bool (conf, "/apps/gcm/prefs/show_time", &err);
		prefs->show_datap = gconf_engine_get_bool (conf, "/apps/gcm/prefs/show_datap", &err);
		prefs->multisel = gconf_engine_get_bool (conf, "/apps/gcm/prefs/multisel", &err);
		prefs->applet = gconf_engine_get_bool (conf, "/apps/gcm/prefs/applet", &err);
		prefs->selectnew = gconf_engine_get_bool (conf, "/apps/gcm/prefs/selectnew", &err);
		prefs->unselall = gconf_engine_get_bool (conf, "/apps/gcm/prefs/unselall", &err);
		prefs->followmode = gconf_engine_get_bool (conf, "/apps/gcm/prefs/followmode", &err);
		prefs->maxitems = gconf_engine_get_int (conf, "/apps/gcm/prefs/maxitems", &err);
		prefs->usemaxitems = gconf_engine_get_bool (conf, "/apps/gcm/prefs/usemaxitems", &err);
	} else {
		prefs->selection_type = PREFS_SELECTION_TYPE_CLIPBOARD;
		prefs->target_type = PREFS_TARGET_TYPE_TEXT;
		prefs->show_menu = TRUE;
		prefs->show_toolbar = TRUE;
#ifdef DEVELOPMENT
		prefs->debug = FALSE;
#endif
		prefs->auto_collect=TRUE;
		prefs->beep_on_collect=TRUE;
		prefs->show_toolbar=TRUE;
		prefs->show_menu=TRUE;
		prefs->show_cliplist_titles=TRUE;
		prefs->show_statusbar=TRUE;
		prefs->show_from=TRUE;
		prefs->show_type=TRUE;
		prefs->show_time=TRUE;
		prefs->show_datap=TRUE;
		prefs->multisel=TRUE;
		prefs->applet=FALSE;
		prefs->unselall=TRUE;
		prefs->followmode=TRUE;
		prefs->selectnew=TRUE;
		prefs->usemaxitems=FALSE;
		prefs->maxitems=99;
		prefs->custom_target_type = g_strdup("TEXT");
		prefs->custom_selection_type = g_strdup("CLIPBOARD");
		saveprefs_tofile (prefs);
		/* Any firsttime messages could go here */
	}
	return prefs;
}


void reload_prefs (MainWin *mwin)
{/* Reloads the preferences. Should be called each time a preference is changed */
  /* Not sure about this one. getprefs_fromfile allocates memory for the prefs
     so we first free the old prefs data if there was prefsdata of course...
     somebody correct me if this is wrong :) !
   */
  if (mwin->prefs) prefs_free(mwin->prefs);

  mwin->prefs = getprefs_fromfile ();
  switch (mwin->prefs->selection_type) {
	 case PREFS_SELECTION_TYPE_CLIPBOARD :
		  mwin->selection = gdk_atom_intern ("CLIPBOARD", FALSE);
		  gtk_entry_set_text (GTK_ENTRY (GNOME_PREFSWIN(mwin->prefswin)->seltypecentry), 
		 _(PREFS_SELECTION_TYPE_CLIPBOARD_TEXT));
		 
	 break;

	 case PREFS_SELECTION_TYPE_PRIMARY :
		  mwin->selection = GDK_SELECTION_PRIMARY;
		  gtk_entry_set_text (GTK_ENTRY (GNOME_PREFSWIN(mwin->prefswin)->seltypecentry), 
		 _(PREFS_SELECTION_TYPE_PRIMARY_TEXT));
		 
	 break;

	 case PREFS_SELECTION_TYPE_SECONDARY :
		  mwin->selection = GDK_SELECTION_SECONDARY;
		  gtk_entry_set_text (GTK_ENTRY (GNOME_PREFSWIN(mwin->prefswin)->seltypecentry), 
		 _(PREFS_SELECTION_TYPE_SECONDARY_TEXT));
		 
	 break;

	 case PREFS_SELECTION_TYPE_CUSTOM :
		  mwin->selection = gdk_atom_intern (mwin->prefs->custom_selection_type, FALSE);
		  gtk_entry_set_text (GTK_ENTRY (GNOME_PREFSWIN(mwin->prefswin)->seltypecentry), 
		 _(PREFS_SELECTION_TYPE_CUSTOM_TEXT));
		 
	 break;

	 default :
		  mwin->selection = gdk_atom_intern ("CLIPBOARD", FALSE);
		  gtk_entry_set_text (GTK_ENTRY (GNOME_PREFSWIN(mwin->prefswin)->seltypecentry), 
		 _(PREFS_SELECTION_TYPE_CLIPBOARD_TEXT));
		 
	 break;
  }
  
  switch (mwin->prefs->target_type) {
	case PREFS_TARGET_TYPE_HTML :
		  mwin->target_atom = gdk_atom_intern ("text/html", FALSE);
		  gtk_entry_set_text (GTK_ENTRY (GNOME_PREFSWIN(mwin->prefswin)->tartypecentry), 
		 _(PREFS_TARGET_TYPE_HTML_TEXT));

	break;
	case PREFS_TARGET_TYPE_CUSTOM :
		  mwin->target_atom = gdk_atom_intern (mwin->prefs->custom_target_type, FALSE);
		  gtk_entry_set_text (GTK_ENTRY (GNOME_PREFSWIN(mwin->prefswin)->tartypecentry), 
		 _(PREFS_TARGET_TYPE_CUSTOM_TEXT));

	break;
	case PREFS_TARGET_TYPE_TARGETS :
		  mwin->target_atom = gdk_atom_intern ("TARGETS", FALSE);
		  gtk_entry_set_text (GTK_ENTRY (GNOME_PREFSWIN(mwin->prefswin)->tartypecentry), 
		 _(PREFS_TARGET_TYPE_TARGETS_TEXT));

	break;
	case PREFS_TARGET_TYPE_RICH :
		  mwin->target_atom = gdk_atom_intern ("text/richtext", FALSE);
		  gtk_entry_set_text (GTK_ENTRY (GNOME_PREFSWIN(mwin->prefswin)->tartypecentry), 
		 _(PREFS_TARGET_TYPE_RICH_TEXT));

	break;

	case PREFS_TARGET_TYPE_TEXT :
	default :
		  mwin->target_atom = gdk_atom_intern ("TEXT", FALSE);
		  gtk_entry_set_text (GTK_ENTRY (GNOME_PREFSWIN(mwin->prefswin)->tartypecentry), 
		 _(PREFS_TARGET_TYPE_TEXT_TEXT));
	break;
	
  }
  
  gtk_selection_add_target (GTK_WIDGET(mwin), mwin->selection, gdk_atom_intern ("TARGETS", FALSE), 0);
  gtk_selection_add_target (GTK_WIDGET(mwin), mwin->selection, gdk_atom_intern ("MULTIPLE", FALSE), 0);
  gtk_selection_add_target (GTK_WIDGET(mwin), mwin->selection, gdk_atom_intern ("TIMESTAMP", FALSE), 0);
  gtk_selection_add_target (GTK_WIDGET(mwin), mwin->selection, gdk_atom_intern ("INCR", FALSE), 0);
  gtk_selection_add_target (GTK_WIDGET(mwin), mwin->selection, gdk_atom_intern ("STRING", FALSE), 0);
  gtk_selection_add_target (GTK_WIDGET(mwin), mwin->selection, gdk_atom_intern ("TEXT", FALSE), 0);
  gtk_selection_add_target (GTK_WIDGET(mwin), mwin->selection, gdk_atom_intern ("text/html", FALSE), 0);
  gtk_selection_add_target (GTK_WIDGET(mwin), mwin->selection, gdk_atom_intern ("text/richtext", FALSE), 0);
  gtk_selection_add_target (GTK_WIDGET(mwin), mwin->selection, gdk_atom_intern ("COMPOUND_TEXT", FALSE), 0);
  gtk_selection_add_target (GTK_WIDGET(mwin), mwin->selection, gdk_atom_intern ("UTF8_STRING", FALSE), 0);
  gtk_selection_add_target (GTK_WIDGET(mwin), mwin->selection, GDK_SELECTION_TYPE_STRING,1);

  gtk_entry_set_text (GTK_ENTRY (GNOME_PREFSWIN(mwin->prefswin)->seltypecustentry), mwin->prefs->custom_selection_type);
  gtk_entry_set_text (GTK_ENTRY (GNOME_PREFSWIN(mwin->prefswin)->tartypecustentry), mwin->prefs->custom_target_type);
#ifdef DEVELOPMENT
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(GNOME_PREFSWIN(mwin->prefswin)->devcheck), mwin->prefs->debug);
#endif
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(GNOME_PREFSWIN(mwin->prefswin)->colbeep), mwin->prefs->beep_on_collect);  
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(GNOME_PREFSWIN(mwin->prefswin)->colauto), mwin->prefs->auto_collect);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(GNOME_PREFSWIN(mwin->prefswin)->colmaxcheck), mwin->prefs->usemaxitems);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(GNOME_PREFSWIN(mwin->prefswin)->guishowtoolbar), mwin->prefs->show_toolbar);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON(GNOME_PREFSWIN(mwin->prefswin)->colmaxspin), mwin->prefs->maxitems);
  gtk_widget_set_sensitive(GNOME_PREFSWIN(mwin->prefswin)->colmaxspin, mwin->prefs->usemaxitems);

  if (mwin->prefs->show_toolbar) {
	  gtk_widget_show(mwin->toolbar);
	  gtk_widget_show(mwin->toolbarhandlebox);
  } else {
	  gtk_widget_hide(mwin->toolbar);
	  gtk_widget_hide(mwin->toolbarhandlebox);
  }
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(GNOME_PREFSWIN(mwin->prefswin)->guishowmenu), mwin->prefs->show_menu);
  if (mwin->prefs->show_menu) {
	  gtk_widget_show(mwin->mainwinmenu);
	  gtk_widget_show(mwin->menuhandlebox);
  } else {
	  gtk_widget_hide(mwin->mainwinmenu);
	  gtk_widget_hide(mwin->menuhandlebox);
  }

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(GNOME_PREFSWIN(mwin->prefswin)->guishowstatus), mwin->prefs->show_statusbar);  
  if (mwin->prefs->show_statusbar) {
	  gtk_widget_show(mwin->appbar);
  } else {
	  gtk_widget_hide(mwin->appbar);
  }

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(GNOME_PREFSWIN(mwin->prefswin)->aplapplet), mwin->prefs->applet);
  reload_cliplist_prefs(mwin);
  
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(GNOME_PREFSWIN(mwin->prefswin)->advmultisel), mwin->prefs->multisel);
  if (mwin->prefs->multisel) {
	gtk_widget_show(mainwinmenu_edit_menu_uiinfo[MAINWINMENU_EDIT_SELECT_ALL].widget);
	gtk_widget_show(mainwinmenu_edit_menu_uiinfo[MAINWINMENU_EDIT_SELECT_NONE].widget);
	gtk_widget_show(popmainwinmenu_edit_menu_uiinfo[POPMAINWINMENU_EDIT_SELECT_ALL].widget);
	gtk_widget_show(popmainwinmenu_edit_menu_uiinfo[POPMAINWINMENU_EDIT_SELECT_NONE].widget);
	gtk_clist_set_selection_mode (GTK_CLIST(mwin->cliplist), GTK_SELECTION_MULTIPLE);  
  } else {
  	gtk_widget_hide(mainwinmenu_edit_menu_uiinfo[MAINWINMENU_EDIT_SELECT_ALL].widget);
	gtk_widget_hide(mainwinmenu_edit_menu_uiinfo[MAINWINMENU_EDIT_SELECT_NONE].widget);
  	gtk_widget_hide(popmainwinmenu_edit_menu_uiinfo[POPMAINWINMENU_EDIT_SELECT_ALL].widget);
	gtk_widget_hide(popmainwinmenu_edit_menu_uiinfo[POPMAINWINMENU_EDIT_SELECT_NONE].widget);
	gtk_clist_set_selection_mode (GTK_CLIST(mwin->cliplist), GTK_SELECTION_SINGLE);  
  }

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(GNOME_PREFSWIN(mwin->prefswin)->advunsel), mwin->prefs->unselall);  
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(GNOME_PREFSWIN(mwin->prefswin)->advfollow), mwin->prefs->followmode);  
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(GNOME_PREFSWIN(mwin->prefswin)->advselnew), mwin->prefs->selectnew);  
  
  app_update_menus(mwin);
}

void reload_cliplist_prefs(MainWin *mwin)
{
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(GNOME_PREFSWIN(mwin->prefswin)->guishowtitles), mwin->prefs->show_cliplist_titles);  
  if (mwin->prefs->show_cliplist_titles) {
	  gtk_clist_column_titles_show (GTK_CLIST(mwin->cliplist));
  } else {
	  gtk_clist_column_titles_hide (GTK_CLIST(mwin->cliplist));
  }

  if (mwin->prefs->applet) {
	  gtk_clist_column_titles_hide (GTK_CLIST(mwin->cliplist));
  }
  
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(GNOME_PREFSWIN(mwin->prefswin)->guishowtime), mwin->prefs->show_time);
  if (mwin->prefs->show_time) {
	  gtk_clist_set_column_visibility (GTK_CLIST(mwin->cliplist), CLIST_DATE_FIELD, TRUE);
  } else {
	  gtk_clist_set_column_visibility (GTK_CLIST(mwin->cliplist), CLIST_DATE_FIELD, FALSE);
  }

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(GNOME_PREFSWIN(mwin->prefswin)->guishowtype), mwin->prefs->show_type);
  if (mwin->prefs->show_type) {
	  gtk_clist_set_column_visibility (GTK_CLIST(mwin->cliplist), CLIST_TYPE_FIELD, TRUE);
  } else {
	  gtk_clist_set_column_visibility (GTK_CLIST(mwin->cliplist), CLIST_TYPE_FIELD, FALSE);
  }

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(GNOME_PREFSWIN(mwin->prefswin)->guishowdatap), mwin->prefs->show_datap);
  if (mwin->prefs->show_datap) {
	  gtk_clist_set_column_visibility (GTK_CLIST(mwin->cliplist), CLIST_DATA_FIELD, TRUE);
  } else {
	  gtk_clist_set_column_visibility (GTK_CLIST(mwin->cliplist), CLIST_DATA_FIELD, FALSE);
  }

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(GNOME_PREFSWIN(mwin->prefswin)->guishowfrom), mwin->prefs->show_from);
  if (mwin->prefs->show_from) {
	  gtk_clist_set_column_visibility (GTK_CLIST(mwin->cliplist), CLIST_FROM_FIELD, TRUE);
  } else {
	  gtk_clist_set_column_visibility (GTK_CLIST(mwin->cliplist), CLIST_FROM_FIELD, FALSE);
  }

  if (!mwin->prefs->show_from && !mwin->prefs->show_time && !mwin->prefs->show_datap && !mwin->prefs->show_type) {
	  gtk_clist_set_column_visibility (GTK_CLIST(mwin->cliplist), CLIST_DATA_FIELD, TRUE);
	  gtk_clist_set_column_visibility (GTK_CLIST(mwin->cliplist), CLIST_FROM_FIELD, FALSE);
  }

}


//}

//{ Widget creation functions
GtkWidget* create_applet (MainWin *mwin)
{
  GtkWidget *applet;
  GtkStyle *style;

  on_mainwinmenu_selectall_activate(NULL, mwin);
  on_mainwinmenu_copy_activate (NULL, mwin);

  applet_widget_init("Gnome Clipboard Manager applet", NULL, (int) mwin->argc, (char**) mwin->argv, NULL,0,NULL);
  applet = applet_widget_new("GCM applet");
  if (!applet)
    gnome_error_dialog(_("Can't create applet!"));

  gtk_container_remove (GTK_CONTAINER (mwin->scrolledwindow), mwin->cliplist);
  app_create_cliplist(mwin);
  
  gtk_signal_connect (GTK_OBJECT (mwin->cliplist), "event",
  				GTK_SIGNAL_FUNC (on_cliplist_applet_event), mwin);

  reload_cliplist_prefs(mwin);
  
  
  
  gtk_widget_set_usize (GTK_WIDGET (mwin->cliplist), 100, 48 );
  
  mwin->prefs->applet=TRUE;
  style = gtk_widget_get_style(mwin->cliplist);
/* fixme: todo Make this a preference */
  style->font = gdk_font_load("6x10");
  gtk_widget_set_style(mwin->cliplist, style);
  
  mwin->appletscroll = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (mwin->appletscroll);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (mwin->appletscroll), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);


  gtk_container_add(GTK_CONTAINER(mwin->appletscroll), mwin->cliplist);
  gtk_clist_column_titles_hide (GTK_CLIST(mwin->cliplist));

  app_update_menus(mwin);

/* fixme: todo
  gtk_signal_connect (GTK_OBJECT(applet),"change_orient",
                            GTK_SIGNAL_FUNC(applet_change_orient),  
                            NULL);
  gtk_signal_connect (GTK_OBJECT(applet),"change_pixel_size",
                            GTK_SIGNAL_FUNC(applet_change_pixel_size),
                            NULL);
                                              
  gtk_signal_connect (GTK_OBJECT(applet),"save_session",
                            GTK_SIGNAL_FUNC(applet_save_session),   
                            NULL);
*/
  applet_widget_add (APPLET_WIDGET (applet), mwin->appletscroll);
  gtk_widget_hide(GTK_WIDGET(mwin));
  mwin->appletheight = applet_widget_get_panel_pixel_size (APPLET_WIDGET(applet));

/* fixme : todo Make this a preference */
  gtk_widget_set_usize (GTK_SCROLLED_WINDOW(mwin->appletscroll)->vscrollbar, 3, mwin->appletheight);
  gtk_widget_set_usize (GTK_WIDGET (mwin->cliplist), 100, mwin->appletheight);


	applet_widget_register_stock_callback(APPLET_WIDGET(applet), "Save selected items to disk",
                                      GNOME_STOCK_MENU_SAVE, _("Save"),
                                      (gpointer)on_mainwinmenu_file_savetodisk_activate, mwin);
	applet_widget_register_stock_callback(APPLET_WIDGET(applet), "Load",
                                      GNOME_STOCK_MENU_OPEN, _("Load"),
                                      (gpointer)on_mainwinmenu_file_openfromdisk_activate, mwin);


	applet_widget_register_stock_callback(APPLET_WIDGET(applet), "New",
                                      GNOME_STOCK_MENU_NEW, _("New"),
                                      (gpointer) on_cliplistpopupmenu_new_activate, mwin);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet), "Edit item",
                                      GNOME_STOCK_MENU_SAVE_AS, _("Edit item"),
                                      (gpointer) on_cliplistpopupmenu_edit_activate, mwin);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet), "Cut",
                                      GNOME_STOCK_MENU_CUT, _("Cut"),
                                      (gpointer) on_cliplistpopupmenu_cut_activate, mwin);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet), "Copy",
                                      GNOME_STOCK_MENU_COPY, _("Copy"),
                                      (gpointer) on_cliplistpopupmenu_copy_activate, mwin);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet), "Paste",
                                      GNOME_STOCK_MENU_PASTE, _("Paste"),
                                      (gpointer) on_cliplistpopupmenu_paste_activate, mwin);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet), "Move item up",
                                      GNOME_STOCK_MENU_UP, _("Move item up"),
                                      (gpointer) on_mainwinmenu_moveup_activate, mwin);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet), "Move item down",
                                      GNOME_STOCK_MENU_DOWN, _("Move item down"),
                                      (gpointer) on_mainwinmenu_movedown_activate, mwin);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet), "Convert data to items",
                                      GNOME_STOCK_MENU_REDO, _("Convert data to items"),
                                      (gpointer) on_cliplistpopupmenu_convert_activate, mwin);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet), "Clear",
                                      GNOME_STOCK_MENU_TRASH, _("Clear"),
                                      (gpointer) on_mainwinmenu_clear_activate, mwin);


	applet_widget_register_callback (APPLET_WIDGET (applet), "Select all",
                                 _("Select all"),
                                 (gpointer) on_mainwinmenu_selectall_activate, mwin);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet), "Select none",
                                      GNOME_STOCK_MENU_UNDO, _("Select none"),
                                      (gpointer) on_mainwinmenu_selectnone_activate, mwin);


	applet_widget_register_stock_callback(APPLET_WIDGET(applet), "Preferences",
                                      GNOME_STOCK_MENU_PREF, _("Preferences"),
                                      (gpointer) on_toolbar_preferences_clicked, mwin);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet), "Quit",
                                      GNOME_STOCK_MENU_OPEN, _("Quit"),
                                      (gpointer)on_mainwinmenu_exit_activate, mwin);

	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
                                      "about",
                                      GNOME_STOCK_MENU_ABOUT,
                                      _("About..."),
                                      (gpointer) on_mainwinmenu_about_activate,
                                      mwin);
	       
  gtk_widget_show (applet);
  on_mainwinmenu_paste_activate (NULL, mwin);
  applet_widget_gtk_main ();
  return applet;
}

GtkWidget* mainwin_new (GtkWidget *windowparent)
{ /* Create the mainwin */
  MainWin *mwin;
  gint i;

  for (i = 0; i < sizeof (atoms) / sizeof (atoms[0]); i++)
  *atoms[i].atom = gdk_atom_intern (atoms[i].atom_name, FALSE);

  mwin = gtk_type_new(GNOME_TYPE_MAINWIN);
  gtk_signal_connect(GTK_OBJECT(mwin), "show", GTK_SIGNAL_FUNC(mainwin_show), mwin);
  gtk_signal_connect(GTK_OBJECT(mwin), "hide", GTK_SIGNAL_FUNC(mainwin_hide), mwin);
  mwin->windowparent = windowparent;
  gtk_widget_set_usize (GTK_WIDGET(mwin), 670, 400);
  gtk_window_set_title (GTK_WINDOW (mwin), _("Gnome Clipboard Manager"));
  gtk_window_set_policy (GTK_WINDOW (mwin), TRUE, TRUE, FALSE);
  mwin->accel_group = gtk_accel_group_new ();
  gtk_window_add_accel_group (GTK_WINDOW (mwin), mwin->accel_group);

  mwin->applet = NULL;
  app_prepare_menus(mwin);

  mwin->topvbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (mwin->topvbox);
  gtk_container_add (GTK_CONTAINER (mwin), mwin->topvbox);

  mwin->vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (mwin->vbox);
  gtk_box_pack_start (GTK_BOX (mwin->topvbox), mwin->vbox, TRUE, TRUE, 0);

  mwin->mainwinmenu = gtk_menu_bar_new ();
  gtk_widget_show (mwin->mainwinmenu);

  mwin->menuhandlebox = gtk_handle_box_new ();
  gtk_widget_show (mwin->menuhandlebox);
  gtk_box_pack_start (GTK_BOX (mwin->vbox), mwin->menuhandlebox, FALSE, TRUE, 0);

  gtk_container_add (GTK_CONTAINER (mwin->menuhandlebox), mwin->mainwinmenu);

  gnome_app_fill_menu (GTK_MENU_SHELL (mwin->mainwinmenu), mainwinmenu_uiinfo,
                       NULL, FALSE, 0);

  mwin->cliplistpopupmenu = gtk_menu_new ();
  gnome_app_fill_menu (GTK_MENU_SHELL (mwin->cliplistpopupmenu), cliplistpopupmenu_uiinfo,
                       NULL, FALSE, 0);

  mwin->toolbarhandlebox = gtk_handle_box_new ();
  gtk_widget_show (mwin->toolbarhandlebox);
  gtk_box_pack_start (GTK_BOX (mwin->vbox), mwin->toolbarhandlebox, FALSE, TRUE, 0);


  mwin->toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
  gtk_widget_show (mwin->toolbar);
  gtk_container_add (GTK_CONTAINER (mwin->toolbarhandlebox), mwin->toolbar);

  gtk_container_set_border_width (GTK_CONTAINER (mwin->toolbar), 1);
 
  /* GNOME 2.0 Porting issue (these are removed?) */
  gtk_toolbar_set_button_relief (GTK_TOOLBAR (mwin->toolbar), GTK_RELIEF_NONE);

  mwin->tmp_toolbar_icon = gnome_stock_pixmap_widget (GTK_WIDGET(mwin), GNOME_STOCK_PIXMAP_NEW);
  mwin->toolbar_newitem = gtk_toolbar_append_element (GTK_TOOLBAR (mwin->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("New item"),
                                _("New File"), NULL,
                                mwin->tmp_toolbar_icon, NULL, NULL);
  gtk_widget_show (mwin->toolbar_newitem);

  mwin->tmp_toolbar_icon = gnome_stock_pixmap_widget (GTK_WIDGET(mwin), GNOME_STOCK_PIXMAP_CLOSE);
  mwin->toolbar_deleteitem = gtk_toolbar_append_element (GTK_TOOLBAR (mwin->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Delete items"),
                                _("Delete items"), NULL,
                                mwin->tmp_toolbar_icon, NULL, NULL);
  gtk_widget_show (mwin->toolbar_deleteitem);

  mwin->tmp_toolbar_icon = gnome_stock_pixmap_widget (GTK_WIDGET(mwin), GNOME_STOCK_PIXMAP_SAVE_AS);
  mwin->toolbar_edititem = gtk_toolbar_append_element (GTK_TOOLBAR (mwin->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Edit item"),
                                _("Edit item"), NULL,
                                mwin->tmp_toolbar_icon, NULL, NULL);
  gtk_widget_show (mwin->toolbar_edititem);

  mwin->tmp_toolbar_icon = gnome_stock_pixmap_widget (GTK_WIDGET(mwin), GNOME_STOCK_PIXMAP_CLEAR);
  mwin->toolbar_clear = gtk_toolbar_append_element (GTK_TOOLBAR (mwin->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Clear list"),
                                _("Clear list"), NULL,
                                mwin->tmp_toolbar_icon, NULL, NULL);
  gtk_widget_show (mwin->toolbar_clear);

  mwin->tmp_toolbar_icon = gnome_stock_pixmap_widget (GTK_WIDGET(mwin), GNOME_STOCK_PIXMAP_PREFERENCES);
  mwin->toolbar_preferences = gtk_toolbar_append_element (GTK_TOOLBAR (mwin->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Preferences"),
                                _("Preferences"), NULL,
                                mwin->tmp_toolbar_icon, NULL, NULL);
  gtk_widget_show (mwin->toolbar_preferences);

  mwin->tmp_toolbar_icon = gnome_stock_pixmap_widget (GTK_WIDGET(mwin), GNOME_STOCK_PIXMAP_REFRESH);
  mwin->toolbar_refresh = gtk_toolbar_append_element (GTK_TOOLBAR (mwin->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Get current"),
                                _("Get current selection from windowmanager"), NULL,
                                mwin->tmp_toolbar_icon, NULL, NULL);
  gtk_widget_show (mwin->toolbar_refresh);

  mwin->tmp_toolbar_icon = gnome_stock_pixmap_widget (GTK_WIDGET(mwin), GNOME_STOCK_PIXMAP_EXIT);
  mwin->toolbar_quit = gtk_toolbar_append_element (GTK_TOOLBAR (mwin->toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Exit"),
                                _("Quit application"), NULL,
                                mwin->tmp_toolbar_icon, NULL, NULL);
  gtk_widget_show (mwin->toolbar_quit);

  mwin->scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (mwin->scrolledwindow);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (mwin->scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start (GTK_BOX (mwin->vbox), mwin->scrolledwindow, TRUE, TRUE, 0);


  app_create_cliplist(mwin);
  gtk_container_add (GTK_CONTAINER (mwin->scrolledwindow), mwin->cliplist);
  gtk_signal_connect (GTK_OBJECT (mwin->cliplist), "event",
  				GTK_SIGNAL_FUNC (on_cliplist_event), mwin);

  mwin->appbar = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_NEVER);
  gtk_widget_show (mwin->appbar);
  gtk_box_pack_start (GTK_BOX (mwin->topvbox), mwin->appbar, FALSE, TRUE, 0);
  gnome_appbar_push (GNOME_APPBAR(mwin->appbar), "I don't own the selection");
 
  mwin->prefswin = prefswin_new(GTK_WIDGET(mwin));
  
  gtk_signal_connect (GTK_OBJECT (mwin), "destroy",
                      GTK_SIGNAL_FUNC (mainwin_destroy),
                      mwin);

  gtk_signal_connect (GTK_OBJECT (mwin->toolbar_newitem), "clicked",
                      GTK_SIGNAL_FUNC (on_toolbar_newitem_clicked),
                      mwin);
  gtk_signal_connect (GTK_OBJECT (mwin->toolbar_deleteitem), "clicked",
                      GTK_SIGNAL_FUNC (on_toolbar_deleteitem_clicked),
                      mwin);
  gtk_signal_connect (GTK_OBJECT (mwin->toolbar_edititem), "clicked",
                      GTK_SIGNAL_FUNC (on_toolbar_edititem_clicked),
                      mwin);
  gtk_signal_connect (GTK_OBJECT (mwin->toolbar_clear), "clicked",
                      GTK_SIGNAL_FUNC (on_toolbar_clear_clicked),
                      mwin);
  gtk_signal_connect (GTK_OBJECT (mwin->toolbar_preferences), "clicked",
                      GTK_SIGNAL_FUNC (on_toolbar_preferences_clicked),
                      mwin);
  gtk_signal_connect (GTK_OBJECT (mwin->toolbar_refresh), "clicked",
                      GTK_SIGNAL_FUNC (on_toolbar_refresh_clicked),
                      mwin);
  gtk_signal_connect (GTK_OBJECT (mwin->toolbar_quit), "clicked",
                      GTK_SIGNAL_FUNC (on_toolbar_quit_clicked),
                      mwin);

  gtk_signal_connect (GTK_OBJECT (mwin), "event",
                      GTK_SIGNAL_FUNC (on_mainwin_event),
                      mwin);
  gtk_signal_connect (GTK_OBJECT (mwin->topvbox), "event",
                      GTK_SIGNAL_FUNC (on_mainwin_event),
                      mwin);
  gtk_signal_connect (GTK_OBJECT (mwin->vbox), "event",
                      GTK_SIGNAL_FUNC (on_mainwin_event),
                      mwin);
  gtk_signal_connect (GTK_OBJECT (mwin->scrolledwindow), "event",
                      GTK_SIGNAL_FUNC (on_mainwin_event),
                      mwin);

  gtk_signal_connect (GTK_OBJECT (mwin), "selection_get",
                      GTK_SIGNAL_FUNC (on_mainwin_selection_get),
                      mwin);

  gtk_signal_connect (GTK_OBJECT (mwin), "selection_clear_event",
                      GTK_SIGNAL_FUNC (on_mainwin_selection_lost),
                      mwin);

  gtk_signal_connect (GTK_OBJECT (mwin), "selection_notify_event",
                      GTK_SIGNAL_FUNC (on_mainwin_in_selection_notify_event),
                      mwin);

  gtk_signal_connect (GTK_OBJECT (mwin), "selection_request_event",
                      GTK_SIGNAL_FUNC (on_mainwin_in_selection_request_event),
                      mwin);

  gtk_signal_connect (GTK_OBJECT (mwin), "selection_clear_event",
                      GTK_SIGNAL_FUNC (on_mainwin_in_selection_clear_event),
                      mwin);

  gtk_signal_connect (GTK_OBJECT (mwin), "selection_received",
                      GTK_SIGNAL_FUNC (on_mainwin_selection_received),
                      mwin);
  gtk_window_set_wmclass (GTK_WINDOW (mwin), "mainwin", "gcm");
  
  gtk_widget_add_accelerator (mainwinmenu_file_menu_uiinfo[MAINWINMENU_FILE_OPENITEMFROMDISK].widget, "activate", mwin->accel_group, GDK_O, GDK_CONTROL_MASK, 0);
 
  gtk_widget_add_accelerator (mwin->toolbar_deleteitem, "clicked", mwin->accel_group, GDK_D, 0, 0);  
  gtk_widget_add_accelerator (mwin->toolbar_newitem, "clicked", mwin->accel_group, GDK_N, GDK_CONTROL_MASK, 0);
  gtk_widget_add_accelerator (mwin->toolbar_quit, "clicked", mwin->accel_group, GDK_Q, GDK_CONTROL_MASK, 0);
  
  app_createhintsfor_menus(mwin);
  reload_prefs(mwin);
  //gnome_window_icon_set_from_file (GTK_WINDOW (mwin), PIXMAPPATH"/gcm_icon32x32.png");
  return GTK_WIDGET(mwin);
}



void app_create_cliplist(MainWin *mwin)
{
  mwin->cliplist = gtk_clist_new (CLIST_FIELD_COUNT);
  gtk_widget_show (mwin->cliplist);
  
  gtk_clist_set_selection_mode (GTK_CLIST(mwin->cliplist), GTK_SELECTION_MULTIPLE);

  gtk_clist_set_column_width (GTK_CLIST (mwin->cliplist), CLIST_DATE_FIELD, 60);
  
  gtk_clist_set_column_width (GTK_CLIST (mwin->cliplist), CLIST_TYPE_FIELD, 60);
  gtk_clist_set_column_width (GTK_CLIST (mwin->cliplist), CLIST_FROM_FIELD, 65);

  /* Future: Make this a preference ? */
  /*gtk_clist_set_column_auto_resize (GTK_CLIST (mwin->cliplist), CLIST_FROM_FIELD, TRUE);*/
  gtk_clist_set_column_auto_resize (GTK_CLIST (mwin->cliplist), CLIST_TYPE_FIELD, TRUE);

  gtk_clist_set_column_width (GTK_CLIST (mwin->cliplist), CLIST_DATA_FIELD, 80);
  gtk_clist_column_titles_show (GTK_CLIST (mwin->cliplist));

  mwin->cliplistlabel_time = gtk_label_new (_("Time"));
  gtk_widget_show (mwin->cliplistlabel_time);
  gtk_clist_set_column_widget (GTK_CLIST (mwin->cliplist), CLIST_DATE_FIELD, mwin->cliplistlabel_time);

  mwin->cliplistlabel_type = gtk_label_new (_("Type"));
  gtk_widget_show (mwin->cliplistlabel_type);
  gtk_clist_set_column_widget (GTK_CLIST (mwin->cliplist), CLIST_TYPE_FIELD, mwin->cliplistlabel_type);

  mwin->cliplistlabel_from = gtk_label_new (_("From"));
  gtk_widget_show (mwin->cliplistlabel_from);
  gtk_clist_set_column_widget (GTK_CLIST (mwin->cliplist), CLIST_FROM_FIELD, mwin->cliplistlabel_from);

  mwin->cliplistlabel_data = gtk_label_new (_("Data preview"));
  gtk_widget_show (mwin->cliplistlabel_data);
  gtk_clist_set_column_widget (GTK_CLIST (mwin->cliplist), CLIST_DATA_FIELD, mwin->cliplistlabel_data);

  gtk_signal_connect (GTK_OBJECT (mwin->cliplist), "click_column",
                      GTK_SIGNAL_FUNC (on_cliplist_click_column),
                      mwin);
  gtk_signal_connect (GTK_OBJECT (mwin->cliplist), "select_row",
                      GTK_SIGNAL_FUNC (on_cliplist_select_row),
                      mwin);
  gtk_signal_connect (GTK_OBJECT (mwin->cliplist), "unselect_row",
                      GTK_SIGNAL_FUNC (on_cliplist_unselect_row),
                      mwin);

}



void app_show_prefswin (MainWin *mwin, gpointer *user_data) 
{/* Creates and/or shows the preferences window */
	if (mwin->prefswin==NULL) {
		reload_prefs(mwin);
		mwin->prefswin = prefswin_new(GTK_WIDGET(mwin));
	}
	gtk_widget_show(mwin->prefswin);

}



GtkWidget* app_create_and_show_newitemwin (MainWin *mwin, gpointer *user_data)
{/* Creates and/or shows a newitemwin */
	if (mwin->newitemwin==NULL) {
		mwin->newitemwin = newitemwin_new(GTK_WIDGET(mwin));
	}
	gtk_widget_show(mwin->newitemwin);
	return mwin->newitemwin;
}


GtkWidget* app_create_and_show_textitemwin (MainWin *mwin, gint row)
{/* Creates and/or shows a textitemwin */
	GtkWidget *textitemwin;
	Selection *item;

	item = gtk_clist_get_row_data (GTK_CLIST(mwin->cliplist), row);
	textitemwin = textitemwin_new(GTK_WIDGET(mwin));
	textitemwin_load_item(textitemwin, row, item->time, item->type, item->from, item->data);
	gtk_widget_show(textitemwin);
	return textitemwin;
}


GtkWidget* app_create_and_show_dataitemwin (MainWin *mwin, gint row)
{/* Creates and/or shows a dataitemwin (unused at this moment)*/
	GtkWidget *dataitemwin;
	dataitemwin = dataitemwin_new(GTK_WIDGET(mwin));
	gtk_widget_show(dataitemwin);
	return dataitemwin;
}



GtkWidget* app_create_and_show_aboutwin (MainWin *mwin, gpointer *user_data)
{/* Creates and/or shows an aboutwin */
	GtkWidget *aboutwin;
	aboutwin = aboutwin_new(GTK_WIDGET(mwin));
	gtk_widget_show(aboutwin);
	return aboutwin;
}








//}

//{ Item-related functions
gpointer
get_typed_property_data (Display *xdisplay,
			 Window   xwindow,
			 Atom     property,
			 Atom     requested_type,
			 gint    *size_p,
			 guint    expected_format)
{
  static const guint prop_buffer_lengh = 1024 * 1024;
  unsigned char *prop_data = NULL;
  Atom type_returned = 0;
  unsigned long nitems_return = 0, bytes_after_return = 0;
  int format_returned = 0;
  gpointer data = NULL;
  gboolean abort = FALSE;

  g_return_val_if_fail (size_p != NULL, NULL);
  *size_p = 0;

  gdk_error_trap_push ();

  abort = XGetWindowProperty (xdisplay,
			      xwindow,
			      property,
			      0, prop_buffer_lengh,
			      False,
			      requested_type,
			      &type_returned, &format_returned,
			      &nitems_return,
			      &bytes_after_return,
			      &prop_data) != Success;
  if (gdk_error_trap_pop () ||
      type_returned == None)
    abort++;
  if (!abort &&
      requested_type != AnyPropertyType &&
      requested_type != type_returned)
    {
      /* g_warning (G_GNUC_PRETTY_FUNCTION "(): Property has wrong type, probably on crack"); */
      abort++;
    }
  if (!abort && bytes_after_return)
    {
      g_warning (G_GNUC_PRETTY_FUNCTION "(): Eeek, property has more than %u bytes, stored on harddisk?",
		 prop_buffer_lengh);
      abort++;
    }
  if (!abort && expected_format && expected_format != format_returned)
    {
      g_warning (G_GNUC_PRETTY_FUNCTION "(): Expected format (%u) unmatched (%d), programmer was drunk?",
		 expected_format, format_returned);
      abort++;
    }
  if (!abort && prop_data && nitems_return && format_returned)
    {
      switch (format_returned)
	{
	case 32:
	  *size_p = nitems_return * 4;
	  if (sizeof (gulong) == 8)
	    {
	      guint32 i, *mem = g_malloc0 (*size_p + 1);
	      gulong *prop_longs = (gulong*) prop_data;

	      for (i = 0; i < *size_p / 4; i++)
		mem[i] = prop_longs[i];
	      data = mem;
	    }
	  break;
	case 16:
	  *size_p = nitems_return * 2;
	  break;
	case 8:
	  *size_p = nitems_return;
	  break;
	default:
	  g_warning ("Unknown property data format with %d bits (extraterrestrial?)",
		     format_returned);
	  break;
	}
      if (!data && *size_p)
	{
	  guint8 *mem = NULL;

	  if (format_returned == 8 && type_returned == XA_COMPOUND_TEXT)
	    {
	      gchar **tlist = NULL;
	      gint count = gdk_text_property_to_text_list (type_returned, 8, prop_data,
							   nitems_return, &tlist);

	      if (count && tlist && tlist[0])
		{
		  mem = g_strdup (tlist[0]);
		  *size_p = strlen (mem);
		}
	      if (tlist)
		gdk_free_text_list (tlist);
	    }
	  if (!mem)
	    {
	      mem = g_malloc (*size_p + 1);
	      memcpy (mem, prop_data, *size_p);
	      mem[*size_p] = 0;
	    }
	  data = mem;
	}
    }

  if (prop_data)
    XFree (prop_data);
  
  return data;
}





gchar* app_make_datapreview (MainWin *mwin, gchar *in)
{ /* v2.0: Should return a pointer to "data" not to a gchar 
	 v2.0: Should have the feature to create icons if the data
	 v2.0: is an image or .. html or .. */
	GdkFont* my_font;
	gint width, cwidth, len, sub;
	gchar *text, *thetext;
	gboolean happend=FALSE;
	gint x, y, height, depth;
	GtkWidget *col;
	
	if (mwin->prefs->applet) {
		my_font = gdk_font_load ("6x10");
		text = g_strndup(in, CLIST_MAXLEN_APPLET);
	} else {
		text = g_strndup(in, CLIST_MAXLEN_NORMAL);
		my_font = gdk_font_load ("fixed");
	}

	col = gtk_clist_get_column_widget (GTK_CLIST(mwin->cliplist), CLIST_DATA_FIELD);
	gdk_window_get_geometry( col->window, 
		&x, &y, &cwidth, &height, &depth);

	if (mwin->prefs->applet) {
		cwidth-=CLIST_SUB_APPLET;
		sub=1;
	} else {
		cwidth-=CLIST_SUB_NORMAL;
		sub=3;
	}

	
		width = gdk_string_width (my_font, text);
		len = strlen(text);

		while (width >  cwidth) {
			happend=TRUE;
			if (text!=NULL) g_free(text);
			if (len > sub) len-=sub;
			text = g_strndup(in, len);
			if (len <= sub) goto done;
			width = gdk_string_width (my_font, text);
		}
done:

	if (happend) thetext = g_strdup_printf("%s...", text);
		else thetext = g_strdup(text);

	g_free(text);

	return thetext;
}

Selection *parseSelection(xmlDocPtr doc, xmlNodePtr cur)
{/* This parses an item from an XML-doc */
	/* v2.0: Should store information about the type and target of the item 
	   v2.0: Default is "text" if no such xml-tag is found (backwards compatibel) */
	Selection *item;
	item = (Selection*) malloc(sizeof(Selection));
	memset(item, 0, sizeof(Selection));
	item->type=NULL;
	item->time=NULL;
	item->from=NULL;
	item->data=NULL;
	cur = cur->xmlChildrenNode;
	while (cur != NULL) {
		if (cur->name != NULL) {
			if (!strcmp(cur->name, "time"))
				item->time = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			if (!strcmp(cur->name, "type"))
				item->type = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			if (!strcmp(cur->name, "from"))
				item->from = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			if (!strcmp(cur->name, "data"))
				item->data = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
		}
		/* if <tag/> then we fill the item with emptyness */
		/* Else, for some reason, touching the struct will segfault */
		if (item->time == NULL) {
			time_t timeval=time(0);
			char *tim = malloc (32);
			strftime(tim, 32, "%H:%M:%S", localtime (&timeval));
			item->time = tim;
		}
		if (item->type == NULL) item->type = g_strdup(_("Unknown"));
		if (item->from == NULL) item->from = g_strdup(_("Unknown"));
		if (item->data == NULL) item->data = g_strdup("");

		cur = cur->next;
	}
	return(item);
}




gboolean app_save_items_to_disk(gchar *path, SaveAsData *sa)
{/* Saves selected items to an XML file */
	/*v2.0: Should save type/target information, default is "text" */
	gint i=0;
	gchar *c;
	xmlDocPtr rootnode;
 
	rootnode = xmlNewDoc((xmlChar *) "1.0");
	rootnode->children = xmlNewDocNode(rootnode, NULL, "gcm", NULL);
	while (sa->items) {
		xmlNodePtr itemnode;
		Selection *item;

		i++; c=g_strdup_printf("%d", i);
		item = (Selection*) sa->items->data;
		itemnode = xmlNewChild(rootnode->children, NULL, "item", NULL);
			xmlSetProp(itemnode, "id", c);
			xmlNewChild (itemnode,NULL,"type", item->type);
			xmlNewChild (itemnode,NULL,"time", item->time);
			xmlNewChild (itemnode,NULL,"from", item->from);
			xmlNewChild (itemnode,NULL,"data", xmlEncodeEntitiesReentrant (rootnode, item->data));
		
		sa->items = g_list_next(sa->items);
	}
	g_free(c);
	return xmlSaveFile(path, rootnode);
}



void app_load_items_from_disk(MainWin *mwin, gchar *path)
{/* Loads items from an XML file */
	xmlDocPtr doc;
	xmlNodePtr cur;

#ifdef DEVELOPMENT
	if (mwin->prefs->debug) g_print("GCM: Loading items from disk..");
#endif		
	gtk_clist_freeze (GTK_CLIST(mwin->cliplist));
	doc = xmlParseFile(path);
	if (doc == NULL) {
		gnome_error_dialog(_("Not an XML file !"));
		gtk_clist_thaw (GTK_CLIST(mwin->cliplist));
		return;
	}
	if (doc == NULL) return;
	cur = xmlDocGetRootElement(doc);
	if (cur == NULL) {
		gnome_error_dialog(_("Empty document!"));
		gtk_clist_thaw (GTK_CLIST(mwin->cliplist));
		xmlFreeDoc(doc);
		return;
	}
	if (xmlStrcmp(cur->name, (const xmlChar *) "gcm")) {
		gnome_error_dialog(_("This is not a GCM XML file !"));
		gtk_clist_thaw (GTK_CLIST(mwin->cliplist));
		xmlFreeDoc(doc);
		return;
	}
	cur = cur->xmlChildrenNode;

	while (cur)
	{
		Selection *item=NULL;
		
		item = (Selection*) malloc(sizeof(Selection));
		memset(item, 0, sizeof(Selection));
		item = parseSelection(doc, cur);
		if (item != NULL) app_add_selection(mwin, item);
			else gnome_error_dialog(_("Invalid item in file."));
		cur = cur -> next;
	}
	xmlFreeDoc(doc);
	gtk_clist_thaw (GTK_CLIST(mwin->cliplist));
#ifdef DEVELOPMENT
	if (mwin->prefs->debug) g_print(".done\n");
#endif			
}


void app_load_items_from_mem(MainWin *mwin, gchar *xml, gint size)
{/* Loads items from an XML buffer */
	xmlDocPtr doc;
	xmlNodePtr cur;
#ifdef DEVELOPMENT
	if (mwin->prefs->debug) g_print("GCM: Loading items from mem..");
#endif		
	gtk_clist_freeze (GTK_CLIST(mwin->cliplist));
	doc = xmlParseMemory(xml, size);
	if (doc == NULL) {
		gnome_error_dialog(_("Not an XML stream !"));
		xmlFreeDoc(doc);
		gtk_clist_thaw (GTK_CLIST(mwin->cliplist));
		return;
	}
	if (doc == NULL) return;
	cur = xmlDocGetRootElement(doc);
	if (cur == NULL) {
		gnome_error_dialog(_("Empty document!"));
		xmlFreeDoc(doc);
		gtk_clist_thaw (GTK_CLIST(mwin->cliplist));
		return;
	}
	if (xmlStrcmp(cur->name, (const xmlChar *) "gcm")) {
		gnome_error_dialog(_("This is not a GCM XML stream !"));
		xmlFreeDoc(doc);
		gtk_clist_thaw (GTK_CLIST(mwin->cliplist));
		return;
	}
	cur = cur->xmlChildrenNode;

	while (cur)
	{
		Selection *item=NULL;
		
		item = (Selection*) malloc(sizeof(Selection));
		memset(item, 0, sizeof(Selection));
#ifdef DEVELOPMENT
	if (mwin->prefs->debug) g_print("\n\tGCM: Parsing item..");
#endif		
		item = parseSelection(doc, cur);
#ifdef DEVELOPMENT
	if (mwin->prefs->debug) g_print(".done\n");
#endif
		if (item != NULL) app_add_selection(mwin, item);
			else gnome_error_dialog(_("Invalid item in file."));
		cur = cur -> next;
	}
	xmlFreeDoc(doc);
	gtk_clist_thaw (GTK_CLIST(mwin->cliplist));
#ifdef DEVELOPMENT
	if (mwin->prefs->debug) g_print(".done\n");
#endif		
	
}





void app_claim_multi_selection (MainWin *mwin)
{/* Claims selection when more then one item can be selected */
/* v2.0: What if the user selected items of different types ? 
   v2.0: How are we going to force the user to only select items
   v2.0: of the same type? */
		GList *selected;
		Selection *item, *ritem;
		gboolean first=TRUE;

		ritem=(Selection*) g_malloc0(sizeof(Selection));
		selected = GTK_CLIST(mwin->cliplist)->selection;   
		if (selected) {
			while (selected) {
				Selection *nitem;
				if (first) {
					item = gtk_clist_get_row_data(GTK_CLIST(mwin->cliplist),(gint) selected->data);
					ritem->time = g_strdup(item->time);
					ritem->type = g_strdup(item->type);
					ritem->from = g_strdup(item->from);
					ritem->data = g_strdup(item->data);
					ritem->row = item->row;
					first = FALSE;
				} else {
					gchar *text;
					nitem = gtk_clist_get_row_data(GTK_CLIST(mwin->cliplist),(gint) selected->data);
					text = g_strdup_printf("%s%s", ritem->data, nitem->data);
					g_free(ritem->data);
					ritem->data = text;
				}
				selected = g_list_next(selected);
			}
			ritem->target = GDK_SELECTION_TYPE_STRING;
			app_claim_text_selection (mwin, ritem);
			selection_free(ritem);
			/* free buffers */
		} else {
			return;
		}
}

void app_claim_text_selection (MainWin *mwin, Selection *item)
{/* Claims selection for one item */
	/* v2.0: Should become : app_claim_selection. Should support multiple itemtypes
	   v2.0: should not use mwin->clipboard */
	if (item) {
		mwin->currentitem = item;
		if (mwin->clipboard) g_free(mwin->clipboard);
		mwin->clipboard = g_strdup(item->data);
		mwin->clipboardtarget = item->target;
		if (gtk_selection_owner_set (GTK_WIDGET(mwin), mwin->selection, GDK_CURRENT_TIME))
			gnome_appbar_push (GNOME_APPBAR(mwin->appbar), _("I own the selection"));
		else
			gnome_appbar_push (GNOME_APPBAR(mwin->appbar), _("Claiming selection failed"));
	} else {
		gnome_appbar_push (GNOME_APPBAR(mwin->appbar), _("Claiming selection failed"));
#ifdef DEVELOPMENT
		if (mwin->prefs->debug) g_print("GNOME Clipboard Manager ERROR: Item empty, not claiming selection\n");
#endif
	}
}


void app_get_new_item (MainWin *mwin, gpointer *user_data) 
{/* Gets a new item; The callback 'on_mainwin_selection_received' will recive it */
  static GdkAtom targets_atom = GDK_NONE;
  gboolean convert_successfull = FALSE;
  mwin->lastnewrow = -1;
  mwin->lastnewitem = NULL;
  targets_atom = mwin->target_atom;
  do
  {  
    convert_successfull = gtk_selection_convert (GTK_WIDGET(mwin), mwin->selection, targets_atom, GDK_CURRENT_TIME);
    /* Let Gtk process all the pending events before continuing. */
    while(gtk_events_pending())
    {
       gtk_main_iteration();
    }
  } while(!convert_successfull);

}


void app_delete_selected_items(MainWin *mwin, GtkCList *cliplist, gpointer *usr_data)
{/* Deletes the selected items (Warning: recursive function call) */
	GList *selected;
	gint row;

	selected = cliplist->selection;
	
	if (!(selected)){
		return;
	} else {
		selected = g_list_first(selected);
		row = (gint) selected->data;
		
		gtk_clist_remove(cliplist, row);
		/*recursive function call*/
		app_delete_selected_items(mwin, cliplist, usr_data);
		if (mwin->prefs->multisel==FALSE) gtk_clist_select_row(GTK_CLIST(mwin->cliplist), row, -1);
	}
}


gint app_add_anytype_selection  (MainWin *mwin, GdkAtom target, gchar *type, gchar *from, gchar *data) {
/* v2.0 data should not be a pointer to gchar but to any memory */
	Selection *item;
	time_t timeval=time(0);
	char *tim = malloc (32);	

	strftime(tim, 32, "%H:%M:%S", localtime (&timeval));
	item = g_malloc0(sizeof(Selection));
	  item->time = tim;
	  item->from = g_strdup(from);
	  item->data = g_strdup(data);
	  item->type = g_strdup(type);
	  item->target = target;
	return app_add_selection(mwin, item);

}

gint app_add_text_selection (MainWin *mwin, gchar *type, gchar *from, gchar *data)
{/* Add a new item with "From" and "Data" */
	Selection *item;
	time_t timeval=time(0);
	char *tim = malloc (32);	

	strftime(tim, 32, "%H:%M:%S", localtime (&timeval));
	item = g_malloc0(sizeof(Selection));
	  item->time = tim;
	  item->from = g_strdup(from);
	  item->data = g_strdup(data);
	  item->type = g_strdup(type);
	  item->target = GDK_SELECTION_TYPE_STRING;
	return app_add_selection(mwin, item);
}


gint app_add_selection (MainWin *mwin, Selection *item)
{/* Add a new selection using a prefilled in item */
	/* v2.0: Should support images if the item->type is an image 
	   v2.0: So a preview of the image should go in the GtkCList */
	gchar *text[CLIST_FIELD_COUNT];
	gint row, i;
	GtkAdjustment *adj;

	item->owning_window = mwin->last_owning_window;
	item->owning_app = mwin->new_owner;
	if (mwin->prefs->applet) {
		adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW
						     (mwin->appletscroll));
	} else {
		adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW
						     (mwin->scrolledwindow));
	}

	if (item != NULL) {
#ifdef DEVELOPMENT
	if (mwin->prefs->debug) g_print("GCM: Adding item..");
#endif		
		if (item->time != NULL) text[CLIST_DATE_FIELD] = g_strdup(item->time);
			else {
				time_t timeval=time(0);
				char *tim = malloc (32);
				strftime(tim, 32, "%H:%M:%S", localtime (&timeval));
				text[CLIST_DATE_FIELD] = tim;
			}
		if (item->type != NULL) text[CLIST_TYPE_FIELD] = g_strdup(item->type);
			else return -1;

		if (item->data != NULL) {
				text[CLIST_DATA_FIELD] = g_strdup(app_make_datapreview(mwin, item->data));
		} else return -1;

		if (item->from != NULL) text[CLIST_FROM_FIELD] = g_strdup(item->from);
			else return -1;
		row = gtk_clist_append(GTK_CLIST (mwin->cliplist), text);
		if (mwin->prefs->beep_on_collect) gdk_beep();
		gtk_clist_set_row_data_full (GTK_CLIST(mwin->cliplist), row, item, 
			(GtkDestroyNotify)cliplist_row_removed);
		mwin->lastnewitem = item;

		if ((mwin->prefs->multisel)&&(mwin->prefs->unselall)) 
			gtk_clist_unselect_all (GTK_CLIST(mwin->cliplist));
		if (mwin->prefs->selectnew) 
			gtk_clist_select_row(GTK_CLIST(mwin->cliplist), row, 1);
		if (mwin->prefs->followmode) 
			gtk_adjustment_set_value (adj, adj->upper - adj->page_size);
		for (i=0; i<CLIST_FIELD_COUNT; i++) g_free(text[i]);
	
		if (mwin->prefs->usemaxitems) {
			/* Only valid when appending*/
			while (GTK_CLIST(mwin->cliplist)->rows > mwin->prefs->maxitems)
				gtk_clist_remove(GTK_CLIST(mwin->cliplist), 0);
		}
		/*free buffers*/

#ifdef DEVELOPMENT
	if (mwin->prefs->debug) g_print(".done\n");
#endif			
		return row;
	} else {
		g_print("GCM ERROR: Item containted no data!\n");
		return 0;
	}
}


void app_update_text_selection (MainWin *mwin, gint row, gchar *type, gchar *from, gchar *data)
{/* Updates a row with "From" and "Data" */
	gtk_clist_remove(GTK_CLIST(mwin->cliplist), row);
	app_add_text_selection(mwin, type, from, data);
}




void app_move_selected_up(MainWin *mwin, GList *remaining, gint startrow)
{/* Moves selected rows one up */
	if (mwin->prefs->multisel) {
		remaining = g_list_first(remaining);
			/* FIXME: move all up */
		gtk_clist_row_move (GTK_CLIST(mwin->cliplist), (gint) remaining->data, (gint) remaining->data-1);
	} else {
		remaining = g_list_first(remaining);
		gtk_clist_row_move (GTK_CLIST(mwin->cliplist), (gint) remaining->data, (gint) remaining->data-1);
	}
	
}


void app_move_selected_down(MainWin *mwin, GList *remaining, gint startrow)
{/* Moves selected rows one down */
	if (mwin->prefs->multisel) {
		remaining = g_list_first(remaining);
			/* FIXME: move all down */
		gtk_clist_row_move (GTK_CLIST(mwin->cliplist), (gint) remaining->data, (gint) remaining->data+1);
	} else {
		remaining = g_list_first(remaining);
		gtk_clist_row_move (GTK_CLIST(mwin->cliplist), (gint) remaining->data, (gint) remaining->data+1);
	}
	
}




//}

//{ Mainwin Callbacks

void mainwin_show(GtkWidget *widget, MainWin *mwin)
{ /* On mainwin show (unused at this moment) */
}
void mainwin_hide(GtkWidget *widget, MainWin *mwin)
{ /* On mainwin hode (unused at this moment)*/
}



int /* Any event on the mainwin */
on_mainwin_event                       (GtkWidget       *widget,
										GdkEvent        *event,
										MainWin         *mwin)
{
	GdkEventButton *button_event = (GdkEventButton *) event;
	app_update_menus(mwin);
	if (event->type == GDK_BUTTON_PRESS && button_event->button == 3) {
		gtk_menu_popup (GTK_MENU(mwin->cliplistpopupmenu), NULL, NULL, NULL, NULL, 3, 
						button_event->time);
		return TRUE;
	}
	return FALSE;
}


//}

//{ Selections related CallBacks
void /* When gcm recived a selection (after we asked for it of course) */
on_mainwin_selection_received          (GtkWidget       *widget,
										GtkSelectionData *data,
										guint            time,
										MainWin          *mwin)
{
#ifdef DEVELOPMENT
	if (mwin->prefs->debug) g_print("GCM: Selection received\n");
#endif
	if (data->data) {
		gchar *type;
/* v2.0: Should support images
   v2.0: Should support more types (OpenOffice.org style) */

		if (mwin->new_owner_title==NULL) mwin->new_owner_title=g_strdup("Unknown");
		if (data->type == XA_COMPOUND_TEXT) {
				type=g_strdup("text");
				mwin->lastnewrow = app_add_text_selection(mwin, type ,mwin->new_owner_title, data->data);
		} else if (
					(data->type == gdk_atom_intern ("ATOM", FALSE)) ||
					(data->type == gdk_atom_intern ("TARGETS", FALSE))
				  ) {
					GdkAtom *atoms_targets = NULL;
					gint counter;
					GString *mdata;

					mdata = g_string_new("");
					type = g_strdup(gdk_atom_name(data->type));
					atoms_targets = (GdkAtom *)data->data;
					for(counter = 0; counter < (data->length / sizeof(GdkAtom)); counter++)
					{
						mdata = g_string_append(mdata, gdk_atom_name(atoms_targets[counter]));
						mdata = g_string_append_c(mdata, '\n');
					}
					g_strdown(type);
					mwin->lastnewrow = app_add_text_selection(mwin, type ,mwin->new_owner_title, mdata->str);
					//mwin->lastnewrow = app_add_anytype_selection(mwin, data->type ,type ,mwin->new_owner_title, mdata->str);
					g_string_free(mdata, TRUE);
		} else {
				gint realen = strlen(data->data);

				type = g_strdup(gdk_atom_name(data->type));
				g_strdown(type);
				/* Some applications (OpenOffice.org) give normal strings back; others (Mozilla) give ucs2 strings.*/
				if ((realen+1 != data->length)&&(realen-1 != data->length)&&(realen != data->length)){
					mwin->lastnewrow = app_add_anytype_selection(mwin, data->type ,type ,mwin->new_owner_title, ucs2tostr(data->data, data->length));
				} else {
					mwin->lastnewrow = app_add_anytype_selection(mwin, data->type ,type ,mwin->new_owner_title, data->data);
				}
				
				
				
		}
		g_free(type);
	}
	if ((mwin->lastnewrow > 0) && (GTK_CLIST(mwin->cliplist)->rows >= mwin->lastnewrow))
		gtk_clist_select_row(GTK_CLIST(mwin->cliplist), mwin->lastnewrow, 1);

}


gboolean /* Unused at this moment */
on_mainwin_in_selection_clear_event    (GtkWidget       *widget,
                                        GdkEventSelection *event,
                                        MainWin         *mwin)
{

#ifdef DEVELOPMENT
	if (mwin->prefs->debug) g_print("GCM: Selection clear\n");
#endif
	app_get_new_item (mwin, NULL);
	return FALSE;
}



gboolean /* Unused at this moment */
on_mainwin_in_selection_notify_event   (GtkWidget       *widget,
                                        GdkEventSelection *event,
                                        MainWin         *mwin)
{
#ifdef DEVELOPMENT
	if (mwin->prefs->debug) g_print("GCM: Selection notify\n");
#endif
	return FALSE;
}



gboolean /* Unused at this moment */
on_mainwin_in_selection_request_event  (GtkWidget       *widget,
                                        GdkEventSelection *event,
                                        MainWin         *mwin)
{ 
#ifdef DEVELOPMENT
	if (mwin->prefs->debug) g_print("GCM: Selection request\n");
#endif
	return TRUE;
}


gboolean /* When gcm lost it's selection (somebody else took it) -we recollect if that option is set- */
on_mainwin_selection_lost              (GtkWidget         *widget,
                                        GdkEventSelection *event,
                                        MainWin           *mwin)
{
	Window sel_owner;
	Window root, parent;
	Window *children = NULL;
	guint nchildren;
	gchar *name;
	gint size;
	gboolean found=FALSE;

	sel_owner = XGetSelectionOwner(gdk_display, event->selection);
	if (sel_owner != BadWindow) {
		mwin->last_owning_window = sel_owner;
		XQueryTree (gdk_display, sel_owner, &root, &parent, &children, &nchildren);

		while ((parent) && (parent != BadWindow)) {
			if (parent != BadWindow) {
				XQueryTree (gdk_display, parent,&root, &parent, &children, &nchildren);
				name = get_typed_property_data (gdk_display, parent, XA_WM_NAME, XA_STRING, &size, 8);
				if (name) {
					found=TRUE;
					mwin->new_owner = parent;
					mwin->new_owner_title = name;
					break;
				}
			}
		}
	}
	if (found==FALSE) {
		if (sel_owner != BadWindow) {
			name = get_typed_property_data (gdk_display, sel_owner, XA_WM_NAME, XA_STRING, &size, 8);
		}
		mwin->new_owner = sel_owner;
		if (name)
			mwin->new_owner_title = name;
		else mwin->new_owner_title = g_strdup("Unknown");
	}

#ifdef DEVELOPMENT
	if (mwin->prefs->debug) g_print("GCM: Selection lost\n");
#endif
	gnome_appbar_push (GNOME_APPBAR(mwin->appbar), _("I don't own the selection"));
	/*if (mwin->clipboard) g_free(mwin->clipboard); segfaults? :(*/
	if (mwin->prefs->auto_collect==TRUE )
		app_get_new_item (mwin, (gpointer*) widget);
	return TRUE;
}


/* When an application asks for "our" selection. The selection that we are holding must be returned */
void on_mainwin_selection_get          (GtkWidget        *widget, 
                                        GtkSelectionData *selection_data,
                                        guint             info,
                                        guint             time_stamp,
                                        MainWin           *mwin)
{
#ifdef DEVELOPMENT
	if (mwin->prefs->debug) g_print("GCM: Selection get request\n");
#endif
	if (mwin->clipboard) {
		gtk_selection_data_set (selection_data, mwin->clipboardtarget,
								8, mwin->clipboard, strlen(mwin->clipboard));
	}
}


//}


//{ MainWinMenu CallBacks
void /* Create a new newitemwin */
on_mainwinmenu_newitem_activate        (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	app_create_and_show_newitemwin(mwin, (gpointer*) menuitem);
}

void /* Delete item */
on_mainwinmenu_deleteitem_activate     (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	gtk_clist_freeze (GTK_CLIST(mwin->cliplist));
	app_delete_selected_items (mwin, GTK_CLIST(mwin->cliplist), (gpointer*) menuitem);
	gtk_clist_thaw (GTK_CLIST(mwin->cliplist));
}







void /* Move all selected rows one up */
on_mainwinmenu_moveup_activate         (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	GList *remaining;
	remaining = g_list_copy(GTK_CLIST(mwin->cliplist)->selection);
	if (remaining!=NULL) {
		gtk_clist_freeze (GTK_CLIST(mwin->cliplist));
		app_move_selected_up (mwin, remaining, -1);
		g_list_free(remaining);
		gtk_clist_thaw (GTK_CLIST(mwin->cliplist));
	}
}



void
on_mainwinmenu_applet_activate         (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	mwin->applet = create_applet (mwin);
	gtk_clist_column_titles_hide (GTK_CLIST(mwin->cliplist));
	gtk_clist_set_column_visibility (GTK_CLIST(mwin->cliplist), CLIST_DATE_FIELD, FALSE);
	gtk_clist_set_column_visibility (GTK_CLIST(mwin->cliplist), CLIST_FROM_FIELD, FALSE);
	gtk_clist_set_column_visibility (GTK_CLIST(mwin->cliplist), CLIST_TYPE_FIELD, FALSE);
}

void /* Move all selected rows one down */
on_mainwinmenu_movedown_activate         (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	GList *remaining;
	remaining = g_list_copy(GTK_CLIST(mwin->cliplist)->selection);
	if (remaining!=NULL) {
		gtk_clist_freeze (GTK_CLIST(mwin->cliplist));
		app_move_selected_down (mwin, remaining, -1);
		g_list_free(remaining);
		gtk_clist_thaw (GTK_CLIST(mwin->cliplist));
	}
}



void /* Callback from the overwrite questiondialog */
on_mainwin_overwrite_yes_no             (gint answer, SaveAsData *sa)
{
	if (answer == 0)
		if (!app_save_items_to_disk(sa->path, sa)) {
			gnome_error_dialog(_("IO error while trying to save file"));
		}
	saveasdata_free(sa);
}


void /* OK callback */
on_mainwin_saveallfilelist_ok_button_clicked
                                       (GtkButton *button,
                                        MainWin *mwin)
{
	gchar *msg;
	gint row;
	GtkWidget *question;
	FILE *f;
	SaveAsData *sa;

	sa = g_malloc0(sizeof(SaveAsData));
	sa->items = NULL;

	for (row=0; row< GTK_CLIST(mwin->cliplist)->rows; row++) {
		sa->items = g_list_append(sa->items, gtk_clist_get_row_data (GTK_CLIST(mwin->cliplist), row));
	}
	
	sa->path = g_strdup(gnome_filelist_get_filename(GNOME_FILELIST(mwin->savefilelist)));
	gtk_widget_destroy(mwin->savefilelist);
	if (f=fopen(sa->path,"r")) {
		fclose(f);
		msg = g_strdup_printf("A file named ''%s'' already exists.\n"
							  "Do you want to replace it with the "
							  "one you are saving?", sa->path);
		question = gnome_question_dialog_modal
				(_(msg), GTK_SIGNAL_FUNC (on_mainwin_overwrite_yes_no), sa);
		gtk_widget_show(question);
	} else {
		if (app_save_items_to_disk(sa->path, sa)==-1) {
			gnome_error_dialog(_("IO error while trying to save file"));
		}
		saveasdata_free(sa);
	}
	mwin->savefilelist_inuse=FALSE;
	app_update_menus(mwin);
}

void /* OK Callback from the gnomefilelist (save) */
on_mainwin_savefilelist_ok_button_clicked
                                       (GtkButton *button,
                                        MainWin *mwin)
{
	gchar *msg;
	gint row;
	GList *selected;
	GtkWidget *question;
	FILE *f;
	SaveAsData *sa;

	sa = g_malloc0(sizeof(SaveAsData));
	sa->items = NULL;

	selected = GTK_CLIST(mwin->cliplist)->selection;
	if (selected) {
		while (selected) {
			row = (gint) selected->data;
			sa->items = g_list_append(sa->items, gtk_clist_get_row_data (GTK_CLIST(mwin->cliplist), row));
			selected = g_list_next(selected);
		}
	} else { 
		gnome_error_dialog(_("No item selected"));
		mwin->savefilelist_inuse=FALSE;
		app_update_menus(mwin);
		return;
	}
	sa->path = g_strdup(gnome_filelist_get_filename(GNOME_FILELIST(mwin->savefilelist)));
	gtk_widget_destroy(mwin->savefilelist);
	if (f=fopen(sa->path,"r")) {
		fclose(f);
		msg = g_strdup_printf("A file named ''%s'' already exists.\n"
							  "Do you want to replace it with the "
							  "one you are saving?", sa->path);
		question = gnome_question_dialog_modal
				(_(msg), GTK_SIGNAL_FUNC (on_mainwin_overwrite_yes_no), sa);
		gtk_widget_show(question);
	} else {
		if (app_save_items_to_disk(sa->path, sa)==-1) {
			gnome_error_dialog(_("IO error while trying to save file"));
		}
		saveasdata_free(sa);
	}
	mwin->savefilelist_inuse=FALSE;
	app_update_menus(mwin);
}

void /* CANCEL Callback from the gnomefilelist (save) */
on_mainwin_savefilelist_cancel_button_clicked
                                       (GtkButton *button,
                                        MainWin *mwin)
{
	mwin->savefilelist_inuse=FALSE;
	app_update_menus(mwin);
	gtk_widget_destroy(mwin->savefilelist);
}


void /* Save selected items to disk */
on_mainwinmenu_file_savetodisk_activate
                                       (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	GList *selected;
	GList *colist=NULL;
	GList *combolist=NULL;

	selected = GTK_CLIST(mwin->cliplist)->selection;
	if (selected) {
		selected = g_list_first (selected);
		if (selected == NULL) {
			mwin->savefilelist_inuse=FALSE;
			app_update_menus(mwin);			
			gnome_error_dialog(_("No item selected"));
			return;
		}
		mwin->savefilelist_inuse=TRUE;
		app_update_menus(mwin);
		mwin->savefilelist = gnome_filelist_new();
		colist = gnome_filelisttype_addtype_f(colist, _("XML file"), ".xml", NULL);
		colist = gnome_filelisttype_addtype(colist, _("All files"), NULL);
		gnome_filelisttype_clearfiletypes(GNOME_FILELIST(mwin->savefilelist));
		GNOME_FILELIST(mwin->savefilelist)->filetypes = colist;
		combolist = gnome_filelisttype_getcombolist(GNOME_FILELIST(mwin->savefilelist)->filetypes);   
		gnome_filelist_set_combolist(GNOME_FILELIST(mwin->savefilelist), combolist);
		mwin->savefilelist_ok_button = GNOME_FILELIST(mwin->savefilelist)->ok_button;
		mwin->savefilelist_cancel_button = GNOME_FILELIST(mwin->savefilelist)->cancel_button;
		gtk_widget_show(mwin->savefilelist);

		gtk_signal_connect (GTK_OBJECT (mwin->savefilelist), "destroy",
				  GTK_SIGNAL_FUNC (on_mainwin_savefilelist_cancel_button_clicked),
				  mwin);

		
		gtk_signal_connect (GTK_OBJECT (mwin->savefilelist_ok_button), "clicked",
						  GTK_SIGNAL_FUNC (on_mainwin_savefilelist_ok_button_clicked),
						  mwin);
		gtk_signal_connect (GTK_OBJECT (mwin->savefilelist_cancel_button), "clicked",
						  GTK_SIGNAL_FUNC (on_mainwin_savefilelist_cancel_button_clicked),
						  mwin);
	} else {
		gnome_error_dialog(_("No item selected"));
	}
}

void /* Save all items to disk */
on_mainwinmenu_file_savealltodisk_activate
                                        (GtkMenuItem    *menuitem,
                                        MainWin         *mwin)
{
	GList *colist=NULL;
	GList *combolist=NULL;

	mwin->savefilelist_inuse=TRUE;
	app_update_menus(mwin);
	mwin->savefilelist = gnome_filelist_new();
	colist = gnome_filelisttype_addtype_f(colist, _("XML file"), ".xml", NULL);
	colist = gnome_filelisttype_addtype(colist, _("All files"), NULL);
	gnome_filelisttype_clearfiletypes(GNOME_FILELIST(mwin->savefilelist));
	GNOME_FILELIST(mwin->savefilelist)->filetypes = colist;
	combolist = gnome_filelisttype_getcombolist(GNOME_FILELIST(mwin->savefilelist)->filetypes);   
	gnome_filelist_set_combolist(GNOME_FILELIST(mwin->savefilelist), combolist);
	mwin->savefilelist_ok_button = GNOME_FILELIST(mwin->savefilelist)->ok_button;
	mwin->savefilelist_cancel_button = GNOME_FILELIST(mwin->savefilelist)->cancel_button;
	gtk_widget_show(mwin->savefilelist);
	
	gtk_signal_connect (GTK_OBJECT (mwin->savefilelist), "destroy",
					  GTK_SIGNAL_FUNC (on_mainwin_savefilelist_cancel_button_clicked),
					  mwin);
	
	gtk_signal_connect (GTK_OBJECT (mwin->savefilelist_ok_button), "clicked",
					  GTK_SIGNAL_FUNC (on_mainwin_saveallfilelist_ok_button_clicked),
					  mwin);
	gtk_signal_connect (GTK_OBJECT (mwin->savefilelist_cancel_button), "clicked",
					  GTK_SIGNAL_FUNC (on_mainwin_savefilelist_cancel_button_clicked),
					  mwin);
}



void /* OK callback from the gneomfilelist (open) Loads items from disk */
on_mainwin_openfilelist_ok_button_clicked
                                       (GtkButton *button,
                                        MainWin *mwin)
{
	gchar *path;

	path = g_strdup(gnome_filelist_get_filename(GNOME_FILELIST(mwin->openfilelist)));
	mwin->openfilelist_inuse=FALSE;
	app_update_menus(mwin);
	gtk_widget_destroy(mwin->openfilelist);
	app_load_items_from_disk(mwin, path);
	g_free(path);
}

void /* CANCEL callback from the gnomefilelist (open) */
on_mainwin_openfilelist_cancel_button_clicked
                                       (GtkButton *button,
                                        MainWin *mwin)
{
	mwin->openfilelist_inuse=FALSE;
	app_update_menus(mwin);
	gtk_widget_destroy(mwin->openfilelist);
}


void /* Load items from disk */
on_mainwinmenu_file_openfromdisk_activate
                                       (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	GList *colist=NULL;
	GList *combolist=NULL;

	mwin->openfilelist_inuse=TRUE;
	app_update_menus(mwin);
	mwin->openfilelist = gnome_filelist_new();
	gtk_widget_show(mwin->openfilelist);
	mwin->openfilelist_ok_button = GNOME_FILELIST(mwin->openfilelist)->ok_button;
	mwin->openfilelist_cancel_button = GNOME_FILELIST(mwin->openfilelist)->cancel_button;
	colist = gnome_filelisttype_addtype_f(colist, _("XML file"), ".xml", NULL);
	colist = gnome_filelisttype_addtype(colist, _("All files"), NULL);
	gnome_filelisttype_clearfiletypes(GNOME_FILELIST(mwin->openfilelist));
	GNOME_FILELIST(mwin->openfilelist)->filetypes = colist;
	combolist = gnome_filelisttype_getcombolist(GNOME_FILELIST(mwin->openfilelist)->filetypes);   
	gnome_filelist_set_combolist(GNOME_FILELIST(mwin->openfilelist), combolist);
	gtk_signal_connect (GTK_OBJECT (mwin->openfilelist_ok_button), "clicked",
					  GTK_SIGNAL_FUNC (on_mainwin_openfilelist_ok_button_clicked),
					  mwin);
	gtk_signal_connect (GTK_OBJECT (mwin->openfilelist_cancel_button), "clicked",
					  GTK_SIGNAL_FUNC (on_mainwin_openfilelist_cancel_button_clicked),
					  mwin);
}






void /* Exit button */
on_mainwinmenu_exit_activate           (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	gtk_exit(0);
}


void /* Cut selected items (copy and delete) */
on_mainwinmenu_cut_activate            (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	on_mainwinmenu_copy_activate (menuitem, mwin);
	gtk_clist_freeze (GTK_CLIST(mwin->cliplist));
	app_delete_selected_items(mwin, GTK_CLIST(mwin->cliplist), (gpointer*) menuitem);
	gtk_clist_thaw (GTK_CLIST(mwin->cliplist));
}


void /* Copy selected items (copy to gcm's copypaste memory and claim selectionownership */
on_mainwinmenu_copy_activate           (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	/* Function one : copypasteing in GCM */
	GList *selected;
	selected = GTK_CLIST(mwin->cliplist)->selection;
	if (mwin->copypaste!=NULL) g_list_free(mwin->copypaste);
	mwin->copypaste = NULL;
	if (selected) {
		while (selected) {
			Selection *nitem, *mitem;
			nitem=(Selection*) g_malloc0(sizeof(Selection));
			mitem = gtk_clist_get_row_data(GTK_CLIST(mwin->cliplist),(gint) selected->data);
			nitem->time = g_strdup(mitem->time);
			nitem->type = g_strdup(mitem->type);
			nitem->from = g_strdup(mitem->from);
			nitem->data = g_strdup(mitem->data);
			nitem->row = mitem->row;
			mwin->copypaste = g_list_append(mwin->copypaste, nitem);
			selected = g_list_next(selected);
		}
	}
	/* Function two : refresh the selectionownership */
	if (mwin->prefs->multisel)
		app_claim_multi_selection (mwin);
	else {
		Selection *item;
		GList *selected;

		selected = GTK_CLIST(mwin->cliplist)->selection;   
		if (selected) {
			item = gtk_clist_get_row_data(GTK_CLIST(mwin->cliplist),(gint) selected->data);
		} else {
			gnome_error_dialog("No item selected");
			return;
		}
		app_claim_text_selection(mwin, item);
	}
}


void /* Get items from gcm's copypaste memory */
on_mainwinmenu_paste_activate          (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	GList *selected = g_list_copy(mwin->copypaste);

	if (selected) {
		gtk_clist_freeze (GTK_CLIST(mwin->cliplist));
		while (selected) {
			Selection *nitem, *item;
			nitem=(Selection*) g_malloc0(sizeof(Selection));
			item = (Selection*) selected->data;
			nitem->time = g_strdup(item->time);
			nitem->type = g_strdup(item->type);
			nitem->from = g_strdup(item->from);
			nitem->data = g_strdup(item->data);
			nitem->row = item->row;
			app_add_selection(mwin, nitem);
			/*selection_free(nitem); question? */
			selected = g_list_next(selected);
		}
		gtk_clist_thaw (GTK_CLIST(mwin->cliplist));
	}
}


void /* Clear all items */
on_mainwinmenu_clear_activate          (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	gtk_clist_freeze (GTK_CLIST(mwin->cliplist));
	gtk_clist_clear (GTK_CLIST(mwin->cliplist));
	gtk_clist_thaw (GTK_CLIST(mwin->cliplist));
}



void /* Select all items */
on_mainwinmenu_selectall_activate     (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	gtk_clist_freeze (GTK_CLIST(mwin->cliplist));
	gtk_clist_select_all (GTK_CLIST(mwin->cliplist));
	gtk_clist_thaw (GTK_CLIST(mwin->cliplist));
}

void /* Select none */
on_mainwinmenu_selectnone_activate     (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	gtk_clist_freeze (GTK_CLIST(mwin->cliplist));
	gtk_clist_unselect_all (GTK_CLIST(mwin->cliplist));
	gtk_clist_thaw (GTK_CLIST(mwin->cliplist));
}

void /* Create/show preferences window */
on_mainwinmenu_preferences_activate    (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	app_show_prefswin (mwin, (gpointer*) menuitem);
}


void /* Create/show about window */
on_mainwinmenu_about_activate          (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	app_create_and_show_aboutwin (mwin, (gpointer*) menuitem);
}


//}


//{ CliplistPopupMenu CallBacks
void /* Create New item (popupmenu) */
on_cliplistpopupmenu_new_activate      (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	app_create_and_show_newitemwin (mwin, (gpointer*) menuitem);
}


void /* Cut selected items (popumenu) */
on_cliplistpopupmenu_cut_activate      (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	on_mainwinmenu_cut_activate (NULL, mwin);
}

void /* Copy selected items (popupmenu) */
on_cliplistpopupmenu_copy_activate     (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	on_mainwinmenu_copy_activate (NULL, mwin);
}

void /* Paste selected items (popupmenu) */
on_cliplistpopupmenu_paste_activate    (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	on_mainwinmenu_paste_activate (NULL, mwin);
}

void /* Edit selected item(s) (popupmenu) */
on_cliplistpopupmenu_edit_activate     (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	GList *selected;
	gint row;

	selected = GTK_CLIST(mwin->cliplist)->selection;
	if (selected) {
		selected = g_list_first (selected);
		if (selected == NULL) {
			gnome_error_dialog(_("No item selected"));
			return;
		}
	row = (gint) selected->data;

	app_create_and_show_textitemwin (mwin, row);
	} else gnome_error_dialog(_("No item selected"));
}



void /* Convert the item to collection of items by parsing for xml */
on_cliplistpopupmenu_convert_activate  (GtkMenuItem     *menuitem,
                                        MainWin         *mwin)
{
	GList *selected;

	selected = GTK_CLIST(mwin->cliplist)->selection;
	if (selected) {
		while (selected) {
			gchar *xml;
			Selection *item;

			item = gtk_clist_get_row_data(GTK_CLIST(mwin->cliplist),(gint) selected->data);
			xml = g_strdup(item->data);
			app_load_items_from_mem(mwin, xml, strlen(xml));
			g_free(xml);
			selected = g_list_next(selected);
		}
	}
}

//}


//{ Toolbar CallBacks
void /* Create new item (toolbar) */
on_toolbar_newitem_clicked             (GtkButton       *button,
                                        MainWin         *mwin)
{
	app_create_and_show_newitemwin (mwin, (gpointer*) button);
}


void /* Clear shelf (toolbar) */
on_toolbar_clear_clicked              (GtkButton       *button,
                                        MainWin         *mwin)
{
	on_mainwinmenu_clear_activate(NULL, mwin);
}

void /* Delete selected items (toolbar) */
on_toolbar_deleteitem_clicked          (GtkButton       *button,
                                        MainWin         *mwin)
{
	gtk_clist_freeze (GTK_CLIST(mwin->cliplist));
	app_delete_selected_items(mwin, GTK_CLIST(mwin->cliplist), (gpointer*) button);
	gtk_clist_thaw (GTK_CLIST(mwin->cliplist));
}

void /* Edit selected item(s) (toolbar) */
on_toolbar_edititem_clicked            (GtkButton       *button,
                                        MainWin         *mwin)
{
	on_cliplistpopupmenu_edit_activate (NULL, mwin);
}


void /* Create/show preferences window (toolbar) */
on_toolbar_preferences_clicked         (GtkButton       *button,
                                        MainWin         *mwin)
{
	app_show_prefswin (mwin, (gpointer*) button);
}


void /* Refresh = get current = get new item (bad naming) (toobar) */
on_toolbar_refresh_clicked             (GtkButton       *button,
                                        MainWin         *mwin)
{
	app_get_new_item (mwin, (gpointer*) button);
}


void /* Exit in toolbar */
on_toolbar_quit_clicked                (GtkButton       *button,
                                        MainWin         *mwin)
{
	gtk_exit(0);
}


//}

//{ ClipList CallBacks
void cliplist_row_removed (Selection *item)
{ /* If a row got removed (clear the attached data. (imp.!)*/
	selection_free(item);
}


void /* unused at this moment */
on_cliplist_click_column               (GtkCList        *clist,
                                        gint             column,
                                        MainWin         *mwin)

{

}


void /* Select a row (return on doubleclick) */
on_cliplist_select_row                 (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        MainWin         *mwin)

{
	Selection *item;
	app_update_menus(mwin);
	item = gtk_clist_get_row_data (clist, row);
	if ((item!=NULL) && (item->type!=NULL)) {
		/*if (!strcmp(item->type, "text")) {*/
			if ((event)&&(event->button.type == GDK_2BUTTON_PRESS)) {
				app_create_and_show_textitemwin (mwin, row);
				return;
			} else {
				if (mwin->prefs->multisel) {
					app_claim_multi_selection (mwin);
				} else {
					app_claim_text_selection (mwin, item);
					//XSetSelectionOwner(gdk_display, mwin->selection , item->owning_window, CurrentTime);
				}
				return;
			}
		/*}	*/
	}
}


void /* unselect row (update menu's) and refresh multiselection if prefs->multisel */
on_cliplist_unselect_row               (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        MainWin         *mwin)
{
	app_update_menus(mwin);
	if (mwin->prefs->multisel) {
		app_claim_multi_selection (mwin);
	}

}

int /* on any event (calling popupmenu on rightclick) */
on_cliplist_event                      (GtkCList        *clist,
                                        GdkEvent        *event,
                                        MainWin         *mwin)
{
	GdkEventButton *button_event = (GdkEventButton *) event;
	
	if (event->type == GDK_BUTTON_PRESS && button_event->button == 3) {
		app_update_menus(mwin);
		gtk_menu_popup (GTK_MENU(mwin->cliplistpopupmenu), NULL, NULL, NULL, NULL, 3, 
						button_event->time);
		return TRUE;
	}
	return FALSE;
}



//}

//{ Applet CallBacks
/* Applet callbacks */

int on_cliplist_applet_event           (GtkCList        *clist,
                                        GdkEvent        *event,
                                        MainWin         *mwin)
{
	GdkEventButton *button_event = (GdkEventButton *) event;

	if ((event->type == GDK_BUTTON_PRESS && button_event->button == 2) && (GTK_CLIST(mwin->cliplist)->rows)>0) {
		gint i;
		gchar *text;
		GtkWidget *amenu;

		amenu = gtk_menu_new();

		for (i=0; i< GTK_CLIST(mwin->cliplist)->rows; i++) {
			GtkWidget *mitem;
			AppletItem *aitem;

			aitem = (AppletItem*) g_malloc0(sizeof(AppletItem));
			aitem->mwin = mwin;
			aitem->row = i;
			aitem->amenu = amenu;
			gtk_clist_get_text (GTK_CLIST(mwin->cliplist), i, CLIST_DATA_FIELD, &text);
			mitem = gtk_menu_item_new_with_label (text);
			gtk_menu_append (GTK_MENU(amenu), mitem);
			gtk_widget_show(mitem);
			gtk_signal_connect_object (GTK_OBJECT (mitem), "activate",
                               GTK_SIGNAL_FUNC (applet_menuitem_activate),
                               (gpointer) aitem);
		}
		gtk_menu_popup (GTK_MENU(amenu), NULL, NULL, NULL, NULL, 2, 
						button_event->time);
		return TRUE;
	}
	return FALSE;
	
}

void applet_menuitem_activate(AppletItem *aitem)
{
	gtk_clist_select_row(GTK_CLIST(aitem->mwin->cliplist), aitem->row, 1);
	gtk_widget_destroy(aitem->amenu);
}

//}


//{ End of file
//}
