/* * C o p y r i g h t *(also read COPYING)* * * * * * * * * * * * * * * * * * *
 *                                                                             *
 *  Copyright (C) 2002  <Philip Van Hoof>                                      *
 *                                                                             *
 *  This program is free software; you can redistribute it and/or modify       *
 *  it under the terms of the GNU General Public License as published by       *
 *  the Free Software Foundation; either version 2 of the License, or          *
 *  (at your option) any later version.                                        *
 *                                                                             *
 *  This program is distributed in the hope that it will be useful,            *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *
 *  GNU General Public License for more details.                               *
 *                                                                             *
 *  You should have received a copy of the GNU General Public License          *
 *  along with this program; if not, write to the Free Software                *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
 *                                                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifdef HAVE_CONFIG_H
#  include "../config.h"
#endif

#include <gnome.h>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>

#include <X11/Xlib.h> 
#include <X11/Xatom.h>
#include <stdio.h> 
#include <string.h> 
#include <gdk/gdk.h>
#include <gdk/gdkprivate.h>
#include <gdk/gdkx.h>
#include <gtk/gtk.h>



#ifdef __cplusplus
extern "C" {
#endif


#define GNOME_TYPE_MAINWIN (mainwin_get_type())
#define GNOME_MAINWIN(obj) (GTK_CHECK_CAST((obj), GNOME_TYPE_MAINWIN, MainWin))
#define GNOME_MAINWIN_CLASS(klass) (GTK_CHECK_CLASS_CAST((klass), GNOME_TYPE_MAINWIN, MainWinClass))
#define GNOME_IS_MAINWIN(obj) (GTK_CHECK_TYPE((obj), GNOME_TYPE_MAINWIN))
#define GNOME_IS_MAINWIN_CLASS(klass) (GTK_CHECK_CLASS_TYPE((klass), GNOME_TYPE_MAINWIN))

#define CLIST_FIELD_COUNT 4
#define CLIST_DATE_FIELD 0
#define CLIST_TYPE_FIELD 1
#define CLIST_FROM_FIELD 2
#define CLIST_DATA_FIELD 3
#define CLIST_SUB_APPLET 20
#define CLIST_SUB_NORMAL 95
#define CLIST_MAXLEN_NORMAL 200
#define CLIST_MAXLEN_APPLET 50

typedef struct _MainWin MainWin;
typedef struct _MainWinClass MainWinClass;
typedef struct _Selection Selection;
typedef struct _Prefs Prefs;
typedef struct _SaveAsData SaveAsData;
typedef struct _AppletItem AppletItem;


struct _Prefs {
	gint selection_type;
	gchar *custom_selection_type;
	
	gint target_type;
	gchar *custom_target_type;
#ifdef DEVELOPMENT
	gboolean debug;
#endif
	gboolean beep_on_collect;
	gboolean auto_collect;
	gboolean show_toolbar;
	gboolean show_menu;
	gboolean show_cliplist_titles;
	gboolean show_statusbar;
	gboolean show_from;
	gboolean show_time;
	gboolean show_type;
	gboolean show_datap;
	gboolean multisel;
	gboolean applet;
	gboolean unselall;
	gboolean followmode;
	gboolean selectnew;
	gboolean usemaxitems;
	gint maxitems;

};

struct _Selection {
  gchar *time;
  gchar *type;
  gchar *from;
  gchar *data;
  gint row;
  Window owning_app;
  Window owning_window;
  GdkAtom target;
};

struct _MainWin {
  GtkWindow window;
  GtkWidget *topvbox;
  GtkWidget *menuhandlebox;
  GtkWidget *toolbarhandlebox;
  GtkWidget *vbox;
  GtkWidget *mainwinmenu;
  GtkWidget *toolbar;
  GtkWidget *tmp_toolbar_icon;
  GtkWidget *toolbar_newitem;
  GtkWidget *toolbar_deleteitem;
  GtkWidget *toolbar_edititem;
  GtkWidget *toolbar_clear;
  GtkWidget *toolbar_preferences;
  GtkWidget *toolbar_refresh;
  GtkWidget *toolbar_quit;
  GtkWidget *scrolledwindow;
  GtkWidget *cliplist;
  GtkWidget *cliplistlabel_time;
  GtkWidget *cliplistlabel_type;
  GtkWidget *cliplistlabel_from;
  GtkWidget *cliplistlabel_data;
  GtkWidget *appbar;
  GtkWidget *cliplistpopupmenu;

  GtkWidget *prefswin;
  GtkWidget *newitemwin;

  GtkWidget *savefilelist;
  GtkWidget *savefilelist_ok_button;
  GtkWidget *savefilelist_cancel_button;
  gboolean savefilelist_inuse;
 
  GtkWidget *openfilelist;
  GtkWidget *openfilelist_ok_button;
  GtkWidget *openfilelist_cancel_button;
  gboolean openfilelist_inuse;
 
  GtkWidget *windowparent;
  gint lastnewrow;
  Selection *lastnewitem;
  GdkAtom selection;
  GdkAtom target_atom;
  Selection *currentitem;

  Prefs *prefs;
  GList *copypaste;

  /* fixme: nomore textbuffer clipboard in v2.0 
	  only "currentitem" should be used.
  */
  gchar *clipboard;
  GdkAtom clipboardtarget;
  GtkAccelGroup *accel_group;
  
  GtkWidget *applet;
  GtkWidget *appletscroll;
  gint appletheight;

  Window new_owner;
  Window last_owning_window;
  gchar *new_owner_title;

  gint argc;
  gchar **argv;
  
};

struct _MainWinClass {
   GtkWindowClass parent_class;
};


struct _SaveAsData {
	GList *items;
	gchar *path;
};

struct _AppletItem {
	gint row;
	MainWin *mwin;
	GtkWidget *amenu;
};

GtkType mainwin_get_type(void);
GtkWidget* mainwin_new (GtkWidget *windowparent);
GtkWidget* create_applet (MainWin *mwin);



gchar* g_unichar_to_utf8 (guint16 c);
gchar* ucs2tostr(guchar *data, gint length);
void saveasdata_free (SaveAsData *sa);
void selection_free (Selection *item);
void prefs_free (Prefs *p);

gchar* app_make_datapreview (MainWin *mwin, gchar *in);
void app_create_cliplist(MainWin *mwin);
gboolean saveprefs_tofile (Prefs *prefs);
Prefs* getprefs_fromfile (void);
void reload_cliplist_prefs(MainWin *mwin);
void reload_prefs (MainWin *mwin);
void app_show_prefswin (MainWin *mwin, gpointer *user_data);
void app_prepare_menus(MainWin *mwin);
void app_createhintsfor_menus(MainWin *mwin);
GtkWidget* app_create_and_show_newitemwin (MainWin *mwin, gpointer *user_data);
GtkWidget* app_create_and_show_textitemwin (MainWin *mwin, gint row);
GtkWidget* app_create_and_show_dataitemwin (MainWin *mwin, gint row);
GtkWidget* app_create_and_show_aboutwin (MainWin *mwin, gpointer *user_data);
gboolean app_save_items_to_disk(gchar *path, SaveAsData *sa);
void app_load_items_from_disk(MainWin *mwin, gchar *path);
void app_load_items_from_mem(MainWin *mwin, gchar *xml, gint size);
void app_get_new_item (MainWin *mwin, gpointer *user_data) ;
void app_delete_selected_items(MainWin *mwin, GtkCList *cliplist, gpointer *usr_data);
gint app_add_selection (MainWin *mwin, Selection *item);
gint app_add_text_selection (MainWin *mwin, gchar *type, gchar *from, gchar *data);
gint app_add_anytype_selection  (MainWin *mwin, GdkAtom target, gchar *type, gchar *from, gchar *data);
void app_update_text_selection (MainWin *mwin, gint row, gchar *type, gchar *from, gchar *data);
void app_claim_multi_selection (MainWin *mwin);
void app_claim_text_selection (MainWin *mwin, Selection *item);
void app_update_menus(MainWin *mwin);

void cliplist_row_removed (Selection *item);


gpointer
get_typed_property_data (Display *xdisplay,
			 Window   xwindow,
			 Atom     property,
			 Atom     requested_type,
			 gint    *size_p,
			 guint    expected_format);

/* Public MainWin CallBacks */

void mainwin_show(GtkWidget *widget, MainWin *mwin);
void mainwin_hide(GtkWidget *widget, MainWin *mwin);

int
on_mainwin_event                       (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        MainWin         *mwin);

void
on_mainwin_selection_received          (GtkWidget       *widget,
                                        GtkSelectionData *data, 
                                        guint            time,  
                                        MainWin         *mwin);

gboolean
on_mainwin_in_selection_clear_event    (GtkWidget       *widget,
                                        GdkEventSelection *event,
                                        MainWin         *mwin);

gboolean
on_mainwin_in_selection_notify_event   (GtkWidget       *widget,
                                        GdkEventSelection *event,
                                        MainWin         *mwin);

gboolean
on_mainwin_in_selection_request_event  (GtkWidget       *widget,
                                        GdkEventSelection *event,
                                        MainWin         *mwin);

gboolean
on_mainwin_selection_lost              (GtkWidget       *widget,
                                        GdkEventSelection *event,
                                        MainWin         *mwin);

void
on_mainwin_selection_get               (GtkWidget       *widget, 
                                        GtkSelectionData *selection_data,
                                        guint           info,
                                        guint           time_stamp,
                                        MainWin         *mwin);

void
on_mainwinmenu_newitem_activate        (GtkMenuItem     *menuitem,
                                        MainWin         *mwin);
void
on_mainwinmenu_applet_activate         (GtkMenuItem     *menuitem,
                                        MainWin         *mwin);

void
on_mainwinmenu_deleteitem_activate     (GtkMenuItem     *menuitem,
                                        MainWin         *mwin);
void
on_mainwinmenu_moveup_activate         (GtkMenuItem     *menuitem,
                                        MainWin         *mwin);

void
on_mainwinmenu_movedown_activate         (GtkMenuItem     *menuitem,
                                        MainWin         *mwin);
void
on_mainwin_savefilelist_ok_button_clicked
                                       (GtkButton       *button,
                                        MainWin         *mwin);

void
on_mainwin_savefilelist_cancel_button_clicked
                                       (GtkButton       *button,
                                        MainWin         *mwin);

void
on_mainwin_openfilelist_ok_button_clicked
                                       (GtkButton       *button,
                                        MainWin         *mwin);

void
on_mainwin_openfilelist_cancel_button_clicked
                                       (GtkButton       *button,
                                        MainWin         *mwin);

void
on_mainwin_overwrite_yes_no            (gint            answer,
                                        SaveAsData      *sa);

void
on_mainwin_saveallfilelist_ok_button_clicked
                                       (GtkButton *button,
                                        MainWin *mwin);
void
on_mainwinmenu_file_savealltodisk_activate
                                        (GtkMenuItem    *menuitem,
                                        MainWin         *mwin);
void
on_mainwinmenu_file_savetodisk_activate
                                        (GtkMenuItem    *menuitem,
                                        MainWin         *mwin);

void
on_mainwinmenu_file_openfromdisk_activate
                                        (GtkMenuItem    *menuitem,
                                        MainWin         *mwin);


void
on_mainwinmenu_exit_activate           (GtkMenuItem     *menuitem,
                                        MainWin         *mwin);
void
on_mainwinmenu_selectnone_activate     (GtkMenuItem     *menuitem,
                                        MainWin         *mwin);
void
on_mainwinmenu_selectall_activate     (GtkMenuItem     *menuitem,
                                        MainWin         *mwin);

void
on_mainwinmenu_cut_activate            (GtkMenuItem     *menuitem,
                                        MainWin         *mwin);


void
on_mainwinmenu_copy_activate           (GtkMenuItem     *menuitem,
                                        MainWin         *mwin);


void
on_mainwinmenu_paste_activate          (GtkMenuItem     *menuitem,
                                        MainWin         *mwin);


void
on_mainwinmenu_clear_activate          (GtkMenuItem     *menuitem,
                                        MainWin         *mwin);


void
on_mainwinmenu_preferences_activate    (GtkMenuItem     *menuitem,
                                        MainWin         *mwin);


void
on_mainwinmenu_about_activate          (GtkMenuItem     *menuitem,
                                        MainWin         *mwin);

void
on_cliplistpopupmenu_new_activate      (GtkMenuItem     *menuitem,
                                        MainWin         *mwin);
void
on_cliplistpopupmenu_cut_activate      (GtkMenuItem     *menuitem,
                                        MainWin         *mwin);

void
on_cliplistpopupmenu_copy_activate     (GtkMenuItem     *menuitem,
                                        MainWin         *mwin);
void
on_cliplistpopupmenu_paste_activate    (GtkMenuItem     *menuitem,
                                        MainWin         *mwin);

void
on_cliplistpopupmenu_edit_activate     (GtkMenuItem     *menuitem,
                                        MainWin         *mwin);

void
on_cliplistpopupmenu_convert_activate  (GtkMenuItem     *menuitem,
                                        MainWin         *mwin);


void
on_toolbar_newitem_clicked             (GtkButton       *button,
                                        MainWin         *mwin);

void
on_toolbar_clear_clicked               (GtkButton       *button,
                                        MainWin         *mwin);

void
on_toolbar_deleteitem_clicked          (GtkButton       *button,
                                        MainWin         *mwin);

void
on_toolbar_edititem_clicked            (GtkButton       *button,
                                        MainWin         *mwin);


void
on_toolbar_preferences_clicked         (GtkButton       *button,
                                        MainWin         *mwin);


void
on_toolbar_refresh_clicked             (GtkButton       *button,
                                        MainWin         *mwin);


void
on_toolbar_quit_clicked                (GtkButton       *button,
                                        MainWin         *mwin);


void
on_cliplist_click_column               (GtkCList        *clist,
                                        gint             column,
                                        MainWin         *mwin);


void
on_cliplist_select_row                 (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        MainWin         *mwin);

void
on_cliplist_unselect_row               (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        MainWin         *mwin);

int
on_cliplist_event                      (GtkCList        *clist,
                                        GdkEvent        *event,
                                        MainWin         *mwin);


int on_cliplist_applet_event           (GtkCList        *clist,
                                        GdkEvent        *event,
                                        MainWin         *mwin);

void applet_menuitem_activate(AppletItem *aitem);

#ifdef __cplusplus /* cpp compatibility */
}
#endif

