/* * C o p y r i g h t *(also read COPYING)* * * * * * * * * * * * * * * * * * *
 *                                                                             *
 *  Copyright (C) 2002  <Philip Van Hoof>                                      *
 *                                                                             *
 *  This program is free software; you can redistribute it and/or modify       *
 *  it under the terms of the GNU General Public License as published by       *
 *  the Free Software Foundation; either version 2 of the License, or          *
 *  (at your option) any later version.                                        *
 *                                                                             *
 *  This program is distributed in the hope that it will be useful,            *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *
 *  GNU General Public License for more details.                               *
 *                                                                             *
 *  You should have received a copy of the GNU General Public License          *
 *  along with this program; if not, write to the Free Software                *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
 *                                                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
#include "prefswin.h"
#include "mainwin.h"

static void prefswin_class_init(PrefsWinClass *klass);
static void prefswin_init(PrefsWin *twin);
static void prefswin_destroy(GtkObject *object);

Prefs* prefswin_getprefs (PrefsWin *pwin);
void prefswin_loadprefs (PrefsWin *pwin, Prefs *prefs);
void prefswin_saveprefs_tofile (PrefsWin *pwin, Prefs *prefs);


static GtkWindowClass *parent_class = NULL;

GtkType prefswin_get_type(void)
{
   static GtkType prefswin_type = 0;
   
   if (!prefswin_type)
   {
      static const GtkTypeInfo prefswin_info =
      {	
         "PrefsWin",
         sizeof (PrefsWin),
         sizeof (PrefsWinClass),
         (GtkClassInitFunc) prefswin_class_init,
         (GtkObjectInitFunc) prefswin_init,
         NULL,
         NULL,
         (GtkClassInitFunc) NULL,
      };
      
      prefswin_type = gtk_type_unique (GTK_TYPE_WINDOW, &prefswin_info);
   }

   return (prefswin_type);
}

static void prefswin_class_init(PrefsWinClass *klass)
{
   GtkObjectClass *object_class;
   object_class = (GtkObjectClass*)klass;
   object_class->destroy = prefswin_destroy;
   parent_class = gtk_type_class(GTK_TYPE_WINDOW);
}

static void prefswin_init(PrefsWin *pwin)
{

}


static void prefswin_destroy(GtkObject *object)
{
   GtkWidget *pwin;
  
   g_return_if_fail(object != NULL);
   g_return_if_fail(GNOME_IS_PREFSWIN(object));
   
   pwin = GTK_WIDGET(object);
      
   gtk_widget_destroy(pwin);
}


GtkWidget* prefswin_new_and_getprefs(GtkWidget *windowparent)
{
	GtkWidget *tpwin;
	tpwin = prefswin_new(windowparent);
	prefswin_getprefs(GNOME_PREFSWIN(tpwin));
	return tpwin;
}



/* Get the new preferences from the GUI */
Prefs* prefswin_getprefs (PrefsWin *pwin)
{
	/* Return the prefrences in the Preferences window as a Prefs */
	Prefs *prefs=NULL;
	gchar *text, *text2;
	
	prefs = g_malloc0(sizeof(Prefs));
	text = gtk_entry_get_text(GTK_ENTRY(pwin->seltypecentry));
	
		   if (!strcmp(text, PREFS_SELECTION_TYPE_CUSTOM_TEXT)) {
		prefs->selection_type = PREFS_SELECTION_TYPE_CUSTOM;
	} else if (!strcmp(text, PREFS_SELECTION_TYPE_CLIPBOARD_TEXT)) {
		prefs->selection_type = PREFS_SELECTION_TYPE_CLIPBOARD;
	} else if (!strcmp(text, PREFS_SELECTION_TYPE_PRIMARY_TEXT)) {
		prefs->selection_type = PREFS_SELECTION_TYPE_PRIMARY;
	} else if (!strcmp(text, PREFS_SELECTION_TYPE_SECONDARY_TEXT)) {
		prefs->selection_type = PREFS_SELECTION_TYPE_SECONDARY;
	} else {
		prefs->selection_type = PREFS_SELECTION_TYPE_CLIPBOARD;
	}

	text2 = gtk_entry_get_text(GTK_ENTRY(pwin->tartypecentry));
	
		   if (!strcmp(text2, PREFS_TARGET_TYPE_CUSTOM_TEXT)) {
		prefs->target_type = PREFS_TARGET_TYPE_CUSTOM;
	} else if (!strcmp(text2, PREFS_TARGET_TYPE_TEXT_TEXT)) {
		prefs->target_type = PREFS_TARGET_TYPE_TEXT;
	} else if (!strcmp(text2, PREFS_TARGET_TYPE_HTML_TEXT)) {
		prefs->target_type = PREFS_TARGET_TYPE_HTML;
	} else if (!strcmp(text2, PREFS_TARGET_TYPE_TARGETS_TEXT)) {
		prefs->target_type = PREFS_TARGET_TYPE_TARGETS;
	} else if (!strcmp(text2, PREFS_TARGET_TYPE_RICH_TEXT)) {
		prefs->target_type = PREFS_TARGET_TYPE_RICH;
	} else {
		prefs->target_type = PREFS_TARGET_TYPE_TEXT;
	}

	prefs->custom_selection_type = g_strdup(gtk_entry_get_text(GTK_ENTRY(pwin->seltypecustentry)));
	prefs->custom_target_type = g_strdup(gtk_entry_get_text(GTK_ENTRY(pwin->tartypecustentry)));
	prefs->beep_on_collect = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->colbeep));
	prefs->auto_collect = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->colauto));
#ifdef DEVELOPMENT
	prefs->debug = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->devcheck));
#endif
	prefs->show_toolbar = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->guishowtoolbar));
	prefs->show_menu = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->guishowmenu));
	prefs->show_cliplist_titles = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->guishowtitles));
	prefs->show_statusbar = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->guishowstatus));
	prefs->show_from = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->guishowfrom));
	prefs->show_time = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->guishowtime));
	prefs->show_type = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->guishowtype));
	prefs->show_datap = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->guishowdatap));
	prefs->multisel = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->advmultisel));
	prefs->applet = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->aplapplet));
	prefs->unselall = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->advunsel));
	prefs->followmode = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->advfollow));
	prefs->selectnew = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->advselnew));
	prefs->usemaxitems = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->colmaxcheck));
	prefs->maxitems = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(pwin->colmaxspin));
	return prefs;
}


/* Unused at this moment */
void prefswin_loadprefs (PrefsWin *pwin, Prefs *prefs)
{
	/* Load the application's preferences from prefs */
//	reload_prefs(GNOME_MAINWIN(pwin->windowparent));
}




/* Creates a new preferences GUI */
GtkWidget* prefswin_new (GtkWidget *windowparent)
{
  PrefsWin *pwin;

  pwin = gtk_type_new(GNOME_TYPE_PREFSWIN);
  gtk_signal_connect(GTK_OBJECT(pwin), "show", GTK_SIGNAL_FUNC(prefswin_show), pwin);
  gtk_signal_connect(GTK_OBJECT(pwin), "hide", GTK_SIGNAL_FUNC(prefswin_hide), pwin);
  pwin->windowparent = windowparent;
  gtk_window_set_title (GTK_WINDOW (pwin), _("Preferences"));

  pwin->appletwarn=TRUE;
  gtk_widget_set_usize (GTK_WIDGET(pwin), 600, 400);
  gtk_window_set_policy (GTK_WINDOW (pwin), TRUE, TRUE, FALSE);
  pwin->tooltips = gtk_tooltips_new();
  gtk_tooltips_set_tip (pwin->tooltips, GTK_WIDGET(pwin), _("Preferences"), NULL);

  pwin->tartypecombo_items = NULL;
  pwin->seltypecombo_items = NULL;
  pwin->tooltips = gtk_tooltips_new ();
  pwin->prefvbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (pwin->prefvbox);
  gtk_container_add (GTK_CONTAINER (pwin), pwin->prefvbox);

  pwin->prefnbk = gtk_notebook_new ();
  gtk_widget_show (pwin->prefnbk);
  gtk_box_pack_start (GTK_BOX (pwin->prefvbox), pwin->prefnbk, TRUE, TRUE, 0);

  pwin->genvbox = gtk_vbox_new (TRUE, 5);
  gtk_widget_show (pwin->genvbox);
  gtk_container_add (GTK_CONTAINER (pwin->prefnbk), pwin->genvbox);

  pwin->seltypeframe = gtk_frame_new (_("Selection/target type"));
  gtk_widget_show (pwin->seltypeframe);
  gtk_box_pack_start (GTK_BOX (pwin->genvbox), pwin->seltypeframe, TRUE, TRUE, 0);

  pwin->seltypetable = gtk_table_new (2, 4, FALSE);
  gtk_widget_show (pwin->seltypetable);
  gtk_container_add (GTK_CONTAINER (pwin->seltypeframe), pwin->seltypetable);
  gtk_table_set_row_spacings (GTK_TABLE (pwin->seltypetable), 5);
  gtk_table_set_col_spacings (GTK_TABLE (pwin->seltypetable), 5);

  pwin->seltypelabel = gtk_label_new (_("Selection type:"));
  gtk_widget_show (pwin->seltypelabel);
  gtk_table_attach (GTK_TABLE (pwin->seltypetable), pwin->seltypelabel, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (pwin->seltypelabel), 0, 0.5);

  pwin->seltypecombo = gtk_combo_new ();
  gtk_widget_show (pwin->seltypecombo);
  gtk_table_attach (GTK_TABLE (pwin->seltypetable), pwin->seltypecombo, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  pwin->seltypecombo_items = g_list_append (pwin->seltypecombo_items, (gpointer) _(PREFS_SELECTION_TYPE_CLIPBOARD_TEXT));
  pwin->seltypecombo_items = g_list_append (pwin->seltypecombo_items, (gpointer) _(PREFS_SELECTION_TYPE_PRIMARY_TEXT));
  pwin->seltypecombo_items = g_list_append (pwin->seltypecombo_items, (gpointer) _(PREFS_SELECTION_TYPE_SECONDARY_TEXT));
  pwin->seltypecombo_items = g_list_append (pwin->seltypecombo_items, (gpointer) _(PREFS_SELECTION_TYPE_CUSTOM_TEXT));
  gtk_combo_set_popdown_strings (GTK_COMBO (pwin->seltypecombo), pwin->seltypecombo_items);
  g_list_free (pwin->seltypecombo_items);
 
  pwin->seltypecentry = GTK_COMBO (pwin->seltypecombo)->entry;
  gtk_widget_show (pwin->seltypecentry);
  gtk_entry_set_editable (GTK_ENTRY (pwin->seltypecentry), FALSE);
  gtk_entry_set_text (GTK_ENTRY (pwin->seltypecentry), _(PREFS_SELECTION_TYPE_CLIPBOARD_TEXT));

  pwin->seltypecustlabel = gtk_label_new (_("Custom atom:"));
  gtk_widget_show (pwin->seltypecustlabel);
  gtk_table_attach (GTK_TABLE (pwin->seltypetable), pwin->seltypecustlabel, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (pwin->seltypecustlabel), 0, 0.5);

  pwin->seltypecustentry = gtk_entry_new ();
  gtk_widget_show (pwin->seltypecustentry);
  gtk_table_attach (GTK_TABLE (pwin->seltypetable), pwin->seltypecustentry, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  gtk_widget_set_sensitive(pwin->seltypecustlabel, FALSE);
  gtk_widget_set_sensitive(pwin->seltypecustentry, FALSE);







  pwin->tartypelabel = gtk_label_new (_("Target type:"));
  gtk_widget_show (pwin->tartypelabel);
  gtk_table_attach (GTK_TABLE (pwin->seltypetable), pwin->tartypelabel, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (pwin->tartypelabel), 0, 0.5);

  pwin->tartypecombo = gtk_combo_new ();
  gtk_widget_show (pwin->tartypecombo);
  gtk_table_attach (GTK_TABLE (pwin->seltypetable), pwin->tartypecombo, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  pwin->tartypecombo_items = g_list_append (pwin->tartypecombo_items, (gpointer) _(PREFS_TARGET_TYPE_TEXT_TEXT));
  pwin->tartypecombo_items = g_list_append (pwin->tartypecombo_items, (gpointer) _(PREFS_TARGET_TYPE_RICH_TEXT));
  pwin->tartypecombo_items = g_list_append (pwin->tartypecombo_items, (gpointer) _(PREFS_TARGET_TYPE_HTML_TEXT));
  pwin->tartypecombo_items = g_list_append (pwin->tartypecombo_items, (gpointer) _(PREFS_TARGET_TYPE_TARGETS_TEXT));
  pwin->tartypecombo_items = g_list_append (pwin->tartypecombo_items, (gpointer) _(PREFS_TARGET_TYPE_CUSTOM_TEXT));
  gtk_combo_set_popdown_strings (GTK_COMBO (pwin->tartypecombo), pwin->tartypecombo_items);
  g_list_free (pwin->tartypecombo_items);
 
  pwin->tartypecentry = GTK_COMBO (pwin->tartypecombo)->entry;
  gtk_widget_show (pwin->tartypecentry);
  gtk_entry_set_editable (GTK_ENTRY (pwin->tartypecentry), FALSE);
  gtk_entry_set_text (GTK_ENTRY (pwin->tartypecentry), _(PREFS_TARGET_TYPE_TEXT_TEXT));

  pwin->tartypecustlabel = gtk_label_new (_("Custom atom:"));
  gtk_widget_show (pwin->tartypecustlabel);
  gtk_table_attach (GTK_TABLE (pwin->seltypetable), pwin->tartypecustlabel, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (pwin->tartypecustlabel), 0, 0.5);

  pwin->tartypecustentry = gtk_entry_new ();
  gtk_widget_show (pwin->tartypecustentry);
  gtk_table_attach (GTK_TABLE (pwin->seltypetable), pwin->tartypecustentry, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  gtk_widget_set_sensitive(pwin->tartypecustlabel, FALSE);
  gtk_widget_set_sensitive(pwin->tartypecustentry, FALSE);





  pwin->colframe = gtk_frame_new (_("Collecting"));
  gtk_widget_show (pwin->colframe);
  gtk_box_pack_start (GTK_BOX (pwin->genvbox), pwin->colframe, FALSE, TRUE, 0);

  pwin->colvbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (pwin->colvbox);
  gtk_container_add (GTK_CONTAINER (pwin->colframe), pwin->colvbox);

  pwin->colauto = gtk_check_button_new_with_label (_("Auto collect new selections on the clipboard shelf"));
  gtk_widget_show (pwin->colauto);
  gtk_box_pack_start (GTK_BOX (pwin->colvbox), pwin->colauto, FALSE, FALSE, 0);

  pwin->colbeep = gtk_check_button_new_with_label (_("Beep when a new selection got collected on the clipboard shelf"));
  gtk_widget_show (pwin->colbeep);
  gtk_box_pack_start (GTK_BOX (pwin->colvbox), pwin->colbeep, FALSE, FALSE, 0);

  pwin->colmaxhbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (pwin->colmaxhbox);
  gtk_container_add (GTK_CONTAINER (pwin->colvbox), pwin->colmaxhbox);

  pwin->colmaxcheck = gtk_check_button_new_with_label (_("Limit the max. amount of collected items to "));
  gtk_widget_show (pwin->colmaxcheck);
  gtk_box_pack_start (GTK_BOX (pwin->colmaxhbox), pwin->colmaxcheck, FALSE, FALSE, 0);

  pwin->colmaxspinadj = gtk_adjustment_new (1, 0, 99999, 1, 10, 10);
  pwin->colmaxspin = gtk_spin_button_new (GTK_ADJUSTMENT (pwin->colmaxspinadj), 1, 0);
  gtk_widget_show (pwin->colmaxspin);
  gtk_box_pack_start (GTK_BOX (pwin->colmaxhbox), pwin->colmaxspin, FALSE, FALSE, 0);


  pwin->prefnbkgeneral = gtk_label_new (_("General"));
  gtk_widget_show (pwin->prefnbkgeneral);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (pwin->prefnbk), gtk_notebook_get_nth_page (GTK_NOTEBOOK (pwin->prefnbk), 0), pwin->prefnbkgeneral);

  pwin->netwlbl = gtk_label_new (_("unfinished at this moment.\nWill be used for the networking features (see the project netclip)."));
  gtk_widget_show (pwin->netwlbl);
  gtk_container_add (GTK_CONTAINER (pwin->prefnbk), pwin->netwlbl);

  pwin->prefnbknetworking = gtk_label_new (_("Networking"));
  gtk_widget_show (pwin->prefnbknetworking);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (pwin->prefnbk), gtk_notebook_get_nth_page (GTK_NOTEBOOK (pwin->prefnbk), 1), pwin->prefnbknetworking);

  pwin->guitvbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (pwin->guitvbox);
  gtk_container_add (GTK_CONTAINER (pwin->prefnbk), pwin->guitvbox);
  pwin->guiframe = gtk_frame_new (_("User interface"));
  gtk_widget_show (pwin->guiframe);
  gtk_box_pack_start (GTK_BOX (pwin->guitvbox), pwin->guiframe, TRUE, TRUE, 0);

  pwin->guivbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (pwin->guivbox);
  gtk_container_add (GTK_CONTAINER (pwin->guiframe), pwin->guivbox);

  pwin->guishowtoolbar = gtk_check_button_new_with_label (_("Show toolbar"));
  gtk_widget_show (pwin->guishowtoolbar);
  gtk_box_pack_start (GTK_BOX (pwin->guivbox), pwin->guishowtoolbar, FALSE, FALSE, 0);

  pwin->guishowmenu = gtk_check_button_new_with_label (_("Show menu"));
  gtk_widget_show (pwin->guishowmenu);
  gtk_box_pack_start (GTK_BOX (pwin->guivbox), pwin->guishowmenu, FALSE, FALSE, 0);

  pwin->guishowtitles = gtk_check_button_new_with_label (_("Show titles in the clipboard shelf"));
  gtk_widget_show (pwin->guishowtitles);
  gtk_box_pack_start (GTK_BOX (pwin->guivbox), pwin->guishowtitles, FALSE, FALSE, 0);

  pwin->guishowstatus = gtk_check_button_new_with_label (_("Show statusbar"));
  gtk_widget_show (pwin->guishowstatus);
  gtk_box_pack_start (GTK_BOX (pwin->guivbox), pwin->guishowstatus, FALSE, FALSE, 0);

  pwin->guishowfrom = gtk_check_button_new_with_label (_("Show 'from' of each item on the clipboard shelf"));
  gtk_widget_show (pwin->guishowfrom);
  gtk_box_pack_start (GTK_BOX (pwin->guivbox), pwin->guishowfrom, FALSE, FALSE, 0);

  pwin->guishowtime = gtk_check_button_new_with_label (_("Show 'time' of each item on the clipboard shelf"));
  gtk_widget_show (pwin->guishowtime);
  gtk_box_pack_start (GTK_BOX (pwin->guivbox), pwin->guishowtime, FALSE, FALSE, 0);

  pwin->guishowtype = gtk_check_button_new_with_label (_("Show 'type' of each item on the clipboard shelf"));
  gtk_widget_show (pwin->guishowtype);
  gtk_box_pack_start (GTK_BOX (pwin->guivbox), pwin->guishowtype, FALSE, FALSE, 0);

  pwin->guishowdatap = gtk_check_button_new_with_label (_("Show 'data' preview of each item on the clipboard shelf"));
  gtk_widget_show (pwin->guishowdatap);
  gtk_box_pack_start (GTK_BOX (pwin->guivbox), pwin->guishowdatap, FALSE, FALSE, 0);
  
  pwin->prefnbkgui = gtk_label_new (_("GUI"));
  gtk_widget_show (pwin->prefnbkgui);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (pwin->prefnbk), gtk_notebook_get_nth_page (GTK_NOTEBOOK (pwin->prefnbk), 2), pwin->prefnbkgui);


  pwin->miscvbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (pwin->miscvbox);

  gtk_container_add (GTK_CONTAINER (pwin->prefnbk), pwin->miscvbox);
#ifdef DEVELOPMENT
  pwin->devframe = gtk_frame_new (_("Development"));
  gtk_widget_show (pwin->devframe);
  gtk_box_pack_start (GTK_BOX (pwin->miscvbox), pwin->devframe, TRUE, TRUE, 0);

  pwin->devvbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (pwin->devvbox);
  gtk_container_add (GTK_CONTAINER (pwin->devframe), pwin->devvbox);

  pwin->devcheck = gtk_check_button_new_with_label (_("Enable debugging information on terminal"));
  gtk_widget_show (pwin->devcheck);
  gtk_box_pack_start (GTK_BOX (pwin->devvbox), pwin->devcheck, FALSE, FALSE, 0);
#endif

  pwin->advframe = gtk_frame_new (_("Advanced options"));
  gtk_widget_show (pwin->advframe);
  gtk_box_pack_start (GTK_BOX (pwin->miscvbox), pwin->advframe, TRUE, TRUE, 0);
  pwin->advvbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (pwin->advvbox);
  gtk_container_add (GTK_CONTAINER (pwin->advframe), pwin->advvbox);

  pwin->advmultisel = gtk_check_button_new_with_label (_("Use multiselect for the clipboard shelf"));
  gtk_widget_show (pwin->advmultisel);
  gtk_box_pack_start (GTK_BOX (pwin->advvbox), pwin->advmultisel, FALSE, FALSE, 0);

  pwin->advunsel = gtk_check_button_new_with_label (_("Enable unselect-all after collecting a new item"));
  gtk_widget_show (pwin->advunsel);
  gtk_box_pack_start (GTK_BOX (pwin->advvbox), pwin->advunsel, FALSE, FALSE, 0);

  pwin->advfollow = gtk_check_button_new_with_label (_("Enable following after collecting a new item"));
  gtk_widget_show (pwin->advfollow);
  gtk_box_pack_start (GTK_BOX (pwin->advvbox), pwin->advfollow, FALSE, FALSE, 0);

  pwin->advselnew = gtk_check_button_new_with_label (_("Enable select-new after collecting a new item"));
  gtk_widget_show (pwin->advselnew);
  gtk_box_pack_start (GTK_BOX (pwin->advvbox), pwin->advselnew, FALSE, FALSE, 0);

  pwin->prefnbkmisc = gtk_label_new (_("Misc"));
  gtk_widget_show (pwin->prefnbkmisc);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (pwin->prefnbk), gtk_notebook_get_nth_page (GTK_NOTEBOOK (pwin->prefnbk), 3), pwin->prefnbkmisc);

  pwin->apltvbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (pwin->apltvbox);

  gtk_container_add (GTK_CONTAINER (pwin->prefnbk), pwin->apltvbox);

  pwin->aplframe = gtk_frame_new (_("Applet support"));
  gtk_widget_show (pwin->aplframe);
  gtk_box_pack_start (GTK_BOX (pwin->apltvbox), pwin->aplframe, TRUE, TRUE, 0);

  pwin->aplvbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (pwin->aplvbox);
  gtk_container_add (GTK_CONTAINER (pwin->aplframe), pwin->aplvbox);

  pwin->aplapplet = gtk_check_button_new_with_label (_("Start Gnome Clipboard Manager as applet"));
  gtk_widget_show (pwin->aplapplet);
  gtk_box_pack_start (GTK_BOX (pwin->aplvbox), pwin->aplapplet, FALSE, FALSE, 0);

  pwin->prefnbkapplet = gtk_label_new (_("Applet"));
  gtk_widget_show (pwin->prefnbkapplet);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (pwin->prefnbk), gtk_notebook_get_nth_page (GTK_NOTEBOOK (pwin->prefnbk), 4), pwin->prefnbkapplet);


  pwin->prefhsep = gtk_hseparator_new ();
  gtk_widget_show (pwin->prefhsep);
  gtk_box_pack_start (GTK_BOX (pwin->prefvbox), pwin->prefhsep, FALSE, TRUE, 0);

  pwin->prefhbtnbox = gtk_hbutton_box_new ();
  gtk_widget_show (pwin->prefhbtnbox);
  gtk_box_pack_end (GTK_BOX (pwin->prefvbox), pwin->prefhbtnbox, FALSE, FALSE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (pwin->prefhbtnbox), GTK_BUTTONBOX_END);

  pwin->ok_button = gnome_stock_button (GNOME_STOCK_BUTTON_OK);
  gtk_widget_show (pwin->ok_button);
  gtk_container_add (GTK_CONTAINER (pwin->prefhbtnbox), pwin->ok_button);
  GTK_WIDGET_SET_FLAGS (pwin->ok_button, GTK_CAN_DEFAULT);

  pwin->apply_button = gnome_stock_button (GNOME_STOCK_BUTTON_APPLY);
  gtk_widget_show (pwin->apply_button);
  gtk_container_add (GTK_CONTAINER (pwin->prefhbtnbox), pwin->apply_button);
  GTK_WIDGET_SET_FLAGS (pwin->apply_button, GTK_CAN_DEFAULT);

  pwin->cancel_button = gnome_stock_button (GNOME_STOCK_BUTTON_CANCEL);
  gtk_widget_show (pwin->cancel_button);
  gtk_container_add (GTK_CONTAINER (pwin->prefhbtnbox), pwin->cancel_button);
  GTK_WIDGET_SET_FLAGS (pwin->cancel_button, GTK_CAN_DEFAULT);

  gtk_widget_grab_default (pwin->cancel_button);
  gtk_object_set_data (GTK_OBJECT (pwin), "tooltips", pwin->tooltips);
  
  gtk_signal_connect (GTK_OBJECT (pwin->apply_button), "clicked",
                      GTK_SIGNAL_FUNC (on_apply_button_clicked),
                      pwin);
  gtk_signal_connect (GTK_OBJECT (pwin->cancel_button), "clicked",
                      GTK_SIGNAL_FUNC (on_cancel_button_clicked),
                      pwin);
  gtk_signal_connect (GTK_OBJECT (pwin->ok_button), "clicked",
                      GTK_SIGNAL_FUNC (on_ok_button_clicked),
                      pwin);

  gtk_signal_connect (GTK_OBJECT (pwin->seltypecentry), "changed",
                      GTK_SIGNAL_FUNC (on_seltypecentry_changed),
                      pwin);

  gtk_signal_connect (GTK_OBJECT (pwin->tartypecentry), "changed",
                      GTK_SIGNAL_FUNC (on_tartypecentry_changed),
                      pwin);

  gtk_signal_connect (GTK_OBJECT (pwin->guishowtype), "toggled",
                      GTK_SIGNAL_FUNC (on_showtitlesgroup_changed),
                      pwin);

  gtk_signal_connect (GTK_OBJECT (pwin->guishowtime), "toggled",
                      GTK_SIGNAL_FUNC (on_showtitlesgroup_changed),
                      pwin);

  gtk_signal_connect (GTK_OBJECT (pwin->guishowdatap), "toggled",
                      GTK_SIGNAL_FUNC (on_showtitlesgroup_changed),
                      pwin);
  gtk_signal_connect (GTK_OBJECT (pwin->guishowfrom), "toggled",
                      GTK_SIGNAL_FUNC (on_showtitlesgroup_changed),
                      pwin);

  gtk_signal_connect (GTK_OBJECT (pwin->aplapplet), "toggled",
                      GTK_SIGNAL_FUNC (on_applettoggle_changed),
                      pwin);

  gtk_signal_connect (GTK_OBJECT (pwin->colmaxcheck), "toggled",
                      GTK_SIGNAL_FUNC (on_colmaxchecktoggle_changed),
                      pwin);

  gtk_window_set_wmclass (GTK_WINDOW (pwin), "prefswin", "gcm");
  return GTK_WIDGET(pwin);
}
/* prefswin callbacks */
void prefswin_show(GtkWidget *widget, PrefsWin *pwin)
{
	pwin->appletwarn=FALSE;
}

void prefswin_hide(GtkWidget *widget, PrefsWin *pwin)
{
}

void
on_apply_button_clicked
                                       (GtkButton *button,
                                        PrefsWin *pwin)
{
	
	Prefs *prefs;
	prefs = prefswin_getprefs (pwin);
	saveprefs_tofile (prefs);
	prefs_free(prefs);
	prefs = getprefs_fromfile();
	GNOME_MAINWIN(pwin->windowparent)->prefs = prefs;
	reload_prefs(GNOME_MAINWIN(pwin->windowparent));
}

void
on_cancel_button_clicked
                                       (GtkButton    *button,
                                        PrefsWin     *pwin)
{
	gtk_widget_hide(GTK_WIDGET(pwin));
}

void
on_ok_button_clicked
                                       (GtkButton    *button,
                                        PrefsWin     *pwin)
{
	on_apply_button_clicked (button, pwin);
	gtk_widget_hide(GTK_WIDGET(pwin));
}


void
on_seltypecentry_changed               (GtkEntry     *entry,
                                        PrefsWin     *pwin)
{
	gchar *text;

	text = gtk_entry_get_text (entry);

	if (!strcmp(text, PREFS_SELECTION_TYPE_CUSTOM_TEXT)) {
		gtk_widget_set_sensitive(pwin->seltypecustlabel, TRUE);
		gtk_widget_set_sensitive(pwin->seltypecustentry, TRUE);
		gtk_widget_grab_focus(pwin->seltypecustentry);
	} else {
		gtk_widget_set_sensitive(pwin->seltypecustlabel, FALSE);
		gtk_widget_set_sensitive(pwin->seltypecustentry, FALSE);
	}
}

void
on_tartypecentry_changed               (GtkEntry     *entry,
                                        PrefsWin     *pwin)
{
	gchar *text;

	text = gtk_entry_get_text (entry);

	if (!strcmp(text, PREFS_TARGET_TYPE_CUSTOM_TEXT)) {
		gtk_widget_set_sensitive(pwin->tartypecustlabel, TRUE);
		gtk_widget_set_sensitive(pwin->tartypecustentry, TRUE);
		gtk_widget_grab_focus(pwin->tartypecustentry);
	} else {
		gtk_widget_set_sensitive(pwin->tartypecustlabel, FALSE);
		gtk_widget_set_sensitive(pwin->tartypecustentry, FALSE);
	}
}




void
on_showtitlesgroup_changed             (GtkWidget     *toggle,
                                        PrefsWin     *pwin)
{
	if ( !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->guishowfrom)) &&
		!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->guishowtime)) &&
		!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->guishowtype)) &&
		!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->guishowdatap)))
	{
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pwin->guishowdatap), TRUE);
	}

}

void
on_applettoggle_changed                  (GtkWidget    *toggle,
                                          PrefsWin     *pwin) 
{
	if (pwin->appletwarn==FALSE) {
		gnome_warning_dialog(_("In order to reload applet-related settings you must restart Gnome Clipboard Manager."));
		pwin->appletwarn=TRUE;
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->aplapplet))) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pwin->guishowtitles), FALSE);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pwin->guishowtoolbar), FALSE);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pwin->guishowmenu), FALSE);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pwin->guishowstatus), FALSE);
	} else {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pwin->guishowtitles), TRUE);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pwin->guishowtoolbar), TRUE);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pwin->guishowmenu), TRUE);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(pwin->guishowstatus), TRUE);
	}
}


void
on_colmaxchecktoggle_changed             (GtkWidget    *toggle,
                                          PrefsWin     *pwin) 
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pwin->colmaxcheck))) {
		gtk_widget_set_sensitive(pwin->colmaxspin, TRUE);
	} else {
		gtk_widget_set_sensitive(pwin->colmaxspin, FALSE);
	}
}
