/* * C o p y r i g h t *(also read COPYING)* * * * * * * * * * * * * * * * * * *
 *                                                                             *
 *  Copyright (C) 2002  <Philip Van Hoof>                                      *
 *                                                                             *
 *  This program is free software; you can redistribute it and/or modify       *
 *  it under the terms of the GNU General Public License as published by       *
 *  the Free Software Foundation; either version 2 of the License, or          *
 *  (at your option) any later version.                                        *
 *                                                                             *
 *  This program is distributed in the hope that it will be useful,            *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *
 *  GNU General Public License for more details.                               *
 *                                                                             *
 *  You should have received a copy of the GNU General Public License          *
 *  along with this program; if not, write to the Free Software                *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
 *                                                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
#ifdef HAVE_CONFIG_H
#  include "../config.h"
#endif

#include <gnome.h>

#ifdef __cplusplus
extern "C" {
#endif
#define GNOME_TYPE_PREFSWIN (prefswin_get_type())
#define GNOME_PREFSWIN(obj) (GTK_CHECK_CAST((obj), GNOME_TYPE_PREFSWIN, PrefsWin))
#define GNOME_PREFSWIN_CLASS(klass) (GTK_CHECK_CLASS_CAST((klass), GNOME_TYPE_PREFSWIN, PrefsWinClass))
#define GNOME_IS_PREFSWIN(obj) (GTK_CHECK_TYPE((obj), GNOME_TYPE_PREFSWIN))
#define GNOME_IS_PREFSWIN_CLASS(klass) (GTK_CHECK_CLASS_TYPE((klass), GNOME_TYPE_PREFSWIN))

#define PREFS_SELECTION_TYPE_CLIPBOARD			0
#define PREFS_SELECTION_TYPE_PRIMARY			1
#define PREFS_SELECTION_TYPE_SECONDARY			2
#define PREFS_SELECTION_TYPE_CUSTOM				3

#define PREFS_SELECTION_TYPE_CLIPBOARD_TEXT		"The CLIPBOARD (GNOME/GTK)"
#define PREFS_SELECTION_TYPE_PRIMARY_TEXT		"The PRIMARY selection (X)"
#define PREFS_SELECTION_TYPE_SECONDARY_TEXT		"The SECONDARY selection (X)"
#define PREFS_SELECTION_TYPE_CUSTOM_TEXT		"A Custom selection atom"


#define PREFS_TARGET_TYPE_TEXT			0
#define PREFS_TARGET_TYPE_HTML			1
#define PREFS_TARGET_TYPE_TARGETS		2
#define PREFS_TARGET_TYPE_CUSTOM		3
#define PREFS_TARGET_TYPE_RICH			4
#define PREFS_TARGET_TYPE_TEXT_TEXT		"The TEXT target (The normal textselection)"
#define PREFS_TARGET_TYPE_RICH_TEXT		"The text/richtext target (For use with StarOffice/OpenOffice.org)"
#define PREFS_TARGET_TYPE_HTML_TEXT		"The text/html target (For use with browsers like Mozilla)"
#define PREFS_TARGET_TYPE_TARGETS_TEXT	"The TARGETS target (A target to get all available targets)"
#define PREFS_TARGET_TYPE_CUSTOM_TEXT	"A Custom target type (Use the TARGETS target to get a list)"

typedef struct _PrefsWin PrefsWin;
typedef struct _PrefsWinClass PrefsWinClass;


struct _PrefsWin {
  GtkWindow window;

  GtkWidget *prefvbox;
  GtkWidget *prefnbk;
  GtkWidget *genvbox;
  GtkWidget *seltypeframe;
  GtkWidget *seltypetable;
  GtkWidget *seltypelabel;
  GtkWidget *seltypecombo;
  GtkWidget *seltypecentry;
  GtkWidget *seltypecustlabel;
  GtkWidget *seltypecustentry;
  GtkWidget *tartypelabel;
  GtkWidget *tartypecombo;
  GtkWidget *tartypecentry;
  GtkWidget *tartypecustlabel;
  GtkWidget *tartypecustentry;

  GtkWidget *colframe;
  GtkWidget *colvbox;
  GtkWidget *colauto;
  GtkWidget *colbeep;
  GtkWidget *colmaxhbox;
  GtkWidget *colmaxcheck;
  GtkObject *colmaxspinadj;
  GtkWidget *colmaxspin;
  GtkWidget *colmaxlabel;
  GtkWidget *prefnbkgeneral;
  GtkWidget *netwlbl;
  GtkWidget *prefnbknetworking;

  GtkWidget *prefnbkgui;
  GtkWidget *guitvbox;
  GtkWidget *guiframe;
  GtkWidget *guivbox;
  GtkWidget *guishowtoolbar;
  GtkWidget *guishowmenu;
  GtkWidget *guishowtitles;
  GtkWidget *guishowstatus;
  GtkWidget *guishowfrom;
  GtkWidget *guishowtime;
  GtkWidget *guishowtype;
  GtkWidget *guishowdatap;

  GtkWidget *prefnbkapplet;
  GtkWidget *apltvbox;
  GtkWidget *aplframe;
  GtkWidget *aplvbox;
  GtkWidget *aplapplet;

  GtkWidget *miscvbox;
#ifdef DEVELOPMENT
  GtkWidget *devframe;
  GtkWidget *devvbox;
  GtkWidget *devcheck;
#endif

  GtkWidget *advframe;
  GtkWidget *advvbox;
  GtkWidget *advunsel;
  GtkWidget *advfollow;
  GtkWidget *advselnew;
  GtkWidget *advmultisel;

  GtkWidget *prefnbkmisc;
  GtkWidget *prefhsep;
  GtkWidget *prefhbtnbox;
  GtkWidget *ok_button;
  GtkWidget *apply_button;
  GtkWidget *cancel_button;
  GtkTooltips *tooltips;
  GList *seltypecombo_items;
  GList *tartypecombo_items;
  gboolean appletwarn;
  GtkWidget *windowparent;
};

struct _PrefsWinClass {
   GtkWindowClass parent_class;
};


GtkType prefswin_get_type(void);
GtkWidget* prefswin_new (GtkWidget *windowparent);
GtkWidget* prefswin_new_and_loadprefs(GtkWidget *windowparent);


/* prefswinmenu Callbacks */

void prefswin_show(GtkWidget *widget, PrefsWin *pwin);
void prefswin_hide(GtkWidget *widget, PrefsWin *pwin);

void
on_apply_button_clicked
                                       (GtkButton *button,
                                        PrefsWin *pwin);
void
on_ok_button_clicked
                                       (GtkButton *button,
                                        PrefsWin *pwin);
void
on_cancel_button_clicked
                                       (GtkButton *button,
                                        PrefsWin *pwin);

void
on_seltypecentry_changed               (GtkEntry     *entry,
                                        PrefsWin *pwin);

void
on_tartypecentry_changed               (GtkEntry     *entry,
                                        PrefsWin     *pwin);

void
on_showtitlesgroup_changed               (GtkWidget    *toggle,
                                          PrefsWin     *pwin);

void
on_applettoggle_changed                  (GtkWidget    *toggle,
                                          PrefsWin     *pwin);

void
on_colmaxchecktoggle_changed              (GtkWidget    *toggle,
                                          PrefsWin     *pwin);

#ifdef __cplusplus /* cpp compatibility */
}
#endif
