/* * C o p y r i g h t *(also read COPYING)* * * * * * * * * * * * * * * * * * *
 *                                                                             *
 *  Copyright (C) 2002  <Philip Van Hoof>                                      *
 *                                                                             *
 *  This program is free software; you can redistribute it and/or modify       *
 *  it under the terms of the GNU General Public License as published by       *
 *  the Free Software Foundation; either version 2 of the License, or          *
 *  (at your option) any later version.                                        *
 *                                                                             *
 *  This program is distributed in the hope that it will be useful,            *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *
 *  GNU General Public License for more details.                               *
 *                                                                             *
 *  You should have received a copy of the GNU General Public License          *
 *  along with this program; if not, write to the Free Software                *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
 *                                                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
#include <unistd.h>
#include <stdio.h>
#include "textitemwin.h"
#include "mainwin.h"
#include "gnomefilelist.h"
#include "config.h"

static void textitemwin_class_init(TextItemWinClass *klass);
static void textitemwin_init(TextItemWin *twin);
static void textitemwin_destroy(GtkObject *object);

static GtkWindowClass *parent_class = NULL;


void textitemsaveasdata_free (TextItemSaveAsData *sa) {
  /*textitemselection_free (sa->item); - we don't want to free the item itself, right?*/
  g_free (sa->path);
  g_free(sa);
}

void textitemselection_free(TextItemSelection *item) {
  g_free (item->time);
  g_free (item->type);
  g_free (item->from);
  g_free (item->data);
  g_free(item);
}

static GnomeUIInfo txtwinmenu_file_menu_uiinfo[] =
{ /* The filemenu */
#define TXTWINMENU_FILE_SAVETODISK 0
  {
    GNOME_APP_UI_ITEM, N_("_Save to disk..."),
    NULL,
    (gpointer) on_txtwinmenu_file_savetodisk_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE_AS,
    0, (GdkModifierType) 0, NULL
  },
#define TXTWINMENU_FILE_SEPARATOR 1
  GNOMEUIINFO_SEPARATOR,
#define TXTWINMENU_FILE_CLOSE 2
  GNOMEUIINFO_MENU_CLOSE_ITEM (on_txtwinmenu_file_close_activate, NULL),
#define TXTWINMENU_FILE_END 3
  GNOMEUIINFO_END
};

static GnomeUIInfo txtwinmenu_edit_menu_uiinfo[] =
{ /* The edit menu */
#define TXTWINMENU_EDIT_CUT 0
  GNOMEUIINFO_MENU_CUT_ITEM (on_txtwinmenu_edit_cut_activate, NULL),
#define TXTWINMENU_EDIT_COPY 1
  GNOMEUIINFO_MENU_COPY_ITEM (on_txtwinmenu_edit_copy_activate, NULL),
#define TXTWINMENU_EDIT_PASTE 2
  GNOMEUIINFO_MENU_PASTE_ITEM (on_txtwinmenu_edit_paste_activate, NULL),
#define TXTWINMENU_EDIT_SELECTALL 3
  GNOMEUIINFO_MENU_SELECT_ALL_ITEM (on_txtwinmenu_edit_selectall_activate, NULL),
#define TXTWINMENU_EDIT_CLEAR 4
  GNOMEUIINFO_MENU_CLEAR_ITEM (on_txtwinmenu_edit_clear_activate, NULL),
#define TXTWINMENU_EDIT_END 5
  GNOMEUIINFO_END
};

static GnomeUIInfo txtwinmenu_uiinfo[] =
{ /* the menu */
#define TXTWINMENU_FILE 0
  {
    GNOME_APP_UI_SUBTREE, N_("_File"),
    NULL,
    txtwinmenu_file_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
#define TXTWINMENU_EDIT 1
  {
    GNOME_APP_UI_SUBTREE, N_("_Edit"),
    NULL,
    txtwinmenu_edit_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
#define TXTWINMENU_END 2
  GNOMEUIINFO_END
};


GtkType textitemwin_get_type(void)
{ /* textitemwin gettype */
   static GtkType textitemwin_type = 0;
   
   if (!textitemwin_type)
   {
      static const GtkTypeInfo textitemwin_info =
      {	
         "TextItemWin",
         sizeof (TextItemWin),
         sizeof (TextItemWinClass),
         (GtkClassInitFunc) textitemwin_class_init,
         (GtkObjectInitFunc) textitemwin_init,
         NULL,
         NULL,
         (GtkClassInitFunc) NULL,
      };
      
      textitemwin_type = gtk_type_unique (GTK_TYPE_WINDOW, &textitemwin_info);
   }

   return (textitemwin_type);
}

static void textitemwin_class_init(TextItemWinClass *klass)
{ /* textitemwin classinit */
   GtkObjectClass *object_class;
   object_class = (GtkObjectClass*)klass;
   object_class->destroy = textitemwin_destroy;
   parent_class = gtk_type_class(GTK_TYPE_WINDOW);
}

static void textitemwin_init(TextItemWin *twin)
{ /* textitemwin init */

}


static void textitemwin_destroy(GtkObject *object)
{ /* ontextitemwin destroy */
   GtkWidget *twin;
  
   g_return_if_fail(object != NULL);
   g_return_if_fail(GNOME_IS_TEXTITEMWIN(object));
   
   twin = GTK_WIDGET(object);
      
   gtk_widget_destroy(twin);
}


GtkWidget* textitemwin_new (GtkWidget *windowparent) 
{ /* Creates a new textitemwin */
  TextItemWin *twin;
  
  twin = gtk_type_new(GNOME_TYPE_TEXTITEMWIN);
  gtk_signal_connect(GTK_OBJECT(twin), "show", GTK_SIGNAL_FUNC(textitemwin_show), twin);
  gtk_signal_connect(GTK_OBJECT(twin), "hide", GTK_SIGNAL_FUNC(textitemwin_hide), twin);
  twin->windowparent = windowparent;
  txtwinmenu_file_menu_uiinfo[TXTWINMENU_FILE_CLOSE].user_data = twin;
  txtwinmenu_file_menu_uiinfo[TXTWINMENU_FILE_SAVETODISK].user_data = twin;

  txtwinmenu_edit_menu_uiinfo[TXTWINMENU_EDIT_CUT].user_data = twin;
  txtwinmenu_edit_menu_uiinfo[TXTWINMENU_EDIT_COPY].user_data = twin;
  txtwinmenu_edit_menu_uiinfo[TXTWINMENU_EDIT_PASTE].user_data = twin;
  txtwinmenu_edit_menu_uiinfo[TXTWINMENU_EDIT_SELECTALL].user_data = twin;
  txtwinmenu_edit_menu_uiinfo[TXTWINMENU_EDIT_CLEAR].user_data = twin;

  gtk_widget_set_usize (GTK_WIDGET (twin), -2, 400);
  gtk_window_set_title (GTK_WINDOW (twin), _("Editing item"));
  gtk_window_set_policy (GTK_WINDOW (twin), TRUE, TRUE, FALSE);
  twin->menubarvbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (twin), twin->menubarvbox);
  gtk_widget_show(twin->menubarvbox);

  twin->menubardock = gtk_handle_box_new ();
  gtk_widget_show (twin->menubardock);
  gtk_box_pack_start (GTK_BOX (twin->menubarvbox), twin->menubardock, FALSE, FALSE, 0);

  twin->txtwinmenu = gtk_menu_bar_new ();
  gtk_widget_show (twin->txtwinmenu);
  gtk_container_add (GTK_CONTAINER (twin->menubardock), twin->txtwinmenu);
  gnome_app_fill_menu (GTK_MENU_SHELL (twin->txtwinmenu), txtwinmenu_uiinfo,
                       NULL, FALSE, 0);

  twin->scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (twin->scrolledwindow);
  gtk_box_pack_start (GTK_BOX (twin->menubarvbox), twin->scrolledwindow, TRUE, TRUE, 0);
  gtk_widget_set_usize (twin->scrolledwindow, 350, 150);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (twin->scrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

  /* GNOME 2.0 porting issue (GtkText is replaced) */
  twin->itemtext = gtk_text_new (NULL, NULL);
  gtk_widget_show (twin->itemtext);
  gtk_container_add (GTK_CONTAINER (twin->scrolledwindow), twin->itemtext);
  GTK_WIDGET_SET_FLAGS (twin->itemtext, GTK_CAN_DEFAULT);
  gtk_editable_set_editable(GTK_EDITABLE(twin->itemtext), TRUE);
  
  twin->notebookvbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (twin->notebookvbox);
  gtk_box_pack_start (GTK_BOX (twin->menubarvbox), twin->notebookvbox, TRUE, TRUE, 0);
  
  twin->notebook = gtk_notebook_new ();
  gtk_widget_show (twin->notebook);
  gtk_box_pack_start (GTK_BOX (twin->notebookvbox), twin->notebook, TRUE, TRUE, 0);

  twin->searchreplacevbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (twin->searchreplacevbox);
  gtk_container_add (GTK_CONTAINER (twin->notebook), twin->searchreplacevbox);
  
  twin->search_replace_table = gtk_table_new (1, 5, FALSE);
  gtk_widget_show (twin->search_replace_table);
  gtk_box_pack_start (GTK_BOX (twin->searchreplacevbox), twin->search_replace_table, TRUE, TRUE, 0);

  twin->search_replace_label1 = gtk_label_new (_(" s/ "));
  gtk_widget_show (twin->search_replace_label1);
  gtk_table_attach (GTK_TABLE (twin->search_replace_table), twin->search_replace_label1, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 8);
  gtk_misc_set_alignment (GTK_MISC (twin->search_replace_label1), 0, 0.5);

  twin->search_for_entry = gtk_entry_new ();
  gtk_widget_show (twin->search_for_entry);
  gtk_table_attach (GTK_TABLE (twin->search_replace_table), twin->search_for_entry, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 12, 35);
  GTK_WIDGET_SET_FLAGS (twin->search_for_entry, GTK_CAN_DEFAULT);

  twin->search_replace_label2 = gtk_label_new (_(" / "));
  gtk_widget_show (twin->search_replace_label2);
  gtk_table_attach (GTK_TABLE (twin->search_replace_table), twin->search_replace_label2, 2, 3, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 8);
  gtk_misc_set_alignment (GTK_MISC (twin->search_replace_label2), 0, 0.5);

  twin->replace_with_entry = gtk_entry_new ();
  gtk_widget_show (twin->replace_with_entry);
  gtk_table_attach (GTK_TABLE (twin->search_replace_table), twin->replace_with_entry, 3, 4, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 12, 35);
  GTK_WIDGET_SET_FLAGS (twin->replace_with_entry, GTK_CAN_DEFAULT);

  twin->search_replace_label3 = gtk_label_new (_(" /g "));
  gtk_widget_show (twin->search_replace_label3);
  gtk_table_attach (GTK_TABLE (twin->search_replace_table), twin->search_replace_label3, 4, 5, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 8);
  gtk_misc_set_alignment (GTK_MISC (twin->search_replace_label3), 0, 0.5);

  twin->search_replace_ok_button = gnome_stock_button (GNOME_STOCK_BUTTON_OK);
  gtk_widget_show (twin->search_replace_ok_button);
  gtk_box_pack_start (GTK_BOX (twin->searchreplacevbox), twin->search_replace_ok_button, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (twin->search_replace_ok_button, GTK_CAN_DEFAULT);
  twin->tooltips = gtk_tooltips_new();
  gtk_tooltips_set_tip (twin->tooltips, twin->search_replace_ok_button, _("Replace"), NULL);

  twin->search_and_replace_label = gtk_label_new (_("Search and Replace"));
  gtk_widget_show (twin->search_and_replace_label);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (twin->notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (twin->notebook), 0), twin->search_and_replace_label);

  twin->misc_vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (twin->misc_vbox);
  gtk_container_add (GTK_CONTAINER (twin->notebook), twin->misc_vbox);
  
  twin->tblMisc01 = gtk_table_new (5, 4, FALSE);
  gtk_widget_show (twin->tblMisc01);
  
  gtk_box_pack_start (GTK_BOX (twin->misc_vbox), twin->tblMisc01, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (twin->tblMisc01), 4);
  gtk_table_set_row_spacings (GTK_TABLE (twin->tblMisc01), 4);
  gtk_table_set_col_spacings (GTK_TABLE (twin->tblMisc01), 4);  

  twin->btnRemLastChars = gtk_button_new_with_label (_("go"));
  gtk_widget_show (twin->btnRemLastChars);
  gtk_table_attach (GTK_TABLE (twin->tblMisc01), twin->btnRemLastChars, 3, 4, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);

  twin->btnRemFirstChars = gtk_button_new_with_label (_("go"));
  gtk_widget_show (twin->btnRemFirstChars);
  gtk_table_attach (GTK_TABLE (twin->tblMisc01), twin->btnRemFirstChars, 3, 4, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);

  twin->lblTlastChars = gtk_label_new (_("characters from each line "));
  gtk_widget_show (twin->lblTlastChars);
  gtk_table_attach (GTK_TABLE (twin->tblMisc01), twin->lblTlastChars, 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (twin->lblTlastChars), 0, 0.5);

  twin->lblTfirstChars = gtk_label_new (_("characters from each line "));
  gtk_widget_show (twin->lblTfirstChars);
  gtk_table_attach (GTK_TABLE (twin->tblMisc01), twin->lblTfirstChars, 2, 3, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (twin->lblTfirstChars), 0, 0.5);

  twin->lblRFirstChars = gtk_label_new (_("Remove first "));     
  gtk_widget_show (twin->lblRFirstChars); 
  gtk_table_attach (GTK_TABLE (twin->tblMisc01), twin->lblRFirstChars, 0, 1, 0, 1, 
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (twin->lblRFirstChars), 0, 0.5);

  twin->lblRLastChars = gtk_label_new (_("Remove last "));
  gtk_widget_show (twin->lblRLastChars);
  gtk_table_attach (GTK_TABLE (twin->tblMisc01), twin->lblRLastChars, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);  
  gtk_misc_set_alignment (GTK_MISC (twin->lblRLastChars), 0, 0.5);

  twin->sbtnLastChars_adj = gtk_adjustment_new (1, 0, 99999, 1, 10, 10);
  twin->sbtnLastChars = gtk_spin_button_new (GTK_ADJUSTMENT (twin->sbtnLastChars_adj), 1, 0);
  
  gtk_widget_show (twin->sbtnLastChars);
  gtk_table_attach (GTK_TABLE (twin->tblMisc01), twin->sbtnLastChars, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);

  twin->sbtnFirstChars_adj = gtk_adjustment_new (1, 0, 99999, 1, 10, 10);
  twin->sbtnFirstChars = gtk_spin_button_new (GTK_ADJUSTMENT (twin->sbtnFirstChars_adj), 1, 0);
  gtk_widget_show (twin->sbtnFirstChars);
  gtk_table_attach (GTK_TABLE (twin->tblMisc01), twin->sbtnFirstChars, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);

  twin->btnRemFirstLines = gtk_button_new_with_label (_("go"));
  gtk_widget_show (twin->btnRemFirstLines);
  gtk_table_attach (GTK_TABLE (twin->tblMisc01), twin->btnRemFirstLines, 3, 4, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);

  twin->btnRemLastLines = gtk_button_new_with_label (_("go"));

  gtk_widget_show (twin->btnRemLastLines);
  gtk_table_attach (GTK_TABLE (twin->tblMisc01), twin->btnRemLastLines, 3, 4, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);

  twin->lblTFirstLines = gtk_label_new (_("lines"));
  gtk_widget_show (twin->lblTFirstLines);
  gtk_table_attach (GTK_TABLE (twin->tblMisc01), twin->lblTFirstLines, 2, 3, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);  
  gtk_misc_set_alignment (GTK_MISC (twin->lblTFirstLines), 0, 0.5);

  twin->lblTLastLines = gtk_label_new (_("lines"));
  gtk_widget_show (twin->lblTLastLines);
  gtk_table_attach (GTK_TABLE (twin->tblMisc01), twin->lblTLastLines, 2, 3, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);  
  gtk_misc_set_alignment (GTK_MISC (twin->lblTLastLines), 0, 0.5);

  twin->sbtnFirstLines_adj = gtk_adjustment_new (1, 0, 99999, 1, 10, 10);
  twin->sbtnFirstLines = gtk_spin_button_new (GTK_ADJUSTMENT (twin->sbtnFirstLines_adj), 1, 0);
  gtk_widget_show (twin->sbtnFirstLines);
  gtk_table_attach (GTK_TABLE (twin->tblMisc01), twin->sbtnFirstLines, 1, 2, 2, 3,
                    (GtkAttachOptions) (0), 
                    (GtkAttachOptions) (0), 0, 0);

  twin->sbtnLastLines_adj = gtk_adjustment_new (1, 0, 99999, 1, 10, 10);
  twin->sbtnLastLines = gtk_spin_button_new (GTK_ADJUSTMENT (twin->sbtnLastLines_adj), 1, 0);

  gtk_widget_show (twin->sbtnLastLines);
  gtk_table_attach (GTK_TABLE (twin->tblMisc01), twin->sbtnLastLines, 1, 2, 3, 4,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  twin->lblRFirstLines = gtk_label_new (_("Remove first "));
  gtk_widget_show (twin->lblRFirstLines);
  gtk_table_attach (GTK_TABLE (twin->tblMisc01), twin->lblRFirstLines, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (twin->lblRFirstLines), 0, 0.5);

  twin->lblRLastLines = gtk_label_new (_("Remove last "));
  gtk_widget_show (twin->lblRLastLines);
  gtk_table_attach (GTK_TABLE (twin->tblMisc01), twin->lblRLastLines, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (twin->lblRLastLines), 0, 0.5);

  twin->btnInvert = gtk_button_new_with_label (_("go"));
  gtk_widget_show (twin->btnInvert);
  gtk_table_attach (GTK_TABLE (twin->tblMisc01), twin->btnInvert, 3, 4, 4, 5,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);

  twin->lblRev = gtk_label_new (_("Invert text"));

  gtk_widget_show (twin->lblRev);
  gtk_table_attach (GTK_TABLE (twin->tblMisc01), twin->lblRev, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (twin->lblRev), 0, 0.5);

  twin->misc_label = gtk_label_new (_("Other functions"));
  gtk_widget_show (twin->misc_label);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (twin->notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (twin->notebook), 1), twin->misc_label);


  twin->hbtnbox = gtk_hbutton_box_new ();
  gtk_widget_show (twin->hbtnbox);
  gtk_box_pack_end (GTK_BOX (twin->menubarvbox), twin->hbtnbox, FALSE, FALSE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (twin->hbtnbox), GTK_BUTTONBOX_END);

  twin->apply_button = gnome_stock_button (GNOME_STOCK_BUTTON_APPLY);
  gtk_widget_show (twin->apply_button);
  gtk_container_add (GTK_CONTAINER (twin->hbtnbox), twin->apply_button);
  GTK_WIDGET_SET_FLAGS (twin->apply_button, GTK_CAN_DEFAULT);

  twin->cancel_button = gnome_stock_button (GNOME_STOCK_BUTTON_CANCEL);
  gtk_widget_show (twin->cancel_button);
  gtk_container_add (GTK_CONTAINER (twin->hbtnbox), twin->cancel_button);
  GTK_WIDGET_SET_FLAGS (twin->cancel_button, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (twin->cancel_button), "clicked",
                      GTK_SIGNAL_FUNC (on_textitemwin_cancel_button_clicked),
                      twin);
  gtk_signal_connect (GTK_OBJECT (twin->apply_button), "clicked",
                      GTK_SIGNAL_FUNC (on_textitemwin_apply_button_clicked),
                      twin);


  gtk_signal_connect (GTK_OBJECT (twin->search_replace_ok_button), "clicked",
                      GTK_SIGNAL_FUNC (on_search_replace_ok_button_clicked),
                      twin);

  gtk_signal_connect (GTK_OBJECT (twin->itemtext), "changed",
                      GTK_SIGNAL_FUNC (on_itemtext_change),
                      twin);

  gtk_signal_connect (GTK_OBJECT (twin->btnRemLastChars), "clicked",
                      GTK_SIGNAL_FUNC (on_btnRemLastChars_clicked),
                      twin);
  gtk_signal_connect (GTK_OBJECT (twin->btnRemFirstChars), "clicked",
                      GTK_SIGNAL_FUNC (on_btnRemFirstChars_clicked),
                      twin);
  gtk_signal_connect (GTK_OBJECT (twin->btnRemFirstLines), "clicked",
                      GTK_SIGNAL_FUNC (on_btnRemFirstLines_clicked),
                      twin);
  gtk_signal_connect (GTK_OBJECT (twin->btnRemLastLines), "clicked",
                      GTK_SIGNAL_FUNC (on_btnRemLastLines_clicked),
                      twin);
  gtk_signal_connect (GTK_OBJECT (twin->btnInvert), "clicked",
                      GTK_SIGNAL_FUNC (on_btnInvert_clicked),
                      twin);

  gtk_widget_grab_focus (twin->itemtext);
  gtk_widget_grab_default (twin->apply_button);

  gtk_object_set_data (GTK_OBJECT (twin), "tooltips", twin->tooltips);
  gtk_window_set_wmclass (GTK_WINDOW (twin), "textitemwin", "gcm");
  return GTK_WIDGET(twin);
}

gchar* search_replace(gchar *buffer, gchar *search, gchar *replace)
{ /* Search/Replace function (uses sed as backend) */
	if ((strlen(buffer) > 0) && (strlen(search) > 0)) {
		gint wp[2], rp[2], status;
		gchar regexp_str[256];	/* For the regexp string, must be enough even for very sophistcated */
		gchar *bufout;
		unsigned int i, subst_size = strlen(buffer) * 2;
		gint growth_step = 64;
	
		bufout = (gchar*) g_malloc0(subst_size);
		if (pipe(wp) == -1)
			return (gchar *) -1;
		if (pipe(rp) == -1)
			return (gchar *) -1;
	
		if (write(wp[1], buffer, strlen(buffer)) == -1)
			return (gchar *) -1;
		close(wp[1]);	/* Send eof to the pipe */
	
		if (fork() == 0) {
			if (dup2(wp[0], 0) == -1)	/* piping STDIN from wp PIPE */
				exit(1);
			if (dup2(rp[1], 1) == -1) 	/* pipng STDOUT to the rp PIPE */
				exit(1);
			sprintf(regexp_str, "s/%s/%s/g", search, replace);
			execl(SEDPATH, "sed", regexp_str, (char *)0);
		}
		wait(&status);
		close(rp[1]);
	
		for(i = 0;read(rp[0], &bufout[i], sizeof(char)) > 0;) {
			if (++i >= subst_size) {
				subst_size += growth_step;
				realloc(bufout, subst_size);
			}
		}
		bufout[i] = '\0';
		return bufout;
	} else return buffer;
}

gboolean textitemwin_save_item_to_disk(gchar *path, TextItemSelection *item)
{ /* Save a TextItemSelection to disk */
	/*Idea: Save to disk using XML*/
	FILE *f;
	f=fopen(path, "w");
	if (f) {
		if (fputs(item->data, f)) {
			fclose(f);
			return TRUE;
		} else {
			fclose(f);
			return FALSE;
		}
	} else {
		return FALSE;
	}
}

TextItemSelection* textitemwin_load_item_from_disk(gchar *path)
{ /* Load a textitemwin from disk (unfinished) */
	TextItemSelection *item=NULL;
	/*Unfinished. And no plans (yet). Load from disk as XML */
	return item;
}

void textitemwin_load_item(GtkWidget *textitemwin, gint row, gchar *time, gchar *type, gchar *from, gchar *data)
{ /* Loads an item */
	TextItemWin *twin = GNOME_TEXTITEMWIN(textitemwin);
	
	twin->item = g_malloc0(sizeof (TextItemSelection));
		twin->item->row = row;
		twin->item->time = g_strdup(time);
		twin->item->type = g_strdup(type);
		twin->item->from = g_strdup(from);
		twin->item->data = g_strdup(data);
		twin->item->changed = FALSE;
	
	/* GNOME 2.0 porting issue (GtkText is replaced) */
	gtk_text_insert (GTK_TEXT(twin->itemtext), NULL, NULL, NULL, data, strlen(data));
}


void textitemwin_close_question (gint reply, TextItemWin *twin) 
{
	if (reply == 0) {
		textitemselection_free(twin->item);
		gtk_widget_destroy(GTK_WIDGET(twin));
	}
}
void textitemwin_cancel_and_close (TextItemWin *twin, gpointer *user_data)
{ /* Cancel and close ... */
	if (twin->item->changed) {
		gnome_question_dialog (_("The item was modified, are you sure you want to close this window?"),
			GTK_SIGNAL_FUNC (textitemwin_close_question), twin);
	} else {
		textitemselection_free(twin->item);
		gtk_widget_destroy(GTK_WIDGET(twin));
	}
}

void texitemwin_apply(TextItemWin *twin, gpointer *user_data)
{ /* Apply's the item to the mainwin's shelf */
	
	twin->item->data = g_strdup(gtk_editable_get_chars (GTK_EDITABLE(twin->itemtext), 0, -1));

	app_update_text_selection (GNOME_MAINWIN(twin->windowparent), 
								twin->item->row, 
								twin->item->type,
								twin->item->from,
								twin->item->data);
	
}


/* TextManipulation functions */
void revstr(gchar *input_buf, gint buflen, gchar *output_buf) 
{ /* invert/revert string */
	register gint i;

	for(i = 0; i < buflen-1; i++)
		*(output_buf+i) = *(input_buf+buflen-i-2);
}


void remove_first_x_lines(gchar *s1, gint x)
{ /* Remove first x lines */
	gint i = 0, j, len = strlen(s1);
	
	while (x-- && i < len) {
		while (s1[i] != '\n' && i < len)
			i++;
		i++;
	}
	
	for (j = 0; i < len; i++, j++)
		s1[j] = s1[i];
	
	s1[j] = '\0';
}

void remove_last_x_lines(gchar *s1, gint x)
{ /* Remove last x lines */
	gint i, len = strlen(s1);
	
	i = len - 1;
	while (x-- && i >= 0) {
		while (s1[i] != '\n' && i > 0)
			i--;
		i--;
	}
	
	s1[i + 1] = '\0';
	
}

void remove_first_x_chars_from_each_line(gchar *s1, gint x) 
{ /* Remove first x chars from each line */
	gint i, j, trigger;
	
	for (i = 0, j = 0, trigger = 0; i < strlen(s1); i++, trigger++) {
	
		if (s1[i] == '\n') {
			s1[j++] = s1[i];
			trigger = 0;
		}
		
		if (trigger > x) {
			s1[j++] = s1[i];
		}
			
	}
	
	s1[j] = '\0';
}

void remove_last_x_chars_from_each_line(gchar *s1, gint x) 
{ /* Remove last x chars from each line */
	gint i, j, back, erase, len = strlen(s1);
	
	for (i = 0, j = 0, back = 0; i < len; i++, j++) {
		
		erase = back > x ? x : back;
		if (s1[i] == '\n' || i == len -1) {
			if (i == len - 1 && s1[i] != '\n')
				s1[i] = '\0';
			else
				j -= erase;
			back = 0;
		}
		
		s1[j] = s1[i];
		
		if (s1[i] != '\n')
			back++;
		
	}
	
	s1[j] = '\0';
}


/*txtwinmenu Callbacks */


void textitemwin_show(GtkWidget *widget, TextItemWin *twin)
{ /* on textitemwin show */
}
void textitemwin_hide(GtkWidget *widget, TextItemWin *twin)
{
	textitemwin_cancel_and_close (twin, NULL);
	
}




void on_textitemwin_overwrite_yes_no (gint answer, TextItemSaveAsData *sa)
{
	if (answer == 0)
		if (!textitemwin_save_item_to_disk(sa->path, sa->item)) {
			gnome_error_dialog(_("IO error while trying to save file"));
		}
		textitemsaveasdata_free(sa);
}



void
on_textitemwin_apply_button_clicked
                                       (GtkButton *button,
                                        TextItemWin *twin)
{
	texitemwin_apply (twin, (gpointer*) button);
	gtk_widget_destroy (GTK_WIDGET(twin));
}


void
on_textitemwin_cancel_button_clicked
                                       (GtkButton *button,
                                        TextItemWin *twin)
{
	textitemwin_cancel_and_close (twin, (gpointer*) button);
}


void
on_txtwinmenu_filelist_ok_button_clicked
                                       (GtkButton *button,
                                        TextItemWin *twin)
{
	gchar *msg;
	GtkWidget *question;
	FILE *f;
	TextItemSaveAsData *sa;

	sa = g_malloc0(sizeof(TextItemSaveAsData));
	sa->path = g_strdup(gnome_filelist_get_filename(GNOME_FILELIST(twin->filelist)));
	sa->item = twin->item;
	gtk_widget_destroy(twin->filelist);
	if (f=fopen(sa->path,"r")) {
		fclose(f);
		msg = g_strdup_printf("A file named ''%s'' already exists.\n"
							  "Do you want to replace it with the "
							  "one you are saving?", sa->path);
		question = gnome_question_dialog_modal
				(_(msg), GTK_SIGNAL_FUNC (on_textitemwin_overwrite_yes_no), sa);
		gtk_widget_show(question);
	} else {
		if (!textitemwin_save_item_to_disk(sa->path, sa->item)) {
			gnome_error_dialog(_("IO error while trying to save file"));
		}
		textitemsaveasdata_free(sa);
	}
	gtk_widget_set_sensitive (GTK_WIDGET (txtwinmenu_file_menu_uiinfo[TXTWINMENU_FILE_SAVETODISK].widget), TRUE);
	twin->filelist_inuse=FALSE;
}

void
on_txtwinmenu_filelist_cancel_button_clicked
                                       (GtkButton *button,
                                        TextItemWin *twin)
{
	twin->filelist_inuse=FALSE;
	gtk_widget_set_sensitive (GTK_WIDGET (txtwinmenu_file_menu_uiinfo[TXTWINMENU_FILE_SAVETODISK].widget), TRUE);
	gtk_widget_destroy(twin->filelist);
}



void
on_txtwinmenu_file_savetodisk_activate (GtkMenuItem     *menuitem,
                                        TextItemWin         *twin)
{
	twin->item->data = g_strdup(gtk_editable_get_chars (GTK_EDITABLE(twin->itemtext), 0, -1));
	gtk_widget_set_sensitive (GTK_WIDGET (txtwinmenu_file_menu_uiinfo[TXTWINMENU_FILE_SAVETODISK].widget), FALSE);
	twin->filelist_inuse=TRUE;
	twin->filelist = gnome_filelist_new();
	gtk_widget_show(twin->filelist);
	twin->filelist_ok_button = GNOME_FILELIST(twin->filelist)->ok_button;
	twin->filelist_cancel_button = GNOME_FILELIST(twin->filelist)->cancel_button;

	gtk_signal_connect (GTK_OBJECT (twin->filelist), "destroy",
                      GTK_SIGNAL_FUNC (on_txtwinmenu_filelist_cancel_button_clicked),
                      twin);
	
	gtk_signal_connect (GTK_OBJECT (twin->filelist_ok_button), "clicked",
                      GTK_SIGNAL_FUNC (on_txtwinmenu_filelist_ok_button_clicked),
                      twin);

	gtk_signal_connect (GTK_OBJECT (twin->filelist_cancel_button), "clicked",
                      GTK_SIGNAL_FUNC (on_txtwinmenu_filelist_cancel_button_clicked),
                      twin);

}


void
on_txtwinmenu_file_close_activate      (GtkMenuItem     *menuitem,
                                        TextItemWin         *twin)
{
	textitemwin_cancel_and_close (twin, (gpointer*) menuitem);
}


void
on_txtwinmenu_edit_cut_activate        (GtkMenuItem     *menuitem,
                                        TextItemWin         *twin)
{
	gtk_editable_cut_clipboard (GTK_EDITABLE(twin->itemtext));
}


void
on_txtwinmenu_edit_copy_activate       (GtkMenuItem     *menuitem,
                                        TextItemWin         *twin)
{
	gtk_editable_copy_clipboard (GTK_EDITABLE(twin->itemtext));
}


void
on_txtwinmenu_edit_paste_activate      (GtkMenuItem     *menuitem,
                                        TextItemWin         *twin)
{
	gtk_editable_paste_clipboard (GTK_EDITABLE(twin->itemtext));
}


void
on_txtwinmenu_edit_selectall_activate  (GtkMenuItem     *menuitem,
                                        TextItemWin         *twin)
{
	gtk_editable_select_region(GTK_EDITABLE(twin->itemtext), 0, -1);
}


void
on_txtwinmenu_edit_clear_activate      (GtkMenuItem     *menuitem,
                                        TextItemWin         *twin)
{
	gtk_editable_delete_text(GTK_EDITABLE(twin->itemtext),0, -1);
}


void
on_search_replace_ok_button_clicked    (GtkButton       *button,
                                        TextItemWin         *twin)
{
	gchar *search, *replace, *buffer, *textout;
	
	buffer = g_strdup(gtk_editable_get_chars (GTK_EDITABLE(twin->itemtext), 0, -1));
	search = g_strdup(gtk_editable_get_chars (GTK_EDITABLE(twin->search_for_entry), 0, -1));
	replace = g_strdup(gtk_editable_get_chars (GTK_EDITABLE(twin->replace_with_entry), 0, -1));
	if ((textout = search_replace(buffer, search, replace)) != (gchar *) -1) {
		gtk_editable_delete_text (GTK_EDITABLE(twin->itemtext), 0, -1);
		gtk_text_insert (GTK_TEXT(twin->itemtext), NULL, NULL, NULL, textout, strlen(textout));
	}
	
	/* Apply and hide */
	//texitemwin_apply_and_close(twin, (gpointer*) button);
}



void on_itemtext_change                (GtkEditable       *editable,
                                        TextItemWin         *twin)
{
	twin->item->changed = TRUE;

}



void
on_btnRemLastChars_clicked             (GtkButton       *button,
                                        TextItemWin         *twin)
{
	gchar *text;
	gint r;

	text = g_strdup(gtk_editable_get_chars (GTK_EDITABLE(twin->itemtext), 0, -1));
	r = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(twin->sbtnLastChars));
	remove_last_x_chars_from_each_line(text, r);
	gtk_editable_delete_text (GTK_EDITABLE(twin->itemtext), 0, -1);
	gtk_text_insert (GTK_TEXT(twin->itemtext), NULL, NULL, NULL, text, strlen(text));
	g_free(text);
}

void
on_btnRemFirstChars_clicked            (GtkButton       *button,
                                        TextItemWin         *twin)
{
	gchar *text;
	gint r;

	text = g_strdup(gtk_editable_get_chars (GTK_EDITABLE(twin->itemtext), 0, -1));
	r = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(twin->sbtnFirstChars));
	remove_first_x_chars_from_each_line(text, r);
	gtk_editable_delete_text (GTK_EDITABLE(twin->itemtext), 0, -1);
	gtk_text_insert (GTK_TEXT(twin->itemtext), NULL, NULL, NULL, text, strlen(text));
	g_free(text);
}

void
on_btnRemFirstLines_clicked            (GtkButton       *button,
                                        TextItemWin         *twin)
{
	gchar *text;
	gint r;

	text = g_strdup(gtk_editable_get_chars (GTK_EDITABLE(twin->itemtext), 0, -1));
	r = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(twin->sbtnFirstLines));
	remove_first_x_lines(text, r);
	gtk_editable_delete_text (GTK_EDITABLE(twin->itemtext), 0, -1);
	gtk_text_insert (GTK_TEXT(twin->itemtext), NULL, NULL, NULL, text, strlen(text));
	g_free(text);
}

void
on_btnRemLastLines_clicked             (GtkButton       *button,
                                        TextItemWin         *twin)
{
	gchar *text;
	gint r;

	text = g_strdup(gtk_editable_get_chars (GTK_EDITABLE(twin->itemtext), 0, -1));
	r = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(twin->sbtnLastLines));
	remove_last_x_lines(text, r);
	gtk_editable_delete_text (GTK_EDITABLE(twin->itemtext), 0, -1);
	gtk_text_insert (GTK_TEXT(twin->itemtext), NULL, NULL, NULL, text, strlen(text));
	g_free(text);
}


void
on_btnInvert_clicked                   (GtkButton       *button,
                                        TextItemWin         *twin)
{
	gchar *text, *textout;
	gint len;

	text = g_strdup(gtk_editable_get_chars (GTK_EDITABLE(twin->itemtext), 0, -1));
	len = strlen(text);
	if (len > 1) {
		textout = (gchar*) g_malloc0(sizeof(gchar) * len);
		revstr(text, len+1, textout);
		gtk_editable_delete_text (GTK_EDITABLE(twin->itemtext), 0, -1);
		gtk_text_insert (GTK_TEXT(twin->itemtext), NULL, NULL, NULL, textout, strlen(textout));
		g_free(text);
		g_free(textout);
	}
}
