/* * C o p y r i g h t *(also read COPYING)* * * * * * * * * * * * * * * * * * *
 *                                                                             *
 *  Copyright (C) 2002  <Philip Van Hoof>                                      *
 *                                                                             *
 *  This program is free software; you can redistribute it and/or modify       *
 *  it under the terms of the GNU General Public License as published by       *
 *  the Free Software Foundation; either version 2 of the License, or          *
 *  (at your option) any later version.                                        *
 *                                                                             *
 *  This program is distributed in the hope that it will be useful,            *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *
 *  GNU General Public License for more details.                               *
 *                                                                             *
 *  You should have received a copy of the GNU General Public License          *
 *  along with this program; if not, write to the Free Software                *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
 *                                                                             *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <regex.h>
#include <strings.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>
#ifdef HAVE_CONFIG_H
#  include "../config.h"
#endif

#include <gnome.h>
#ifdef __cplusplus
extern "C" {
#endif

#define GNOME_TYPE_TEXTITEMWIN (textitemwin_get_type())
#define GNOME_TEXTITEMWIN(obj) (GTK_CHECK_CAST((obj), GNOME_TYPE_TEXTITEMWIN, TextItemWin))
#define GNOME_TEXTITEMWIN_CLASS(klass) (GTK_CHECK_CLASS_CAST((klass), GNOME_TYPE_TEXTITEMWIN, TextItemWinClass))
#define GNOME_IS_TEXTITEMWIN(obj) (GTK_CHECK_TYPE((obj), GNOME_TYPE_TEXTITEMWIN))
#define GNOME_IS_TEXTITEMWIN_CLASS(klass) (GTK_CHECK_CLASS_TYPE((klass), GNOME_TYPE_TEXTITEMWIN))

typedef struct _TextItemWin TextItemWin;
typedef struct _TextItemWinClass TextItemWinClass;
typedef struct _TextItemSelection TextItemSelection;
typedef struct _TextItemSaveAsData TextItemSaveAsData;
	
struct _TextItemSaveAsData {
	TextItemSelection *item;
	gchar *path;
};

struct _TextItemSelection {
  gchar *time;
  gchar *type;
  gchar *from;
  gchar *data;
  gint row;
  gboolean changed;
};

struct _TextItemWin {
  GtkWindow window;

  GtkWidget *menubarvbox;
  GtkWidget *menubardock;
  GtkWidget *txtwinmenu;
  GtkWidget *scrolledwindow;
  GtkWidget *itemtext;
  GtkWidget *notebookvbox;
  GtkWidget *notebook;
  GtkWidget *searchreplacevbox;
  GtkWidget *search_replace_table;
  GtkWidget *search_replace_label1;
  GtkWidget *search_replace_label2;
  GtkWidget *search_replace_label3;
  GtkWidget *replace_with_label;
  GtkWidget *search_for_entry;
  GtkWidget *replace_with_entry;
  GtkWidget *search_and_replace_label;
  GtkWidget *search_replace_ok_button;
  GtkWidget *misc_vbox;
  GtkWidget *tblMisc01;
  GtkWidget *btnRemLastChars;
  GtkWidget *btnRemFirstChars;
  GtkWidget *lblTlastChars;
  GtkWidget *lblTfirstChars;
  GtkWidget *lblRFirstChars;
  GtkWidget *lblRLastChars;
  GtkObject *sbtnLastChars_adj;
  GtkWidget *sbtnLastChars;
  GtkObject *sbtnFirstChars_adj;
  GtkWidget *sbtnFirstChars;
  GtkWidget *btnRemFirstLines;
  GtkWidget *btnRemLastLines;
  GtkWidget *lblTFirstLines;
  GtkWidget *lblTLastLines;
  GtkObject *sbtnFirstLines_adj;
  GtkWidget *sbtnFirstLines;
  GtkObject *sbtnLastLines_adj;
  GtkWidget *sbtnLastLines;
  GtkWidget *lblRFirstLines;
  GtkWidget *lblRLastLines;
  GtkWidget *btnInvert;
  GtkWidget *lblRev;

  GtkWidget *hbtnbox;
  GtkWidget *apply_button;
  GtkWidget *cancel_button;

  GtkWidget *misc_label;
  GtkTooltips *tooltips;


  GtkWidget *filelist;
  GtkWidget *filelist_ok_button;
  GtkWidget *filelist_cancel_button;
  gboolean filelist_inuse;
 
  GtkWidget *windowparent;
  TextItemSelection *item;
};

struct _TextItemWinClass {
   GtkWindowClass parent_class;
};

void textitemsaveasdata_free (TextItemSaveAsData *sa);
void textitemselection_free(TextItemSelection *item);

GtkType textitemwin_get_type(void);
GtkWidget* textitemwin_new (GtkWidget *windowparent);
void textitemwin_load_item(GtkWidget *textitemwin, gint row, gchar *time, gchar *type, gchar *from, gchar *data);
void texitemwin_apply(TextItemWin *twin, gpointer *user_data);
void textitemwin_cancel_and_close (TextItemWin *twin, gpointer *user_data);
gboolean textitemwin_save_item_to_disk(gchar *path, TextItemSelection *item);
TextItemSelection* textitemwin_load_item_from_disk(gchar *path);
/* Textmanipulation functions */
gchar* search_replace(gchar *buffer, gchar *search, gchar *replace);
void revstr(gchar *input_buf, gint buflen, gchar *output_buf);
void remove_first_x_chars_from_each_line(gchar *s1, gint x);
void remove_last_x_chars_from_each_line(gchar *s1, gint x);
void remove_first_x_lines(gchar *s1, gint x);
void remove_last_x_lines(gchar *s1, gint x);
void textitemwin_close_question (gint reply, TextItemWin *twin) ;
/* txtwinmenu Callbacks */

void textitemwin_show(GtkWidget *widget, TextItemWin *twin);

void textitemwin_hide(GtkWidget *widget, TextItemWin *twin);

void
on_txtwinmenu_filelist_ok_button_clicked
                                       (GtkButton *button,
                                        TextItemWin *twin);

void
on_txtwinmenu_filelist_cancel_button_clicked
                                       (GtkButton *button,
                                        TextItemWin *twin);


void
on_textitemwin_apply_button_clicked
                                       (GtkButton *button,
                                        TextItemWin *twin);
void
on_textitemwin_ok_button_clicked
                                       (GtkButton *button,
                                        TextItemWin *twin);
void
on_textitemwin_cancel_button_clicked
                                       (GtkButton *button,
                                        TextItemWin *twin);

void
on_textitemwin_overwrite_yes_no        (gint answer,
                                        TextItemSaveAsData *sa);

void
on_txtwinmenu_file_savetodisk_activate (GtkMenuItem     *menuitem,
                                        TextItemWin         *twin);

void
on_txtwinmenu_file_close_activate      (GtkMenuItem     *menuitem,
                                        TextItemWin         *twin);

void
on_txtwinmenu_edit_cut_activate        (GtkMenuItem     *menuitem,
                                        TextItemWin         *twin);

void
on_txtwinmenu_edit_copy_activate       (GtkMenuItem     *menuitem,
                                        TextItemWin         *twin);

void
on_txtwinmenu_edit_paste_activate      (GtkMenuItem     *menuitem,
                                        TextItemWin         *twin);

void
on_txtwinmenu_edit_selectall_activate  (GtkMenuItem     *menuitem,
                                        TextItemWin         *twin);

void
on_txtwinmenu_edit_clear_activate      (GtkMenuItem     *menuitem,
                                        TextItemWin         *twin);
void
on_itemtext_change                    (GtkEditable       *editable,
                                        TextItemWin         *twin);
void
on_search_replace_ok_button_clicked    (GtkButton       *button,
                                        TextItemWin         *twin);


void
on_regular_expression_ok_button_clicked
                                        (GtkButton       *button,
                                        TextItemWin         *twin);

void
on_regular_expression_cancel_button_clicked
                                        (GtkButton       *button,
                                        TextItemWin         *twin);

void
on_btnRemFirstChars_clicked            (GtkButton        *button,
                                        TextItemWin         *twin);

void
on_btnRemLastChars_clicked            (GtkButton        *button,
                                        TextItemWin         *twin);

void
on_btnRemFirstLines_clicked            (GtkButton       *button,
                                        TextItemWin         *twin);

void
on_btnRemLastLines_clicked             (GtkButton       *button,
                                        TextItemWin         *twin);

void
on_btnInvert_clicked                   (GtkButton       *button,
                                        TextItemWin         *twin);


#ifdef __cplusplus /* cpp compatibility */
}
#endif
